@@ -47,13 +47,18 @@ int a1time::from_asn1(const ASN1_TIME *a)
 		return -1;
 	t = QString::fromLatin1((char*)gt->data, gt->length);
 	ASN1_GENERALIZEDTIME_free(gt);
-	if (t == UNDEFINED_DATE) {
+	return fromPlain(t);
+}
+
+int a1time::fromPlain(QString plain)
+{
+	setTimeSpec(Qt::LocalTime);
+	if (plain == UNDEFINED_DATE)
 		setUndefined();
-		return 0;
-	}
-	*this = QDateTime::fromString(t, GEN_FORMAT);
+	else
+		*this = fromString(plain, GEN_FORMAT);
 	setTimeSpec(Qt::UTC);
-	return 0;
+	return isValid() ? 0 : -1;
 }
 
 int a1time::set_asn1(QString str, int type)
@@ -101,6 +106,12 @@ a1time::a1time(const ASN1_TIME *a)
 	from_asn1(a);
 }
 
+a1time::a1time(QString plain)
+{
+	atime = NULL;
+	fromPlain(plain);
+}
+
 a1time::~a1time()
 {
 	if (atime)
