@@ -26,9 +26,11 @@ class a1time : public QDateTime
 	a1time(const QDateTime &a);
 	a1time(const ASN1_TIME *a);
 	a1time(const a1time &a);
+	a1time(QString plain);
 	a1time &operator = (const a1time &a);
 	~a1time();
 	a1time &set(const ASN1_TIME *a);
+	int fromPlain(QString plain);
 	void setUndefined();
 	bool isUndefined() const;
 	QString toPretty() const;
