@@ -16,12 +16,17 @@
 #include <QMimeData>
 #include "widgets/MainWindow.h"
 #include "widgets/ImportMulti.h"
+#include "widgets/XcaDialog.h"
+#include "ui_ItemProperties.h"
 
-db_base::db_base(QString db, MainWindow *mw)
+
+QHash<quint64, pki_base*> db_base::lookup;
+
+db_base::db_base(MainWindow *mw)
 	:QAbstractItemModel(NULL)
 {
-	dbName = db;
 	rootItem = newPKI();
+	rootItem->setIntName(rootItem->getClassName());
 	mainwin = mw;
 	colResizing = 0;
 	currentIdx = QModelIndex();
@@ -34,8 +39,9 @@ db_base::~db_base()
 	delete rootItem;
 }
 
-pki_base *db_base::newPKI(db_header_t *)
+pki_base *db_base::newPKI(enum pki_type type)
 {
+	(void)type;
 	return new pki_base("rootItem");
 }
 
@@ -44,13 +50,13 @@ void db_base::createSuccess(pki_base *pki)
 	if (!pki)
 		return;
 
-	if (pki_base::suppress_messages)
+	if (Settings["suppress_messages"])
 		return;
 
 	XCA_INFO(pki->getMsg(pki_base::msg_create).arg(pki->getIntName()));
 }
 
-void db_base::remFromCont(QModelIndex &idx)
+void db_base::remFromCont(const QModelIndex &idx)
 {
 	if (!idx.isValid())
 		return;
@@ -64,139 +70,70 @@ void db_base::remFromCont(QModelIndex &idx)
 	emit columnsContentChanged();
 }
 
-int db_base::handleBadEntry(unsigned char *p, db_header_t *head)
+QString db_base::sqlItemSelector()
 {
-	QString name = QString::fromUtf8(head->name);
-	QString txt = tr("Bad database item\nName: %1\nType: %2\nSize: %3\n%4")
-		.arg(name).arg(class_name)
-		.arg(head->len - sizeof(db_header_t))
-		.arg(tr("Do you want to delete the item from the database? The bad item may be extracted into a separate file."));
+	QStringList sl;
+	QString selector;
 
-	xcaWarning msg(mainwin, txt);
-	msg.addButton(QMessageBox::Ok)->setText(tr("Delete"));
-	msg.addButton(QMessageBox::Apply)->setText(tr("Delete and extract"));
-	msg.addButton(QMessageBox::Cancel)->setText(tr("Continue"));
+	foreach(enum pki_type pt, pkitype)
+		sl << QString("type=%1").arg(pt);
 
-	switch (msg.exec()) {
-	case QMessageBox::Ok:
-		return 1;
-	case QMessageBox::Cancel:
-	default:
-		return 0;
-	case QMessageBox::Apply:
-		break;
+	return sl.join(" OR ");
+}
+
+XSqlQuery db_base::sqlSELECTpki(QString query, QList<QVariant> values)
+{
+	XSqlQuery q;
+	int i, num_values = values.size();
+
+	SQL_PREPARE(q, query);
+	for (i=0; i < num_values; i++) {
+		q.bindValue(i, values[i]);
 	}
-
-	QString s = QFileDialog::getSaveFileName(mainwin, QString(), name,
-			QString(), NULL, QFileDialog::DontConfirmOverwrite);
-
-	size_t l;
-	db_header_t h;
-	FILE *fp = fopen_write(s);
-	if (!fp) {
-		throw errorEx(tr("Error opening file: '%1': %2").
-                        arg(s).arg(strerror(errno)), class_name);
-	}
-
-	h.magic   = xntohl(head->magic);
-	h.len     = xntohl(head->len);
-	h.headver = xntohs(head->headver);
-	h.type    = xntohs(head->type);
-	h.version = xntohs(head->version);
-	h.flags   = xntohs(head->flags);
-	memcpy(h.name, head->name, NAMELEN);
-
-	l = fwrite(&h, sizeof h, 1, fp);
-	l += fwrite(p, head->len - sizeof h, 1, fp);
-	fclose(fp);
-
-	return (l == 2);
+	q.exec();
+	MainWindow::dbSqlError(q.lastError());
+	return q;
 }
 
 void db_base::loadContainer()
 {
-	db mydb(dbName);
-	unsigned char *p = NULL;
-	db_header_t head;
-	pki_base *pki;
+	XSqlQuery q;
+	QSqlError e;
+	QString stmt;
 
-	for (int i=0; i < pkitype.count(); i++) {
-		mydb.first();
-		while (mydb.find(pkitype[i], QString()) == 0) {
-			QString s;
-			p = mydb.load(&head);
-			if (!p) {
-				qWarning("Load was empty !");
-				goto next;
-			}
-			pki = newPKI(&head);
-			if (pki->getVersion() < head.version) {
-				qWarning("Item[%s]: Version %d "
-					"> known version: %d -> ignored",
-					head.name, head.version,
-					pki->getVersion()
-				);
-				free(p);
-				delete pki;
-				goto next;
-			}
-			pki->setIntName(QString::fromUtf8(head.name));
+	SQL_PREPARE(q, QString("SELECT * FROM view_") + sqlHashTable);
+	q.exec();
+	e = q.lastError();
+	mainwin->dbSqlError(e);
 
-			try {
-				pki->fromData(p, &head);
-			}
-			catch (errorEx &err) {
-				err.appendString(pki->getIntName());
-				mainwin->Error(err);
-				delete pki;
-				pki = NULL;
-				try {
-					if (handleBadEntry(p, &head)) {
-						mydb.erase();
-					}
-				} catch (errorEx &err) {
-					mainwin->Error(err);
-				}
-			}
-			free(p);
-			if (pki) {
-				inToCont(pki);
-			}
-next:
-			if (mydb.next())
-				break;
-		}
+	while (q.next()) {
+		enum pki_type t;
+		QSqlRecord rec = q.record();
+		t = (enum pki_type)q.value(VIEW_item_type).toInt();
+		pki_base *pki = newPKI(t);
+		pki->restoreSql(rec);
+		insertChild(rootItem, pki);
+		lookup[q.value(VIEW_item_id).toULongLong()] = pki;
 	}
 
-	mydb.first();
-	if (!mydb.find(setting, class_name + "_hdView")) {
-		QByteArray ba;
-		char *p;
-		if ((p = (char *)mydb.load(&head))) {
-			ba = QByteArray(p, head.len - sizeof(db_header_t));
-			free(p);
-		}
-		if (head.version != 5)
-			return;
-		try {
-			allHeaders.fromData(ba);
-		}  catch (errorEx()) {
-			for (int i=0; i< allHeaders.count(); i++) {
-				allHeaders[i]->reset();
-			}
+	QString view = Settings[class_name + "_hdView"];
+	if (view.isEmpty()) {
+		for (int i=0; i< allHeaders.count(); i++) {
+			allHeaders[i]->reset();
 		}
+	} else {
+		allHeaders.fromData(view);
 	}
 	emit columnsContentChanged();
-	return;
 }
 
 void db_base::updateHeaders()
 {
-	QByteArray ba = allHeaders.toData();
+	QString s = allHeaders.toData();
 	foreach(dbheader *h, allHeaders)
 		delete h;
 	allHeaders = getHeaders();
-	allHeaders.fromData(ba);
+	allHeaders.fromData(s);
 }
 
 dbheaderList db_base::getHeaders()
@@ -204,19 +141,22 @@ dbheaderList db_base::getHeaders()
 	dbheaderList h;
 	/* "No." handled in XcaProxyModel */
 	h << new dbheader(HD_internal_name, true, tr("Internal name"))
-	  << new dbheader(HD_counter, false, tr("No."));
+	  << new dbheader(HD_counter, false, tr("No."))
+	  << new dbheader(HD_primary_key, false, tr("Primary key"),
+			tr("Database unique number"))
+	  << new dbheader(HD_creation, false, tr("Date"),
+			tr("Date of creation or insertion"))
+	  << new dbheader(HD_source, false, tr("Source"),
+			tr("Generated, Imported, Transformed"))
+	  << new dbheader(HD_comment, false, tr("Comment"),
+			tr("First line of the comment field"));
 	return h;
 }
 
-
 void db_base::saveHeaderState()
 {
-	if (dbName.isEmpty())
-		return;
-	QByteArray ba = allHeaders.toData();
-	db mydb(dbName);
-	mydb.set((const unsigned char *)ba.constData(), ba.size(), 5,
-		setting, class_name + "_hdView");
+	if (QSqlDatabase::database().isOpen())
+		Settings[class_name + "_hdView"] = allHeaders.toData();
 }
 
 void db_base::setVisualIndex(int i, int visualIndex)
@@ -269,17 +209,18 @@ void db_base::sortIndicatorChanged(int logicalIndex, Qt::SortOrder order)
 
 void db_base::insertPKI(pki_base *pki)
 {
-	QString name;
-	db mydb(dbName);
-	QByteArray ba = pki->toData();
-
-	if (ba.count() > 0) {
-		name = mydb.uniq_name(pki->getIntName(), pkitype);
-		pki->setIntName(name);
-		mydb.add((const unsigned char*)ba.constData(), ba.count(),
-			pki->getVersion(), pki->getType(), name);
+	Transaction;
+	if (!TransBegin())
+		return;
+	QSqlError e = pki->insertSql();
+	if (e.isValid()) {
+		mainwin->dbSqlError(e);
+		TransRollback();
+		return;
 	}
+	lookup[pki->getSqlItemId().toULongLong()] = pki;
 	inToCont(pki);
+	TransCommit();
 	emit columnsContentChanged();
 }
 
@@ -319,45 +260,37 @@ void db_base::pem2clipboard(QModelIndexList indexes) const
 void db_base::deletePKI(QModelIndex idx)
 {
 	pki_base *pki = static_cast<pki_base*>(idx.internalPointer());
+	QSqlDatabase db = QSqlDatabase::database();
 	try {
 		try {
 			pki->deleteFromToken();
 		} catch (errorEx &err) {
 			MainWindow::Error(err);
 		}
-
-		remFromCont(idx);
-
-		db mydb(dbName);
-		mydb.find(pki->getType(), pki->getIntName());
-		mydb.erase();
-		delete pki;
+		Transaction;
+		if (TransBegin()) {
+			QSqlError e = pki->deleteSql();
+			TransDone(e);
+			if (!e.isValid())
+				remFromCont(idx);
+			mainwin->dbSqlError(e);
+		}
 	} catch (errorEx &err) {
 		MainWindow::Error(err);
 	}
 }
 
-void db_base::updatePKI(pki_base *pki)
-{
-	db mydb(dbName);
-
-	QByteArray ba = pki->toData();
-
-	if (ba.count() > 0) {
-		mydb.set((const unsigned char*)ba.constData(), ba.count(),
-			pki->getVersion(), pki->getType(), pki->getIntName());
-	}
-}
-
 void db_base::showItem(const QModelIndex &index)
 {
-	showPki(static_cast<pki_base*>(index.internalPointer()));
+	pki_base *pki = static_cast<pki_base*>(index.internalPointer());
+	if (pki->isVisible() == 1)
+		showPki(pki);
 }
 
 void db_base::showItem(const QString name)
 {
-	pki_base *pki = getByName(name);
-	if (pki)
+	pki_base *pki = lookupPki<pki_base>(name.toULongLong());
+	if (pki && pki->isVisible() == 1)
 		showPki(pki);
 }
 
@@ -376,6 +309,10 @@ void db_base::insertChild(pki_base *parent, pki_base *child)
 	endInsertRows();
 }
 
+/* Does all the linking from existing keys, crls, certs
+ * to the new imported or generated item
+ * called before the new item will be inserted into the database
+ */
 void db_base::inToCont(pki_base *pki)
 {
 	insertChild(rootItem, pki);
@@ -383,34 +320,27 @@ void db_base::inToCont(pki_base *pki)
 
 pki_base *db_base::getByName(QString desc)
 {
-	FOR_ALL_pki(pki, pki_base) {
-		if (pki->getIntName() == desc)
-			return pki;
-	}
-	return NULL;
+	QList<pki_base*> list = sqlSELECTpki<pki_base>(
+		QString("SELECT id FROM items WHERE name=? AND ") +
+			sqlItemSelector(),
+		QList<QVariant>() << QVariant(desc));
+	return list.isEmpty() ? NULL : list[0];
 }
 
 pki_base *db_base::getByReference(pki_base *refpki)
 {
 	if (refpki == NULL)
 		return NULL;
-	FOR_ALL_pki(pki, pki_base) {
+	QList<pki_base*> list = sqlSELECTpki<pki_base>(
+		QString("SELECT item FROM %1 WHERE hash=?").arg(sqlHashTable),
+		QList<QVariant>() << QVariant(refpki->hash()));
+	foreach(pki_base *pki, list) {
 		if (refpki->compare(pki))
 			return pki;
 	}
 	return NULL;
 }
 
-QStringList db_base::getDesc()
-{
-	QStringList x;
-	x.clear();
-	FOR_ALL_pki(pki, pki_base) {
-		x.append(pki->getIntName());
-	}
-	return x;
-}
-
 pki_base *db_base::insert(pki_base *item)
 {
 	insertPKI(item);
@@ -444,6 +374,8 @@ QModelIndex db_base::index(int row, int column, const QModelIndex &parent)
 {
 	pki_base *parentItem;
 
+	if(column <0)
+		abort();
 	if (!parent.isValid())
 		parentItem = rootItem;
 	else
@@ -503,7 +435,8 @@ QVariant db_base::data(const QModelIndex &index, int role) const
 	switch (role) {
 		case Qt::EditRole:
 		case Qt::DisplayRole:
-			return item->column_data(hd);
+			if (hd->id == HD_internal_name || item->isVisible() == 1)
+				return item->column_data(hd);
 		case Qt::DecorationRole:
 			return item->getIcon(hd);
 		case Qt::TextAlignmentRole:
@@ -513,7 +446,7 @@ QVariant db_base::data(const QModelIndex &index, int role) const
 		case Qt::BackgroundRole:
 			return item->bg_color(hd);
 		case Qt::UserRole:
-			return item->visible();
+			return item->isVisible();
 	}
 	return QVariant();
 }
@@ -549,14 +482,16 @@ QVariant db_base::headerData(int section, Qt::Orientation orientation,
 Qt::ItemFlags db_base::flags(const QModelIndex &index) const
 {
 	if (!index.isValid())
-		return Qt::ItemIsEnabled;
+		return Qt::NoItemFlags;
 
-	Qt::ItemFlags defaultFlags = QAbstractItemModel::flags(index);
-	defaultFlags |= Qt::ItemIsDragEnabled;
-
-	if (index.column() == 0)
-		return defaultFlags |= Qt::ItemIsEditable;
-	return defaultFlags;
+	Qt::ItemFlags flags = QAbstractItemModel::flags(index) |
+				Qt::ItemIsDragEnabled;
+	pki_base *item = static_cast<pki_base*>(index.internalPointer());
+	if (item->isVisible() == 2)
+		flags &= ~Qt::ItemIsEnabled;
+	else if (index.column() == 0)
+		flags |= Qt::ItemIsEditable;
+	return flags;
 }
 
 bool db_base::setData(const QModelIndex &index, const QVariant &value, int role)
@@ -569,30 +504,72 @@ bool db_base::setData(const QModelIndex &index, const QVariant &value, int role)
 		on = item->getIntName();
 		if (nn == on)
 			return true;
-		db mydb(dbName);
-		try {
-			mydb.rename(item->getType(), on, nn);
-			item->setIntName(nn);
-			emit dataChanged(index, index);
-			return true;
-		} catch (errorEx &err) {
-			mainwin->Error(err);
-		}
+		updateItem(item, nn, item->getComment());
+		return true;
 	}
 	return false;
 }
 
+void db_base::updateItem(pki_base *pki, QString name, QString comment)
+{
+	XSqlQuery q;
+	QSqlError e;
+
+	Transaction;
+	TransThrow();
+
+	SQL_PREPARE(q, "UPDATE items SET name=?, comment=? WHERE id=?");
+	q.bindValue(0, name);
+	q.bindValue(1, comment);
+	q.bindValue(2, pki->getSqlItemId());
+	q.exec();
+	e = q.lastError();
+	AffectedItems(pki->getSqlItemId());
+	mainwin->dbSqlError(e);
+	if (e.isValid())
+		return;
+	TransDone(e);
+	pki->setIntName(name);
+	pki->setComment(comment);
+
+	QModelIndex i, j;
+	i = index(pki);
+	j = index(i.row(), allHeaders.size(), i.parent());
+	emit dataChanged(i, j);
+	emit pkiChanged(pki);
+}
+
+void db_base::editComment(const QModelIndex &index)
+{
+	pki_base *item = static_cast<pki_base*>(index.internalPointer());
+	if (!index.isValid() || !item)
+		return;
+
+	QWidget *w = new QWidget(NULL);
+	Ui::ItemProperties *prop = new Ui::ItemProperties();
+	prop->setupUi(w);
+	prop->comment->setPlainText(item->getComment());
+	prop->name->setText(item->getIntName());
+	prop->source->setText(item->pki_source_name());
+	prop->insertionDate->setText(item->getInsertionDate().toPretty());
+	XcaDialog *d = new XcaDialog(mainwin, item->getType(), w,
+		tr("Item properties"), QString());
+	if (d->exec())
+		updateItem(item, prop->name->text(), prop->comment->toPlainText());
+	delete d;
+}
+
 void db_base::load_default(load_base &load)
 {
 	QString s;
 	QStringList slist = QFileDialog::getOpenFileNames(mainwin, load.caption,
-			mainwin->getPath(), load.filter);
+				Settings["workingdir"], load.filter);
 
 	if (!slist.count())
 		return;
 
 	QString fn = slist[0];
-	mainwin->setPath(fn.mid(0, fn.lastIndexOf("/")));
+	Settings["workingdir"] = fn.mid(0, fn.lastIndexOf("/"));
 
 	ImportMulti *dlgi = new ImportMulti(mainwin);
 	foreach(s, slist) {
@@ -628,14 +605,14 @@ void db_base::store(QModelIndexList indexes)
 		return;
 	}
 
-	QString fn = mainwin->getPath() + QDir::separator() + "export.pem";
+	QString fn = Settings["workingdir"] + QDir::separator() + "export.pem";
 	QString s = QFileDialog::getSaveFileName(mainwin,
 		tr("Save %1 items in one file as").arg(indexes.size()), fn,
 		tr("PEM files ( *.pem );; All files ( * )"));
 	if (s.isEmpty())
 		return;
 	s = nativeSeparator(s);
-	mainwin->setPath(s.mid(0, s.lastIndexOf(QRegExp("[/\\\\]")) ));
+	Settings["workingdir"] = s.mid(0, s.lastIndexOf(QRegExp("[/\\\\]")));
 	try {
 		QString pem = pem2QString(indexes);
 		FILE *fp = fopen_write(s);
@@ -657,7 +634,7 @@ bool db_base::columnHidden(int col) const
 {
 	if (!isValidCol(col))
 		return true;
-	if (pki_x509::disable_netscape &&
+	if (Settings["disable_netscape"] &&
 	    allHeaders[col]->type == dbheader::hd_v3ext_ns)
 		return true;
 	return !allHeaders[col]->show;
