@@ -8,6 +8,7 @@
 #ifndef __DB_BASE_H
 #define __DB_BASE_H
 
+#include <typeinfo>
 #include "db.h"
 #include "base.h"
 #include "load_obj.h"
@@ -16,6 +17,7 @@
 #include <QContextMenuEvent>
 #include <QStringList>
 #include <QAbstractItemModel>
+#include <QHash>
 #include "widgets/ExportDialog.h"
 #include "pki_base.h"
 #include "headerlist.h"
@@ -35,7 +37,7 @@ class db_base: public QAbstractItemModel
 	Q_OBJECT
 
 	protected:
-		QString dbName;
+		static QHash<quint64, pki_base*> lookup;
 		QModelIndex currentIdx;
 		void _writePKI(pki_base *pki, bool overwrite);
 		void _removePKI(pki_base *pki );
@@ -43,32 +45,77 @@ class db_base: public QAbstractItemModel
 		QList<enum pki_type> pkitype;
 		MainWindow *mainwin;
 		QString class_name;
+		/* Sql table containing the 'hash' of this items */
+		QString sqlHashTable;
 		dbheaderList allHeaders;
 		virtual dbheaderList getHeaders();
 		int colResizing;
-		int handleBadEntry(unsigned char *p, db_header_t *head);
+		QString sqlItemSelector();
+		void updateItem(pki_base *pki, QString name, QString comment);
 		virtual exportType::etype clipboardFormat(QModelIndexList) const
 		{
 			return exportType::Separator;
 		}
 		bool isValidCol(int col) const;
+		static XSqlQuery sqlSELECTpki(QString query,
+				QList<QVariant> values = QList<QVariant>());
 
 	public:
+		template <class T> static T *lookupPki(quint64 i)
+		{
+			T *pki = dynamic_cast<T*>(lookup[i]);
+			if (!pki && i > 0) {
+				pki_base *p = lookup[i];
+				QString f = QString("Invalid Type of ItemId(%1) %2 %3."
+						" Expected to be %4.")
+						.arg(i).arg(typeid(p).name())
+						.arg(p?p->getIntName() : "<NULL item>")
+						.arg(typeid(T*).name());
+				qCritical("%s", CCHAR(f));
+			}
+			return pki;
+		}
+		template <class T> static T *lookupPki(QVariant v)
+		{
+			return lookupPki<T>(v.toULongLong());
+		}
+		static void flushLookup()
+		{
+			lookup.clear();
+		}
+		template <class T> static QList<T *>
+				sqlSELECTpki(QString query,
+				QList<QVariant> values = QList<QVariant>())
+		{
+			XSqlQuery q = sqlSELECTpki(query, values);
+			QList<T *> x;
+			while (q.next()) {
+				T *pki = lookupPki<T>(q.value(0));
+				if (pki)
+					x << pki;
+			}
+			return x;
+		}
+
+		virtual pki_base *newPKI(enum pki_type type = none);
 		pki_base *rootItem;
-		db_base(QString db, MainWindow *mw);
+		db_base(MainWindow *mw);
 		virtual void updateHeaders();
 		virtual ~db_base();
-		virtual pki_base *newPKI(db_header_t *head = NULL);
 		virtual void insertPKI(pki_base *pki);
-		virtual void updatePKI(pki_base *pki);
 		pki_base *getByName(QString desc);
 		pki_base *getByReference(pki_base *refpki);
 		pki_base *getByPtr(void *);
 		virtual void loadContainer();
-		QStringList getDesc();
+		template <class T> QList<T *> getAll()
+		{
+			return sqlSELECTpki<T>(
+		                QString("SELECT item FROM %1")
+					.arg(sqlHashTable));
+		}
 		virtual pki_base* insert(pki_base *item);
 		virtual void inToCont(pki_base *pki);
-		virtual void remFromCont(QModelIndex &idx);
+		virtual void remFromCont(const QModelIndex &idx);
 
 		QPixmap *loadImg(const char *name);
 		void writeAll(void);
@@ -89,7 +136,7 @@ class db_base: public QAbstractItemModel
 		void createSuccess(pki_base *pki);
 		bool columnHidden(int col) const;
 		bool isNumericCol(int col) const;
-		void saveHeaderState();
+		virtual void saveHeaderState();
 		void initHeaderView(QHeaderView *hv);
 		void setVisualIndex(int i, int visualIndex);
 		bool fixedHeaderSize(int sect);
@@ -111,7 +158,7 @@ class db_base: public QAbstractItemModel
 
 		void deletePKI(QModelIndex idx);
 		QMimeData *mimeData(const QModelIndexList &indexes) const;
-
+		void editComment(const QModelIndex &index);
 
 	public slots:
 		virtual void newItem() { }
@@ -128,6 +175,7 @@ class db_base: public QAbstractItemModel
 		void resetHeader();
 		void updateHeader();
 		void columnsContentChanged();
+		void pkiChanged(pki_base *pki);
 };
 
 #endif
