@@ -28,16 +28,37 @@
 #include "widgets/KeyDetail.h"
 #include "widgets/NewKey.h"
 
-db_key::db_key(QString db, MainWindow *mw)
-	:db_base(db, mw)
+db_key::db_key(MainWindow *mw)
+	:db_base(mw)
 {
-	rootItem->setIntName("[key root]");
 	class_name = "keys";
+	sqlHashTable = "public_keys";
 	pkitype << asym_key << smartCard;
 	updateHeaders();
 	loadContainer();
 }
 
+void db_key::loadContainer()
+{
+	XSqlQuery q;
+
+	db_base::loadContainer();
+	FOR_ALL_pki(key, pki_key)
+		key->setUcount(0);
+
+	SQL_PREPARE(q, "SELECT pkey, COUNT(*) FROM x509super WHERE pkey IS NOT NULL GROUP by pkey");
+	q.exec();
+	while (q.next()) {
+		pki_key *key = lookupPki<pki_key>(q.value(0));
+		if (!key) {
+			qDebug() << "Unknown key" << q.value(0).toULongLong();
+			continue;
+		}
+		key->setUcount(q.value(1).toInt());
+	}
+	MainWindow::dbSqlError(q.lastError());
+}
+
 dbheaderList db_key::getHeaders()
 {
 	dbheaderList h = db_base::getHeaders();
@@ -51,63 +72,84 @@ dbheaderList db_key::getHeaders()
 	return h;
 }
 
-pki_base *db_key::newPKI(db_header_t *head)
+pki_base *db_key::newPKI(enum pki_type type)
 {
-	if (!head || head->type == asym_key)
+	if (type == asym_key)
 		return new pki_evp("");
 	return new pki_scard("");
 }
 
-
-QStringList db_key::getPrivateDesc()
+QList<pki_key *> db_key::getAllKeys()
 {
-	QStringList x;
-	x.clear();
-	FOR_ALL_pki(pki, pki_key)
-		if (pki->isPrivKey())
-			x.append(pki->getIntName());
-	return x;
+	return sqlSELECTpki<pki_key>("SELECT item from public_keys");
 }
 
-QStringList db_key::get0KeyDesc(bool all)
+QList<pki_key *> db_key::getUnusedKeys()
 {
-	QStringList x;
-	FOR_ALL_pki(pki, pki_key) {
-		if ((pki->getUcount() == 0) || all)
-			x.append(pki->getIntNameWithType());
-	}
-	return x;
+	return sqlSELECTpki<pki_key>("SELECT public_keys.item FROM public_keys "
+		"LEFT OUTER JOIN x509super ON x509super.pkey= public_keys.item "
+		"WHERE x509super.item IS NULL");
 }
 
-void db_key::remFromCont(QModelIndex &idx)
+void db_key::remFromCont(const QModelIndex &idx)
 {
 	db_base::remFromCont(idx);
-	pki_base *pki = static_cast<pki_base*>(idx.internalPointer());
-	emit delKey((pki_key *)pki);
+	XSqlQuery q;
+
+	/* "pkey" column in "x509super" table already updated
+	 * in deleteSql()
+	 */
+	QList<pki_x509super*> items = sqlSELECTpki<pki_x509super>(
+		"SELECT item FROM x509super WHERE pkey is NULL");
+	foreach(pki_x509super *x509s, items) {
+		x509s->setRefKey(NULL);
+	}
+	/* "UPDATE x509super SET pkey=NULL WHERE pkey=?" done in
+	 * pki->deleteSqlData() */
 }
 
 void db_key::inToCont(pki_base *pki)
 {
 	db_base::inToCont(pki);
-	emit newKey((pki_key *)pki);
+	pki_key *key = static_cast<pki_key*>(pki);
+	unsigned hash = key->hash();
+	QList<pki_x509super*> items = sqlSELECTpki<pki_x509super>(
+		"SELECT item FROM x509super WHERE pkey IS NULL AND key_hash=?",
+		QList<QVariant>() << QVariant(hash));
+	XSqlQuery q;
+	SQL_PREPARE(q, "UPDATE x509super SET pkey=? WHERE item=?");
+	q.bindValue(0, key->getSqlItemId());
+	foreach(pki_x509super *x509s, items) {
+		if (!x509s->compareRefKey(key))
+			continue;
+		/* Found item matching this key */
+		x509s->setRefKey(key);
+		q.bindValue(1, x509s->getSqlItemId());
+		AffectedItems(x509s->getSqlItemId());
+		q.exec();
+		mainwin->dbSqlError(q.lastError());
+	}
 }
 
 pki_base* db_key::insert(pki_base *item)
 {
-	pki_key *lkey = (pki_key *)item;
+	pki_key *lkey = static_cast<pki_key *>(item);
 	pki_key *oldkey;
 
-	oldkey = (pki_key *)getByReference(lkey);
+	oldkey = static_cast<pki_key *>(getByReference(lkey));
 	if (oldkey != NULL) {
 		if ((oldkey->isPrivKey() && lkey->isPrivKey()) || lkey->isPubKey()){
 			XCA_INFO(
 			tr("The key is already in the database as:\n'%1'\nand is not going to be imported").arg(oldkey->getIntName()));
 			delete(lkey);
 			return NULL;
-		}
-		else {
+		} else {
 			XCA_INFO(
 			tr("The database already contains the public part of the imported key as\n'%1\nand will be completed by the new, private part of the key").arg(oldkey->getIntName()));
+			lkey->setComment(oldkey->getComment());
+			lkey->selfComment(tr("Extending public key from %1 by imported key '%2'")
+				.arg(oldkey->getInsertionDate().toPretty())
+				.arg(lkey->getIntName()));
 			lkey->setIntName(oldkey->getIntName());
 			deletePKI(index(oldkey->row(), 0, QModelIndex()));
 		}
@@ -135,7 +177,10 @@ void db_key::newItem(QString name)
 		return;
 	}
 	int ksize = dlg->getKeysize();
-	if (ksize > 0) {
+#ifndef OPENSSL_NO_EC
+	if (dlg->getKeytype() != EVP_PKEY_EC)
+#endif
+	{
 		if (ksize < 32) {
 			XCA_WARN(tr("Key size too small !"));
 			delete dlg;
@@ -161,8 +206,9 @@ void db_key::newItem(QString name)
 			nkey->generate(ksize, dlg->getKeytype(), bar,
 				dlg->getKeyCurve_nid());
 		}
+		key->pkiSource = generated;
 		key = (pki_key*)insert(key);
-		emit keyDone(key->getIntNameWithType());
+		emit keyDone(key);
 		createSuccess(key);
 
 	} catch (errorEx &err) {
@@ -172,7 +218,7 @@ void db_key::newItem(QString name)
 	if (dlg->rememberDefault->isChecked()) {
 		QString def = dlg->getAsString();
 		if (dlg->setDefault(def) == 0)
-			mainwin->setDefaultKey(def);
+			Settings["defaultkey"] = def;
 	}
 	status->removeWidget(bar);
 	delete bar;
@@ -187,13 +233,24 @@ void db_key::load(void)
 
 void db_key::showPki(pki_base *pki)
 {
-	pki_evp *key = (pki_evp *)pki;
+	pki_key *key = dynamic_cast<pki_key *>(pki);
+	if (!key)
+		return;
 	KeyDetail *dlg = new KeyDetail(mainwin);
-	if (dlg) {
-		dlg->setKey(key);
-		dlg->exec();
-		delete dlg;
+	if (!dlg)
+		return;
+	dlg->setKey(key);
+
+	if (dlg->exec()) {
+		QString newname = dlg->keyDesc->text();
+		QString newcomment = dlg->comment->toPlainText();
+		if (newname != pki->getIntName() ||
+		    newcomment != pki->getComment())
+		{
+			updateItem(pki, newname, newcomment);
+		}
 	}
+	delete dlg;
 }
 exportType::etype db_key::clipboardFormat(QModelIndexList indexes) const
 {
@@ -203,7 +260,7 @@ exportType::etype db_key::clipboardFormat(QModelIndexList indexes) const
 
 	foreach(QModelIndex idx, indexes) {
 		pki_key *key = static_cast<pki_key*>
-				(idx.internalPointer());
+			(idx.internalPointer());
 		if (key->isPubKey() || key->isToken())
 			allPriv = false;
 		if (key->getKeyType() != EVP_PKEY_RSA &&
@@ -239,7 +296,7 @@ exportType::etype db_key::clipboardFormat(QModelIndexList indexes) const
 
 void db_key::store(QModelIndex index)
 {
-	const EVP_CIPHER *enc = NULL;
+	const EVP_CIPHER *algo = NULL, *encrypt = EVP_aes_256_cbc();
 	QString title = tr("Export public key [%1]");
 	QList<exportType> types;
 
@@ -298,17 +355,17 @@ void db_key::store(QModelIndex index)
 			key->writePublic(fname, true);
 			break;
 		case exportType::PEM_private_encrypt:
-			enc = EVP_des_ede3_cbc();
+			algo = encrypt;
 			/* fallthrough */
 		case exportType::PEM_private:
-			privkey->writeKey(fname, enc,
+			privkey->writeKey(fname, algo,
 				PwDialog::pwCallback, true);
 			break;
 		case exportType::PKCS8_encrypt:
-			enc = EVP_des_ede3_cbc();
+			algo = encrypt;
 			/* fallthrough */
 		case exportType::PKCS8:
-			privkey->writePKCS8(fname, enc,
+			privkey->writePKCS8(fname, algo,
 				PwDialog::pwCallback, true);
 			break;
 		case exportType::SSH2_public:
@@ -327,12 +384,16 @@ void db_key::store(QModelIndex index)
 void db_key::setOwnPass(QModelIndex idx, enum pki_key::passType x)
 {
 	pki_evp *targetKey;
+	enum pki_key::passType old_type;
+
 	if (!idx.isValid())
 		return;
 	targetKey = static_cast<pki_evp*>(idx.internalPointer());
 	if (targetKey->isToken()) {
 		throw errorEx(tr("Tried to change password of a token"));
 	}
+	old_type = targetKey->getOwnPass();
 	targetKey->setOwnPass(x);
-	updatePKI(targetKey);
+	if (!targetKey->sqlUpdatePrivateKey())
+		targetKey->setOwnPass(old_type);
 }
