@@ -9,6 +9,7 @@
 #include "db_temp.h"
 #include "func.h"
 #include <widgets/NewX509.h>
+#include <widgets/XcaDialog.h>
 #include <widgets/MainWindow.h>
 #include <QFileDialog>
 #include <QDir>
@@ -17,23 +18,26 @@
 #include <QInputDialog>
 #include <QMessageBox>
 
-db_temp::db_temp(QString DBfile, MainWindow *mw)
-	:db_x509name(DBfile, mw)
+db_temp::db_temp(MainWindow *mw)
+	:db_x509name(mw)
 {
 	class_name = "templates";
+	sqlHashTable = "templates";
 	pkitype << tmpl;
 
 	updateHeaders();
 	loadContainer();
 
-	predefs = newPKI();
 	QDir dir;
 	if (!dir.cd(getPrefix()))
 		return;
 	dir.setFilter(QDir::Files | QDir::NoSymLinks);
 	QFileInfoList list = dir.entryInfoList();
 	load_temp l;
-	pki_base *tmpl;
+	pki_temp *tmpl = new pki_temp(tr("Empty template"));
+	tmpl->setAsPreDefined();
+	predefs << tmpl;
+
 	for (int i = 0; i < list.size(); ++i) {
 		QFileInfo fileInfo = list.at(i);
 		QString name = getPrefix() + QDir::separator() +
@@ -41,55 +45,33 @@ db_temp::db_temp(QString DBfile, MainWindow *mw)
 		if (!name.endsWith(".xca", Qt::CaseInsensitive))
 			continue;
 		try {
-			tmpl = l.loadItem(name);
-			if (tmpl)
-				predefs->append(tmpl);
+			tmpl = dynamic_cast<pki_temp*>(l.loadItem(name));
+			if (tmpl) {
+				tmpl->setAsPreDefined();
+				predefs << tmpl;
+			}
 		} catch(errorEx &err) {
 			XCA_WARN(tr("Bad template: %1").arg(name));
 		}
 	}
 }
 
-dbheaderList db_temp::getHeaders()
-{
-	dbheaderList h = db_x509name::getHeaders();
-	h << new dbheader(HD_temp_type, true, tr("Type"));
-	return h;
-}
-
 db_temp::~db_temp()
 {
-	delete predefs;
+	return;
+	while (!predefs.isEmpty())
+		delete predefs.takeFirst();
 }
 
-pki_base *db_temp::newPKI(db_header_t *)
+pki_base *db_temp::newPKI(enum pki_type type)
 {
+	(void)type;
 	return new pki_temp("");
 }
 
-QStringList db_temp::getDescPredefs()
+QList<pki_temp *> db_temp::getAllAndPredefs()
 {
-	QStringList x;
-	x.clear();
-	for(pki_temp *pki=(pki_temp*)predefs->iterate(); pki;
-			pki=(pki_temp*)pki->iterate()) {
-		x.append(QString("[default] ") + pki->getIntName());
-	}
-	x += getDesc();
-	return x;
-}
-
-pki_base *db_temp::getByName(QString desc)
-{
-	if (!desc.startsWith("[default] "))
-		return db_base::getByName(desc);
-	desc.remove(0, 10); // "[default] "
-	for(pki_temp *pki=(pki_temp*)predefs->iterate(); pki;
-	            pki=(pki_temp*)pki->iterate()) {
-		if (pki->getIntName() == desc)
-			return pki;
-	}
-	return NULL;
+	return predefs + getAll<pki_temp>();
 }
 
 bool db_temp::runTempDlg(pki_temp *temp)
@@ -98,7 +80,6 @@ bool db_temp::runTempDlg(pki_temp *temp)
 	emit connNewX509(dlg);
 
 	dlg->setTemp(temp);
-	dlg->fromTemplate(temp);
 	if (!dlg->exec()) {
 		delete dlg;
 		return false;
@@ -111,43 +92,31 @@ bool db_temp::runTempDlg(pki_temp *temp)
 void db_temp::newItem()
 {
 	pki_temp *temp = NULL;
-	QStringList sl;
 	QString type;
-	bool ok;
-	int i, len;
-	len = predefs->childCount();
-	sl << tr("Nothing");
-	for (i=0; i<len; i++) {
-		sl << predefs->child(i)->getIntName();
-	}
-	type = QInputDialog::getItem(mainwin, XCA_TITLE,
-		tr("Preset Template values"), sl, 0, false, &ok, 0);
-	if (ok) {
-		if (type == sl[0]) {
-			temp = new pki_temp("");
-		} else {
-			for (i=0; i<len; i++) {
-				pki_temp *t = (pki_temp *)predefs->child(i);
-				if (type == t->getIntName()) {
-					temp = new pki_temp(t);
-					break;
-				}
+
+	itemComboTemp *ic = new itemComboTemp(NULL);
+	ic->insertPkiItems(predefs);
+	XcaDialog *dlg = new XcaDialog(mainwin, tmpl, ic,
+				tr("Preset Template values"), QString());
+	if (dlg->exec()) {
+		temp = new pki_temp(ic->currentPkiItem());
+		temp->pkiSource = generated;
+		if (temp) {
+			if (runTempDlg(temp)) {
+				insertPKI(temp);
+				createSuccess(temp);
+			} else {
+				delete temp;
 			}
 		}
-		if (!temp)
-			return;
-		temp->setIntName("--");
-		if (runTempDlg(temp)) {
-			insertPKI(temp);
-			createSuccess(temp);
-			return;
-		}
 	}
-	delete temp;
+	delete dlg;
 }
 void db_temp::showPki(pki_base *pki)
 {
-	alterTemp(static_cast<pki_temp *>(pki));
+	pki_temp *t = dynamic_cast<pki_temp *>(pki);
+	if (t)
+		alterTemp(t);
 }
 
 void db_temp::load()
@@ -163,7 +132,7 @@ void db_temp::store(QModelIndex index)
 
 	pki_temp *temp = static_cast<pki_temp*>(index.internalPointer());
 
-	QString fn = mainwin->getPath() + QDir::separator() +
+	QString fn = Settings["workingdir"] + QDir::separator() +
 		temp->getUnderlinedName() + ".xca";
 	QString s = QFileDialog::getSaveFileName(mainwin,
 		tr("Save template as"),	fn,
@@ -171,7 +140,7 @@ void db_temp::store(QModelIndex index)
 	if (s.isEmpty())
 		return;
 	s = nativeSeparator(s);
-	mainwin->setPath(s.mid(0, s.lastIndexOf(QRegExp("[/\\\\]")) ));
+	Settings["workingdir"] = s.mid(0, s.lastIndexOf(QRegExp("[/\\\\]")));
 	try {
 		temp->writeTemp(s);
 	}
@@ -182,8 +151,27 @@ void db_temp::store(QModelIndex index)
 
 bool db_temp::alterTemp(pki_temp *temp)
 {
+	XSqlQuery q;
+	QSqlError e;
+
 	if (!runTempDlg(temp))
 		return false;
-	updatePKI(temp);
+
+	Transaction;
+	if (!TransBegin())
+		return false;
+	SQL_PREPARE(q, "UPDATE templates SET version=?, template=? WHERE item=?");
+	q.bindValue(0, TMPL_VERSION);
+	q.bindValue(1, temp->toB64Data());
+	q.bindValue(2, temp->getSqlItemId());
+	q.exec();
+	e = q.lastError();
+	mainwin->dbSqlError(e);
+	if (e.isValid()) {
+		TransRollback();
+		return false;
+	}
+	updateItem(temp, temp->getIntName(), temp->getComment());
+	TransCommit();
 	return true;
 }
