@@ -15,19 +15,22 @@
 #include "pkcs11_lib.h"
 #include "db_base.h"
 
+class pki_scard;
 class db_token: public db_base
 {
 		Q_OBJECT
 	private:
 		slotid slot;
 	public:
-		db_token(QString db, MainWindow *mw);
+		db_token(MainWindow *mw);
 		bool setData(const QModelIndex &index,
 			const QVariant &value, int role);
 		void setSlot(slotid s)
 		{
 			slot = s;
 		}
+		void saveHeaderState();
+		void rename_token_in_database(pki_scard *token);
 };
 
 #endif
