@@ -11,43 +11,50 @@
 #include "pki_evp.h"
 #include "pki_scard.h"
 #include "pass_info.h"
-#include "widgets/CertDetail.h"
 #include "widgets/CertExtend.h"
 #include "widgets/ExportDialog.h"
 #include "widgets/MainWindow.h"
 #include "widgets/PwDialog.h"
 #include "widgets/RevocationList.h"
-#include "ui_TrustState.h"
 #include "ui_CaProperties.h"
-#include "ui_About.h"
 #include <QMessageBox>
 #include <QContextMenuEvent>
 #include <QAction>
 
+#include <openssl/rand.h>
+
 bool db_x509::treeview = true;
 
-db_x509::db_x509(QString DBfile, MainWindow *mw)
-	:db_x509super(DBfile, mw)
+db_x509::db_x509(MainWindow *mw)
+	:db_x509super(mw)
 {
-	rootItem->setIntName("[x509 root]");
 	class_name = "certificates";
+	sqlHashTable = "certs";
 	pkitype << x509;
 	updateHeaders();
 	loadContainer();
+	dereferenceIssuer();
 }
 
-void db_x509::updateAfterCrlLoad(pki_x509 *pki)
+void db_x509::dereferenceIssuer()
 {
-	if (pki->revList.merged) {
-		updatePKI(pki);
-		pki->revList.merged = false;
-	}
-}
-
-void db_x509::updateAfterDbLoad()
-{
-	FOR_ALL_pki(pki, pki_x509) {
-		updateAfterCrlLoad(pki);
+	XSqlQuery q("SELECT item, issuer FROM certs WHERE issuer is NOT NULL");
+	while (q.next()) {
+		pki_base *root = rootItem;
+		pki_x509 *cert = lookupPki<pki_x509>(q.value(0));
+		pki_x509 *issuer = lookupPki<pki_x509>(q.value(1));
+		if (cert && issuer) {
+			cert->setSigner(issuer);
+			if (cert != issuer)
+				root = issuer;
+		}
+		if (cert && cert->getParent() != root) {
+			qDebug() << "MOVE" << cert->getIntName()
+				<< "from" << cert->getParent()->getIntName()
+				<< "to" << root->getIntName();
+			cert->getParent()->takeChild(cert);
+			insertChild(root, cert);
+		}
 	}
 }
 
@@ -64,85 +71,56 @@ dbheaderList db_x509::getHeaders()
 				tr("Not before")) <<
 		new dbheader(HD_cert_notAfter,	true, tr("Expiry date"),
 				tr("Not after")) <<
-		new dbheader(HD_cert_trust,	false,tr("Trust state")) <<
 		new dbheader(HD_cert_revocation,false, tr("Revocation")) <<
 		new dbheader(HD_cert_crl_expire,true, tr("CRL Expiration"));
 	return h;
 }
 
-pki_base *db_x509::newPKI(db_header_t *)
+pki_base *db_x509::newPKI(enum pki_type type)
 {
+	(void)type;
 	return new pki_x509();
 }
 
-pki_x509 *db_x509::findSigner(pki_x509 *client)
+QList<pki_x509 *> db_x509::getAllIssuers()
 {
-	pki_x509 *signer;
-	if ((signer = client->getSigner()) != NULL)
-		return signer;
-	// first check for self-signed
-	if (client->verify(client)) {
-		return client;
-	}
-	FOR_ALL_pki(pki, pki_x509) {
-		if (client->verify(pki)) {
-			return pki;
-		}
-	}
-	return NULL;
+	/* Select X509 CA certificates with available private key */
+	return sqlSELECTpki<pki_x509>("SELECT x509super.item FROM x509super "
+		"JOIN private_keys ON x509super.pkey = private_keys.item "
+		"JOIN certs ON certs.item = x509super.item "
+		"WHERE certs.ca=1") +
+		sqlSELECTpki<pki_x509>("SELECT x509super.item FROM x509super "
+		"JOIN tokens ON x509super.pkey = tokens.item "
+		"JOIN certs ON certs.item = x509super.item "
+		"WHERE certs.ca=1");
 }
 
-QStringList db_x509::getPrivateDesc()
+void db_x509::remFromCont(const QModelIndex &idx)
 {
-	QStringList x;
-	FOR_ALL_pki(pki, pki_x509)
-		if (pki->getRefKey())
-			x.append(pki->getIntName());
-	return x;
-}
-
-QStringList db_x509::getSignerDesc()
-{
-	QStringList x;
-	FOR_ALL_pki(pki, pki_x509)
-		if (pki->canSign())
-			x.append(pki->getIntName());
-	return x;
-}
-
-
-void db_x509::remFromCont(QModelIndex &idx)
-{
-	int row;
+	db_x509super::remFromCont(idx);
 	pki_base *pki = static_cast<pki_base*>(idx.internalPointer());
-	pki_base *parent_pki = pki->getParent();
-	row = pki->row();
 	pki_x509 *child;
 	pki_base *new_parent;
 	QModelIndex new_idx;
-
-	beginRemoveRows(parent(idx), row, row);
-	parent_pki->takeChild(pki);
-	endRemoveRows();
+	QList<pki_x509 *> childs;
 
 	while (pki->childCount()) {
 		child = (pki_x509*)pki->childItems.takeFirst();
 		child->delSigner((pki_x509*)pki);
-		new_parent = findSigner(child);
+		new_parent = child->findIssuer();
 		insertChild(new_parent, child);
+		if (new_parent)
+			childs << child;
+	}
+	XSqlQuery q;
+	SQL_PREPARE(q, "UPDATE certs SET issuer=? WHERE item=?");
+	foreach(pki_x509 *child, childs) {
+		q.bindValue(0, child->getSigner()->getSqlItemId());
+		q.bindValue(1, child->getSqlItemId());
+		AffectedItems(child->getSqlItemId());
+		q.exec();
 	}
 	mainwin->crls->removeSigner(pki);
-	pki_key *pub = ((pki_x509*)pki)->getPubKey();
-	if (pub) {
-		if (mainwin->certs->findByPubKey(pub).count() == 0) {
-			QList<pki_x509super *> reqs;
-			reqs = mainwin->reqs->findByPubKey(pub);
-			foreach(pki_x509super *r, reqs)
-				((pki_x509req*)r)->setDone(false);
-		}
-		delete pub;
-	}
-	return;
 }
 
 void db_x509::changeView()
@@ -157,9 +135,9 @@ void db_x509::changeView()
 	beginRemoveRows(QModelIndex(), 0, rows -1);
 	pki_base *pki = rootItem;
 	pki_base *parent;
-	while(pki->childCount()) {
+	while (pki->childCount()) {
 		pki = pki->takeFirst();
-		while(pki != rootItem && !pki->childCount()) {
+		while (pki != rootItem && !pki->childCount()) {
 			parent = pki->getParent();
 			temproot->append(pki);
 			pki = parent;
@@ -174,18 +152,15 @@ void db_x509::changeView()
 		mainwin->BNviewState->setText(tr("Tree View"));
 
 	while ((temproot->childCount())) {
+		pki_base *parent = rootItem;
 		pki = temproot->takeFirst();
-		inToCont(pki);
+		if (treeview)
+			parent = static_cast<pki_x509*>(pki)->getSigner();
+		insertChild(parent, pki);
 	}
 	delete temproot;
 }
 
-void db_x509::calcEffTrust()
-{
-	FOR_ALL_pki(pki, pki_x509)
-		pki->calcEffTrust();
-}
-
 static bool recursiveSigning(pki_x509 *cert, pki_x509 *client)
 {
 	/* recursive signing check */
@@ -194,9 +169,8 @@ static bool recursiveSigning(pki_x509 *cert, pki_x509 *client)
 			return false;
 		}
 		if (s == client) {
-			printf("Recursive signing: '%s' <-> '%s'\n",
-				CCHAR(s->getIntName()),
-				CCHAR(cert->getIntName()));
+			qWarning() << "Recursive signing:" << s->getIntName()
+				<< "<->" << cert->getIntName();
 			return true;
 		}
 	}
@@ -205,10 +179,8 @@ static bool recursiveSigning(pki_x509 *cert, pki_x509 *client)
 
 void db_x509::inToCont(pki_base *pki)
 {
-	pki_x509 *cert = (pki_x509*)pki;
+	pki_x509 *cert = static_cast<pki_x509*>(pki);
 	cert->setParent(NULL);
-	cert->delSigner(cert->getSigner());
-	findSigner(cert);
 	pki_base *root = cert->getSigner();
 	if (!treeview || root == cert || root == NULL)
 		root = rootItem;
@@ -216,30 +188,47 @@ void db_x509::inToCont(pki_base *pki)
 	insertChild(root, cert);
 
 	QList<pki_x509 *> childs;
+	QList<pki_x509 *> items;
+	unsigned pubhash = cert->pubHash();
+	unsigned namehash = cert->getSubject().hashNum();
+	x509revList revList;
+
 	/* Search for another certificate (name and key)
 	 * and use its childs if we are newer */
-	FOR_ALL_pki(client, pki_x509) {
-		if (client == cert)
+	items = sqlSELECTpki<pki_x509>(
+		"SELECT x509super.item FROM x509super "
+		"JOIN public_keys ON x509super.pkey = public_keys.item "
+		"JOIN certs ON certs.item = x509super.item "
+		"WHERE certs.ca=1 AND x509super.subj_hash=? "
+		"AND x509super.key_hash=?",
+			QList<QVariant>() << namehash << pubhash);
+	foreach(pki_x509 *other, items) {
+		if (other == cert)
 			continue;
-		if (!client->compareNameAndKey(cert))
+		if (!other->compareNameAndKey(cert))
 			continue;
-		if (cert->getNotAfter() < client->getNotAfter())
+		if (cert->getNotAfter() < other->getNotAfter())
 			continue;
-		foreach(pki_base *_child, client->childItems) {
-			pki_x509 *child = static_cast<pki_x509*>(_child);
-			child->delSigner(client);
+		foreach(pki_base *b, other->childItems) {
+			pki_x509 *child = static_cast<pki_x509*>(b);
+			child->delSigner(other);
 			childs << child;
 		}
+		revList.merge(other->getRevList());
 	}
 	/* Search rootItem childs, whether they are ours */
-	foreach(pki_base *_child, rootItem->childItems) {
-		pki_x509 *child = static_cast<pki_x509*>(_child);
+	foreach(pki_base *b, rootItem->childItems) {
+		pki_x509 *child = static_cast<pki_x509*>(b);
 		if (child == cert || child->getSigner() == child)
 			continue;
 		if (child->verify_only(cert))
 			childs << child;
 	}
 	/* move collected childs to us */
+	XSqlQuery q;
+	x509revList revokedChilds;
+	SQL_PREPARE(q, "UPDATE certs SET issuer=? WHERE item=?");
+	q.bindValue(0, cert->getSqlItemId());
 	foreach(pki_x509 *child, childs) {
 		int row;
 		if (recursiveSigning(cert, child))
@@ -247,61 +236,58 @@ void db_x509::inToCont(pki_base *pki)
 		if (!child->verify(cert))
 			continue;
 		row = child->row();
-		beginRemoveRows(index(child), row, row);
-		child->getParent()->takeChild(child);
-		endRemoveRows();
-		if (treeview)
+		if (treeview) {
+			beginRemoveRows(index(child->getParent()), row, row);
+			child->getParent()->takeChild(child);
+			endRemoveRows();
 			insertChild(cert, child);
-		else
-			insertChild(rootItem, child);
-	}
-	findKey(cert);
-	pki_key *pub = cert->getPubKey();
-	if (pub) {
-		QList<pki_x509super *> reqs = mainwin->reqs->findByPubKey(pub);
-		delete pub;
-
-		foreach(pki_x509super *r, reqs) {
-			((pki_x509req*)r)->setDone();
 		}
+		q.bindValue(1, child->getSqlItemId());
+		AffectedItems(child->getSqlItemId());
+		q.exec();
+		mainwin->dbSqlError(q.lastError());
+		if (child->isRevoked())
+			revokedChilds << child->getRevocation();
+	}
+	q.finish();
+	revList.merge(revokedChilds);
+	cert->setRevocations(revList);
+
+	/* Update CRLs */
+	QList<pki_crl *> crls = sqlSELECTpki<pki_crl>(
+			"SELECT item FROM crls WHERE iss_hash=?",
+			QList<QVariant>() << namehash);
+	SQL_PREPARE(q, "UPDATE crls SET issuer=? WHERE item=?");
+	foreach(pki_crl *crl, crls) {
+		crl->verify(cert);
+		if (cert != crl->getIssuer())
+			continue;
+		q.bindValue(0, cert->getSqlItemId());
+		q.bindValue(1, crl->getSqlItemId());
+		AffectedItems(crl->getSqlItemId());
+		q.exec();
+		mainwin->dbSqlError(q.lastError());
 	}
-	calcEffTrust();
 }
 
-pki_x509 *db_x509::getBySubject(const x509name &xname, pki_x509 *last)
-{
-	bool lastfound = false;
-	if (last == NULL) lastfound = true;
-
-	FOR_ALL_pki(pki, pki_x509) {
-		if ( pki->getSubject() ==  xname) {
-			if (lastfound) {
-				return pki;
-			}
-		}
-		if (pki == last) {
-			lastfound = true;
-		}
-	}
-	return NULL;
-}
-
-void db_x509::writeAllCerts(const QString fname, bool onlyTrusted)
+void db_x509::writeAllCerts(const QString fname, bool unrevoked)
 {
 	bool append = false;
 	FOR_ALL_pki(pki, pki_x509) {
-		if (onlyTrusted && pki->getTrust() != 2) continue;
+		if (unrevoked && pki->isRevoked())
+			 continue;
 		pki->writeCert(fname.toLatin1(), true, append);
 		append = true;
 	}
 }
 
-QList<pki_x509*> db_x509::getCerts(bool onlyTrusted)
+QList<pki_x509*> db_x509::getCerts(bool unrevoked)
 {
 	QList<pki_x509*> c;
 	c.clear();
 	FOR_ALL_pki(pki, pki_x509) {
-		if (onlyTrusted && pki->getTrust() != 2) continue;
+		if (unrevoked && pki->isRevoked())
+			continue;
 		c.append(pki);
 	}
 	return c;
@@ -309,63 +295,57 @@ QList<pki_x509*> db_x509::getCerts(bool onlyTrusted)
 
 void db_x509::writeIndex(const QString fname, bool hierarchy)
 {
-	bool append = false;
 	if (hierarchy) {
-		FOR_ALL_pki(pki, pki_x509) {
-			if (pki->childCount()) {
-
-				QString newfname = fname + "." + pki->getIntName().replace(QRegExp("[^a-zA-Z0-9]"),QString(""));;
-
-				append = false;
-				foreach(pki_base *_child, pki->childItems) {
-					pki_x509 *child = static_cast<pki_x509*>(_child);
-					child->writeIndexEntry(newfname, append);
-					append = true;
-				}
-			}
+		QString dir = fname + "/";
+		dir = nativeSeparator(dir);
+		QList<pki_x509*> issuers = sqlSELECTpki<pki_x509>(
+			"SELECT DISTINCT issuer FROM certs WHERE issuer != item");
+		foreach(pki_x509 *ca, issuers) {
+			writeIndex(dir + ca->getUnderlinedName() + ".txt",
+				sqlSELECTpki<pki_x509>(
+				  "SELECT item FROM certs WHERE issuer=?",
+				  QList<QVariant>() << QVariant(ca->getSqlItemId())
+				)
+			);
 		}
 	} else {
-		FOR_ALL_pki(pki, pki_x509) {
-			pki->writeIndexEntry(fname, append);
-			append = true;
-		}
+		writeIndex(fname,sqlSELECTpki<pki_x509>("SELECT item FROM certs"));
 	}
 }
 
-a1int db_x509::searchSerial(pki_x509 *signer)
+#define SERIAL_LEN 8
+static a1int randomSerial()
 {
-	// returns the highest certificate serial
-	// of all certs with this signer (itself too)
-	a1int sserial, myserial;
-	if (!signer)
-		return sserial;
-	sserial = signer->getCaSerial();
-	FOR_ALL_pki(pki, pki_x509)
-		if (pki->getSigner() == signer)  {
-			myserial = pki->getSerial();
-			if (sserial < myserial) {
-				sserial = myserial;
-			}
-		}
-	return sserial;
+	unsigned char buf[SERIAL_LEN];
+	RAND_bytes(buf, SERIAL_LEN);
+	a1int serial;
+	serial.setRaw(buf, SERIAL_LEN);
+	return serial;
 }
 
 a1int db_x509::getUniqueSerial(pki_x509 *signer)
 {
-	// returnes an unused unique serial
-	a1int serial;
+	// returns an unused unique serial
+	a1int serial, signer_serial;
 	x509rev rev;
+	x509revList revList;
+	if (signer) {
+		signer_serial = signer->getSerial();
+		revList = signer->getRevList();
+	}
 	while (true) {
-		serial = signer->getIncCaSerial();
+		serial = randomSerial();
+		if (!signer)
+			break;
+		if (signer_serial == serial)
+			continue;
 		rev.setSerial(serial);
-		if (signer->revList.contains(rev))
+		if (revList.contains(rev))
 			continue;
 		if (signer->getBySerial(serial))
 			continue;
 		break;
 	}
-	if (!signer->usesRandomSerial())
-		updatePKI(signer);
 	return serial;
 }
 
@@ -378,28 +358,7 @@ pki_base *db_x509::insert(pki_base *item)
 		delete(cert);
 		return NULL;
 	}
-	cert->setCaSerial((cert->getSerial()));
 	insertPKI(cert);
-	a1int serial;
-
-	// check the CA serial of the CA of this cert to avoid serial doubles
-	if (cert->getSigner() != cert && cert->getSigner()) {
-		serial = cert->getSerial();
-		if (cert->getSigner()->getCaSerial() < ++serial ) {
-			cert->getSigner()->setCaSerial(serial);
-			updatePKI(cert->getSigner());
-		}
-	}
-
-	// check CA serial of this cert
-	serial = searchSerial(cert);
-	if ( ++serial > cert->getCaSerial()) {
-		cert->setCaSerial(serial);
-	}
-	if (mainwin->crls) {
-		mainwin->crls->updateRevocations(cert);
-	}
-	updatePKI(cert);
 	return cert;
 }
 
@@ -420,6 +379,35 @@ pki_x509 *db_x509::get1SelectedCert()
 	return static_cast<pki_x509*>(index.internalPointer());
 }
 
+void db_x509::markRequestSigned(pki_x509req *req, pki_x509 *cert)
+{
+	if (!req || !cert)
+		return;
+	pki_x509 *issuer = cert->getSigner();
+
+	Transaction;
+	if (!TransBegin())
+		return;
+
+	XSqlQuery q;
+	req->setDone();
+	SQL_PREPARE(q, "UPDATE requests SET signed=? WHERE item=?");
+	q.bindValue(0, 1);
+	q.bindValue(1, req->getSqlItemId());
+	AffectedItems(req->getSqlItemId());
+	q.exec();
+
+	a1time a;
+	req->selfComment(tr("Signed on %1 by '%2'").arg(a.toPretty())
+		.arg(issuer ? issuer->getIntName() : tr("Unknown")));
+	SQL_PREPARE(q, "UPDATE items SET comment=? WHERE id=?");
+	q.bindValue(0, req->getComment());
+	q.bindValue(1, req->getSqlItemId());
+	q.exec();
+
+	TransCommit();
+}
+
 void db_x509::newItem()
 {
 	NewX509 *dlg = new NewX509(mainwin);
@@ -471,7 +459,7 @@ void db_x509::newCert(pki_x509 *cert)
 	delete dlg;
 }
 
-void db_x509::newCert(NewX509 *dlg)
+pki_x509 *db_x509::newCert(NewX509 *dlg)
 {
 	pki_x509 *cert = NULL;
 	pki_x509 *signcert = NULL;
@@ -482,19 +470,19 @@ void db_x509::newCert(NewX509 *dlg)
 	QString intname;
 
     try {
-
+	Transaction;
 	// Step 1 - Subject and key
 	if (!dlg->fromReqCB->isChecked()) {
 		clientkey = dlg->getSelectedKey();
 		if (!clientkey)
-			return;
+			return NULL;
 		subject = dlg->getX509name();
 		intname = dlg->description->text();
 	} else {
 		// A PKCS#10 Request was selected
 		req = dlg->getSelectedReq();
 		if (!req)
-			return;
+			return NULL;
 		clientkey = req->getRefKey();
 		if (clientkey == NULL) {
 			clientkey = req->getPubKey();
@@ -506,6 +494,8 @@ void db_x509::newCert(NewX509 *dlg)
 			subject = req->getSubject();
 		intname = req->getIntName();
 	}
+	TransThrow();
+
 	if (clientkey == NULL)
 		throw errorEx(tr("Invalid public key"));
 	// initially create cert
@@ -517,27 +507,16 @@ void db_x509::newCert(NewX509 *dlg)
 	// Step 2 - select Signing
 	if (dlg->foreignSignRB->isChecked()) {
 		signcert = dlg->getSelectedSigner();
-		if (!signcert)
-			return;
+		if (!signcert) {
+			delete cert;
+			return NULL;
+		}
 		serial = getUniqueSerial(signcert);
 		signkey = signcert->getRefKey();
-		cert->setTrust(1);
-#ifdef WG_QA_SERIAL
-	} else if (dlg->selfQASignRB->isChecked()){
-		Passwd pass;
-		pass_info p(XCA_TITLE, tr("Please enter the new hexadecimal secret number for the QA process."));
-		if (PwDialog::execute(&p, &pass, true) != 1)
-			throw errorEx(tr("The QA process has been terminated by the user."));
-		signcert = cert;
-		signkey = clientkey;
-		serial.setHex(pass);
-		cert->setTrust(2);
-#endif
 	} else {
 		signcert = cert;
 		signkey = clientkey;
-		serial.setHex(dlg->serialNr->text());
-		cert->setTrust(2);
+		serial = getUniqueSerial(NULL);
 	}
 
 	dlg->initCtx(cert, signcert, NULL);
@@ -576,21 +555,14 @@ void db_x509::newCert(NewX509 *dlg)
 	}
 
 	const EVP_MD *hashAlgo = dlg->hashAlgo->currentHash();
-#ifdef WG_QA_SERIAL
-	if (dlg->selfQASignRB->isChecked()) {
-		// sign the request intermediately in order to finally fill
-		// up the cert_info substructure.
-		cert->sign(signkey, hashAlgo);
-		// now set the QA serial.
-		cert->setSerial(cert->hashInfo(EVP_md5()));
-	}
-#endif
 	// and finally sign the request
 	cert->sign(signkey, hashAlgo);
 
+	// set the comment field
+	cert->setComment(dlg->comment->toPlainText());
+	cert->pkiSource = dlg->getPkiSource();
 	cert = (pki_x509*)insert(cert);
 	createSuccess(cert);
-	updatePKI(signcert);
 	if (cert && clientkey->isToken()) {
 		pki_scard *card = (pki_scard*)clientkey;
 		if (XCA_YESNO(tr("Store the certificate to the key on the token '%1 (#%2)' ?").
@@ -605,6 +577,8 @@ void db_x509::newCert(NewX509 *dlg)
 	}
 	if (tempkey != NULL)
 		delete(tempkey);
+	markRequestSigned(req, cert);
+	TransCommit();
     }
 
     catch (errorEx &err) {
@@ -612,23 +586,9 @@ void db_x509::newCert(NewX509 *dlg)
 		delete cert;
 		if (tempkey != NULL)
 			delete(tempkey);
+		cert = NULL;
     }
-}
-
-void db_x509::showPki(pki_base *pki)
-{
-	pki_x509 *crt = (pki_x509 *)pki;
-	CertDetail *dlg;
-	dlg = new CertDetail(mainwin);
-	if (dlg) {
-		dlg->setCert(crt);
-		connect(dlg->privKey, SIGNAL(doubleClicked(QString)),
-			mainwin->keys, SLOT(showItem(QString)));
-		connect(dlg->signature, SIGNAL(doubleClicked(QString)),
-			this, SLOT( showItem(QString) ));
-		dlg->exec();
-		delete dlg;
-	}
+    return cert;
 }
 
 void db_x509::store(QModelIndex idx)
@@ -694,12 +654,12 @@ void db_x509::store(QModelIndexList list)
 				"PEM + PKCS#8");
 	}
 	types << exportType() <<
-		exportType(exportType::PKCS7_trusted, "p7b",
-			tr("PKCS#7 trusted")) <<
+		exportType(exportType::PKCS7_unrevoked, "p7b",
+			tr("PKCS#7 unrevoked")) <<
 		exportType(exportType::PKCS7_all, "p7b",
 			tr("PKCS#7 all")) <<
-		exportType(exportType::PEM_trusted, "pem",
-			tr("PEM trusted")) <<
+		exportType(exportType::PEM_unrevoked, "pem",
+			tr("PEM unrevoked")) <<
 		exportType(exportType::PEM_all, "pem",
 			tr("PEM all")) <<
 		exportType(exportType::Index, "txt",
@@ -714,6 +674,7 @@ void db_x509::store(QModelIndexList list)
 		return;
 	}
 	QString fname = dlg->filename->text();
+	QList<pki_x509*> certs;
 	enum exportType::etype type = dlg->type();
 	delete dlg;
 	try {
@@ -738,7 +699,7 @@ void db_x509::store(QModelIndexList list)
 				append = true;
 			}
 			break;
-		case exportType::PEM_trusted:
+		case exportType::PEM_unrevoked:
 			writeAllCerts(fname,true);
 			break;
 		case exportType::PEM_all:
@@ -749,7 +710,7 @@ void db_x509::store(QModelIndexList list)
 			break;
 		case exportType::PKCS7:
 		case exportType::PKCS7_chain:
-		case exportType::PKCS7_trusted:
+		case exportType::PKCS7_unrevoked:
 		case exportType::PKCS7_selected:
 		case exportType::PKCS7_all:
 			writePKCS7(crt, fname, type, list);
@@ -783,12 +744,10 @@ void db_x509::store(QModelIndexList list)
 			crt->writeCert(fname, true, true);
 			break;
 		case exportType::Index:
-			append = false;
-			foreach(QModelIndex idx, list) {
-				crt = static_cast<pki_x509*>(idx.internalPointer());
-				crt->writeIndexEntry(fname, append);
-				append = true;
-			}
+			foreach(QModelIndex idx, list)
+				certs << static_cast<pki_x509*>
+					(idx.internalPointer());
+			writeIndex(fname, certs);
 			break;
 		default:
 			exit(1);
@@ -799,6 +758,23 @@ void db_x509::store(QModelIndexList list)
 	}
 }
 
+void db_x509::writeIndex(const QString fname, QList<pki_x509*> items)
+{
+	QFile file(fname);
+	file.open(QFile::ReadWrite | QFile::Truncate);
+	if (file.error()) {
+		throw errorEx(tr("Error opening file: '%1': %2")
+			.arg(fname).arg(strerror(errno)));
+		return;
+	}
+
+	QString index;
+	foreach(pki_x509 *cert, items) {
+		if (cert)
+			index += cert->getIndexEntry();
+	}
+	file.write(index.toUtf8());
+}
 
 void db_x509::writePKCS12(pki_x509 *cert, QString s, bool chain)
 {
@@ -858,11 +834,11 @@ void db_x509::writePKCS7(pki_x509 *cert, QString s, exportType::etype type,
 				p7->addCert(cert);
 			}
 			break;
-		case exportType::PKCS7_trusted:
+		case exportType::PKCS7_unrevoked:
 		case exportType::PKCS7_all:
 			FOR_ALL_pki(cer, pki_x509) {
 				if ((type == exportType::PKCS7_all) ||
-				    (cer->getTrust() == 2))
+				    (!cer->isRevoked()))
 					p7->addCert(cer);
 			}
 			break;
@@ -885,61 +861,15 @@ void db_x509::manageRevocations(QModelIndex idx)
 	if (!cert)
 		return;
 	RevocationList *dlg = new RevocationList(mainwin);
-	dlg->setRevList(cert->revList, cert);
+	dlg->setRevList(cert->getRevList(), cert);
 	connect(dlg, SIGNAL(genCRL(pki_x509*)),
 		mainwin->crls, SLOT(newItem(pki_x509*)));
 	if (dlg->exec()) {
 		cert->setRevocations(dlg->getRevList());
-		updatePKI(cert);
 		emit columnsContentChanged();
 	}
 }
 
-void db_x509::setTrust(QModelIndexList indexes)
-{
-	int newstate = -1;
-	Ui::TrustState ui;
-	pki_x509 *cert = NULL;
-
-	if (indexes.size() == 0)
-		return;
-	if (indexes.size() == 1)
-		cert = static_cast<pki_x509*>(indexes[0].internalPointer());
-
-	QDialog *dlg = new QDialog(mainwin);
-	ui.setupUi(dlg);
-	ui.image->setPixmap(*MainWindow::certImg);
-	if (cert) {
-		int state = cert->getTrust();
-		if (cert->getSigner() == cert) {
-			if (state == 1)
-				state = 0;
-			ui.trust1->setDisabled(true);
-		}
-		if (state == 0 ) ui.trust0->setChecked(true);
-		if (state == 1 ) ui.trust1->setChecked(true);
-		if (state == 2 ) ui.trust2->setChecked(true);
-	}
-	if (!dlg->exec()) {
-		delete dlg;
-		return;
-	}
-	if (ui.trust0->isChecked()) newstate = 0;
-	if (ui.trust1->isChecked()) newstate = 1;
-	if (ui.trust2->isChecked()) newstate = 2;
-	delete dlg;
-	if (newstate == -1) {
-		return;
-	}
-	foreach(QModelIndex idx, indexes) {
-		cert = static_cast<pki_x509*>(idx.internalPointer());
-		if (newstate != cert->getTrust()) {
-			cert->setTrust(newstate);
-			updatePKI(cert);
-		}
-	}
-}
-
 void db_x509::certRenewal(QModelIndexList indexes)
 {
 	pki_x509 *oldcert = NULL, *signer = NULL, *newcert =NULL;
@@ -1037,7 +967,7 @@ void db_x509::do_revoke(QModelIndexList indexes, const x509rev &r)
 		}
 	}
 	if (!parent) {
-		qWarning("%s(%d): Certs have different/no signer\n",
+		qWarning("%s(%d): Certs have different/no signer",
 			 __func__, __LINE__);
 	}
 	foreach(QModelIndex idx, indexes) {
@@ -1048,12 +978,13 @@ void db_x509::do_revoke(QModelIndexList indexes, const x509rev &r)
 		revlist << rev;
 	}
 	parent->mergeRevList(revlist);
-	updatePKI(parent);
 }
 
 void db_x509::unRevoke(QModelIndexList indexes)
 {
 	pki_x509 *parent = NULL;
+	x509revList revList;
+
 	foreach(QModelIndex idx, indexes) {
 		pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
 		if (!cert)
@@ -1067,9 +998,12 @@ void db_x509::unRevoke(QModelIndexList indexes)
 		}
 	}
 	if (!parent) {
-		qWarning("%s(%d): Certs have different/no signer\n",
+		qWarning("%s(%d): Certs have different/no issuer\n",
 			 __func__, __LINE__);
+		return;
 	}
+	revList = parent->getRevList();
+
 	foreach(QModelIndex idx, indexes) {
 		pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
 		int i;
@@ -1077,11 +1011,11 @@ void db_x509::unRevoke(QModelIndexList indexes)
 
 		cert->setRevoked(x509rev());
 		rev.setSerial(cert->getSerial());
-		i = parent->revList.indexOf(rev);
+		i = revList.indexOf(rev);
 		if (i != -1)
-			parent->revList.takeAt(i);
+			revList.takeAt(i);
 	}
-	updatePKI(parent);
+	parent->setRevocations(revList);
 	emit columnsContentChanged();
 }
 
@@ -1092,6 +1026,7 @@ void db_x509::toCertificate(QModelIndex index)
 		return;
 	if (!cert->getRefKey() && cert->getSigner() != cert)
 		extractPubkey(index);
+	cert->pkiSource = transformed;
 	newCert(cert);
 }
 
@@ -1104,6 +1039,7 @@ void db_x509::toRequest(QModelIndex idx)
 	try {
 		pki_x509req *req = new pki_x509req();
 		check_oom(req);
+		req->pkiSource = transformed;
 		req->setIntName(cert->getIntName());
 		req->createReq(cert->getRefKey(), cert->getSubject(),
 			cert->getDigest(), cert->getV3ext());
@@ -1128,8 +1064,8 @@ void db_x509::toToken(QModelIndex idx, bool alwaysSelect)
 
 void db_x509::caProperties(QModelIndex idx)
 {
+	QStringList actions;
 	Ui::CaProperties ui;
-	int i;
 
 	pki_x509 *cert = static_cast<pki_x509*>(idx.internalPointer());
 	if (!cert)
@@ -1137,30 +1073,44 @@ void db_x509::caProperties(QModelIndex idx)
 
 	QDialog *dlg = new QDialog(mainwin);
 	ui.setupUi(dlg);
-	ui.serial->setText(cert->getCaSerial().toHex());
-	ui.serial->setDisabled(cert->usesRandomSerial());
-	ui.randomSerial->setChecked(cert->usesRandomSerial());
 	ui.days->setSuffix(tr(" days"));
 	ui.days->setMaximum(1000000);
 	ui.days->setValue(cert->getCrlDays());
 	ui.image->setPixmap(*MainWindow::certImg);
-	QString templ = cert->getTemplate();
-	QStringList tempList = mainwin->temps->getDesc();
-	for (i=0; i<tempList.count(); i++) {
-		if (tempList[i] == templ)
-			break;
-	}
-	ui.temp->addItems(tempList);
-	ui.temp->setCurrentIndex(i);
+
+	QVariant tmplId = cert->getTemplateSqlId();
+	pki_temp *templ = mainwin->temps->lookupPki<pki_temp>(tmplId);
+
+	ui.temp->insertPkiItems(mainwin->temps->getAll<pki_temp>());
+        ui.temp->setNullItem(tr("No template"));
+	ui.temp->setCurrentIndex(0);
+	if (templ)
+		ui.temp->setCurrentPkiItem(templ);
+
 	ui.certName->setTitle(cert->getIntName());
+
 	if (dlg->exec()) {
-		a1int nserial;
+		XSqlQuery q;
+		QSqlError e;
+		Transaction;
+		TransThrow();
+
+		templ = ui.temp->currentPkiItem();
+		tmplId = templ ? templ->getSqlItemId() : QVariant();
+
+		cert->setTemplateSqlId(tmplId);
 		cert->setCrlDays(ui.days->value());
-		nserial.setHex(ui.serial->text());
-		cert->setCaSerial(nserial);
-		cert->setTemplate(ui.temp->currentText());
-		cert->setUseRandomSerial(ui.randomSerial->isChecked());
-		updatePKI(cert);
+
+		SQL_PREPARE(q, "UPDATE authority SET crlDays=?, "
+				"template=? WHERE item=?");
+
+		q.bindValue(0, cert->getCrlDays());
+		q.bindValue(1, tmplId);
+		q.bindValue(2, cert->getSqlItemId());
+		AffectedItems(cert->getSqlItemId());
+		q.exec();
+	        TransDone(q.lastError());
+		mainwin->dbSqlError(q.lastError());
 	}
 	delete dlg;
 }
