@@ -27,30 +27,26 @@ class db_x509: public db_x509super
 		QPixmap *certicon[4];
 		pki_x509 *get1SelectedCert();
 		dbheaderList getHeaders();
+		void dereferenceIssuer();
 
 	public:
 		static bool treeview;
-		db_x509(QString DBfile, MainWindow *mw);
-		pki_base *newPKI(db_header_t *head = NULL);
-		pki_x509 *findSigner(pki_x509 *client);
-		void updateAfterDbLoad();
-		void updateAfterCrlLoad(pki_x509 *pki);
+		db_x509(MainWindow *mw);
+		pki_base *newPKI(enum pki_type type = none);
+		pki_x509 *findIssuer(pki_x509 *client);
 
 		bool updateView();
 		void updateViewAll();
 		void updateViewPKI(pki_base *pki);
-		void remFromCont(QModelIndex &idx);
-		QStringList getPrivateDesc();
-		QStringList getSignerDesc();
-		void calcEffTrust();
-		QList<pki_x509*> getCerts(bool onlyTrusted);
-		a1int searchSerial(pki_x509 *signer);
-		void writeAllCerts(const QString fname, bool onlyTrusted);
+		void remFromCont(const QModelIndex &idx);
+		QList<pki_x509*> getAllIssuers();
+		QList<pki_x509*> getCerts(bool unrevoked);
 		void writeIndex(const QString fname, bool hierarchy);
-		pki_x509 *getByIssSerial(const pki_x509 *issuer, const a1int &a);
-		pki_x509 *getBySubject(const x509name &xname, pki_x509 *last = NULL);
+		void writeIndex(const QString fname, QList<pki_x509*> items);
+		void writeAllCerts(const QString fname, bool unrevoked);
 		pki_base *insert(pki_base *item);
-		void newCert(NewX509 *dlg);
+		void markRequestSigned(pki_x509req *req, pki_x509 *cert);
+		pki_x509 *newCert(NewX509 *dlg);
 		void newCert(pki_x509 *cert);
 		void writePKCS12(pki_x509 *cert, QString s, bool chain);
 		void writePKCS7(pki_x509 *cert, QString s,
@@ -63,7 +59,6 @@ class db_x509: public db_x509super
 		void toRequest(QModelIndex idx);
 		void store(QModelIndex idx);
 		void store(QModelIndexList list);
-		void showPki(pki_base *pki);
 		void load();
 		void caProperties(QModelIndex idx);
 		void toCertificate(QModelIndex index);
@@ -72,7 +67,6 @@ class db_x509: public db_x509super
 		void revoke(QModelIndexList indexes);
 		void do_revoke(QModelIndexList indexes, const x509rev &r);
 		void unRevoke(QModelIndexList indexes);
-		void setTrust(QModelIndexList indexes);
 
 	public slots:
 		void newItem();
