@@ -8,12 +8,13 @@
 #include "pki_base.h"
 #include "db_x509super.h"
 #include "widgets/MainWindow.h"
-#include "ui_About.h"
+#include "widgets/CertDetail.h"
+#include "widgets/XcaDialog.h"
 #include "oid.h"
 #include <QMessageBox>
 
-db_x509name::db_x509name(QString db, MainWindow *mw)
-	:db_base(db, mw)
+db_x509name::db_x509name(MainWindow *mw)
+	:db_base(mw)
 {
 }
 
@@ -30,11 +31,26 @@ dbheaderList db_x509name::getHeaders()
 	return h;
 }
 
-db_x509super::db_x509super(QString db, MainWindow *mw)
-	:db_x509name(db, mw)
+db_x509super::db_x509super(MainWindow *mw)
+	:db_x509name(mw)
 {
 }
 
+void db_x509super::loadContainer()
+{
+	db_x509name::loadContainer();
+	/* Resolve Key references */
+	FOR_ALL_pki(pki, pki_x509super) {
+		QVariant keySqlId = pki->getKeySqlId();
+		if (!keySqlId.isValid())
+			continue;
+		quint64 id = keySqlId.toULongLong();
+		if (!lookup.contains(id))
+			continue;
+		pki->setRefKey(static_cast<pki_key*>(lookup[id]));
+	}
+}
+
 dbheaderList db_x509super::getHeaders()
 {
 	dbheaderList h = db_x509name::getHeaders();
@@ -71,16 +87,6 @@ dbheaderList db_x509super::getHeaders()
 	return h;
 }
 
-void db_x509super::delKey(pki_key *delkey)
-{
-	FOR_ALL_pki(pki, pki_x509super) { pki->delRefKey(delkey); }
-}
-
-void db_x509super::newKey(pki_key *newkey)
-{
-	 FOR_ALL_pki(pki,pki_x509super) { pki->setRefKey(newkey); }
-}
-
 pki_key *db_x509super::findKey(pki_x509super *ref)
 {
 	pki_key *key, *refkey;
@@ -123,10 +129,15 @@ void db_x509super::extractPubkey(QModelIndex index)
 	if (!key)
 		return;
 	key->setIntName(pki->getIntName());
+	key->pkiSource = transformed;
+	key->selfComment(tr("Extracted from %1 '%2'")
+		.arg(pki->getType() == x509 ?
+			tr("Certificate") : tr("Certificate request"))
+		.arg(pki->getIntName()));
 	key = (pki_key*)mainwin->keys->insert(key);
 	if (!key)
 		return;
-	if (pki_base::suppress_messages)
+	if (Settings["suppress_messages"])
 		return;
 	XCA_INFO(key->getMsg(pki_base::msg_import).arg(pki->getIntName()));
 }
@@ -134,7 +145,7 @@ void db_x509super::extractPubkey(QModelIndex index)
 void db_x509super::toOpenssl(QModelIndex index) const
 {
 	pki_x509super *pki = static_cast<pki_x509super*>(index.internalPointer());
-	QString fn = mainwin->getPath() + QDir::separator() +
+	QString fn = Settings["workingdir"] + QDir::separator() +
 		pki->getUnderlinedName() + ".conf";
 	QString fname = QFileDialog::getSaveFileName(mainwin,
 		tr("Save as OpenSSL config"),	fn,
@@ -142,7 +153,7 @@ void db_x509super::toOpenssl(QModelIndex index) const
 	if (fname.isEmpty())
 		return;
 	fname = nativeSeparator(fname);
-	mainwin->setPath(fname.mid(0, fname.lastIndexOf(QRegExp("[/\\\\]")) ));
+	Settings["workingdir"] = fname.mid(0, fname.lastIndexOf(QRegExp("[/\\\\]")));
 	pki->opensslConf(fname);
 }
 
@@ -158,21 +169,23 @@ void db_x509super::toTemplate(QModelIndex index)
 		temp->setIntName(pki->getIntName());
 		extList el = temp->fromCert(pki);
 		if (el.size()) {
-			Ui::About ui;
 			QString etext;
-		        QDialog *d = new QDialog(mainwin, 0);
-		        ui.setupUi(d);
 			etext = QString("<h3>") +
 				tr("The following extensions were not ported into the template") +
 				QString("</h3><hr>") +
 				el.getHtml("<br>");
-			ui.textbox->setHtml(etext);
-			d->setWindowTitle(XCA_TITLE);
-			ui.image->setPixmap(*MainWindow::tempImg);
-			ui.image1->setPixmap(*MainWindow::certImg);
+			QTextEdit *textbox = new QTextEdit(etext);
+		        XcaDialog *d = new XcaDialog(mainwin, x509, textbox,
+						QString(), QString());
+			d->aboutDialog(MainWindow::tempImg);
 		        d->exec();
 		        delete d;
 		}
+		temp->pkiSource = transformed;
+		temp->selfComment(tr("Transformed from %1 '%2'")
+			.arg(pki->getType() == x509 ?
+				tr("Certificate") : tr("Certificate request"))
+			.arg(pki->getIntName()));
 		createSuccess(mainwin->temps->insert(temp));
 	}
 	catch (errorEx &err) {
@@ -180,3 +193,38 @@ void db_x509super::toTemplate(QModelIndex index)
 	}
 }
 
+void db_x509super::showPki(pki_base *pki)
+{
+	pki_x509super *x = dynamic_cast<pki_x509super *>(pki);
+	if (!x)
+		return;
+	CertDetail *dlg = new CertDetail(mainwin);
+	if (!dlg)
+		return;
+
+	switch (x->getType()) {
+		case x509_req: dlg->setReq((pki_x509req*)x); break;
+		case x509: dlg->setCert((pki_x509*)x); break;
+		default:
+			delete dlg;
+			return;
+	}
+	connect(dlg->privKey, SIGNAL(doubleClicked(QString)),
+		mainwin->keys, SLOT(showItem(QString)));
+	connect(dlg->signature, SIGNAL(doubleClicked(QString)),
+		this, SLOT(showItem(QString)));
+	connect(this, SIGNAL(pkiChanged(pki_base*)),
+		dlg, SLOT(itemChanged(pki_base*)));
+	connect(mainwin->keys, SIGNAL(pkiChanged(pki_base*)),
+		dlg, SLOT(itemChanged(pki_base*)));
+	if (dlg->exec()) {
+		QString newname = dlg->descr->text();
+		QString newcomment = dlg->comment->toPlainText();
+		if (newname != pki->getIntName() ||
+		    newcomment != pki->getComment())
+		{
+			updateItem(pki, newname, newcomment);
+		}
+	}
+	delete dlg;
+}
