@@ -10,6 +10,7 @@
 #include "exception.h"
 #include "lib/asn1time.h"
 #include "widgets/validity.h"
+#include "widgets/XcaWarning.h"
 #include <openssl/objects.h>
 #include <openssl/asn1.h>
 #include <openssl/err.h>
@@ -34,6 +35,7 @@
 #include <QPushButton>
 #include <QProgressBar>
 #include <QTextEdit>
+#include <QDebug>
 
 #if defined(Q_OS_WIN32)
 #include <shlobj.h>
@@ -265,10 +267,13 @@ QString asn1ToQString(const ASN1_STRING *str, bool quote)
 			qs = QString::fromLatin1((const char*)str->data, str->length);
 	}
 #if 0
-	printf("Convert %s (%d %d) string to '%s' len %d:", ASN1_tag2str(str->type), str->type, V_ASN1_UTF8STRING, CCHAR(qs), str->length);
+	QString s;
+	qDebug("Convert %s (%d %d) string to '%s' len %d:",
+		ASN1_tag2str(str->type), str->type,
+		V_ASN1_UTF8STRING, CCHAR(qs), str->length);
 	for (int i=0; i< str->length; i++)
-		printf(" %02x", str->data[i]);
-	printf("\n");
+		s += QString(" %1").arg(str->data[i], 2, 16);
+	qDebug() << s;
 #endif
 	if (quote)
 		qs.replace('\n', "\\n\\");
@@ -372,7 +377,7 @@ bool _ign_openssl_error(const QString txt, const char *file, int line)
 	QString errtxt;
 #if PRINT_IGNORED_ANYWAY
 	if (!txt.isEmpty() && ERR_peek_error())
-		fprintf(stderr, "%s\n", CCHAR(txt));
+		qDebug() << txt;
 #else
 	(void)txt;
 	(void)file;
@@ -381,13 +386,22 @@ bool _ign_openssl_error(const QString txt, const char *file, int line)
 	while (int i = ERR_get_error() ) {
 		errtxt = ERR_error_string(i, NULL);
 #if PRINT_IGNORED_ANYWAY
-		fprintf(stderr, CCHAR(QString("IGNORED (%1:%2) : %3\n").
-			arg(file).arg(line).arg(errtxt)));
+		qDebug() << QString("IGNORED (%1:%2) : %3\n")
+				.arg(file).arg(line).arg(errtxt);
 #endif
 	}
 	return !errtxt.isEmpty();
 }
 
+QString formatHash(const unsigned char *md, unsigned size, bool colon)
+{
+	QString s, t;
+	for (unsigned j = 0; j < size; j++)
+		s += t.sprintf("%02X%s", md[j],
+				(j+1 == size) || !colon ? "" : ":");
+	return s;
+}
+
 void inc_progress_bar(int, int, void *p)
 {
 	QProgressBar *bar = (QProgressBar *)p;
@@ -400,7 +414,6 @@ void inc_progress_bar(int, int, void *p)
 	}
 }
 
-bool translate_dn = false;
 QMap<int, QString> dn_translations;
 
 void dn_translations_setup()
@@ -445,3 +458,10 @@ void dn_translations_setup()
 	dn_translations[NID_netscape_ssl_server_name] = QObject::tr("SSL server name");
 	dn_translations[NID_netscape_comment] = QObject::tr("Comment");
 }
+
+QString appendXcaComment(QString current, QString msg)
+{
+	if (!current.endsWith("\n") && !current.isEmpty())
+		current += "\n";
+	return current + QString("(%1)\n").arg(msg);
+}
