@@ -9,6 +9,7 @@
 #include "pki_crl.h"
 #include "func.h"
 #include "exception.h"
+#include "db_base.h"
 #include <QDir>
 
 #include "openssl_compat.h"
@@ -20,9 +21,7 @@ pki_crl::pki_crl(const QString name )
 {
 	issuer = NULL;
 	crl = X509_CRL_new();
-	class_name="pki_crl";
 	pki_openssl_error();
-	dataVersion=1;
 	pkiType=revocation;
 }
 
@@ -36,7 +35,7 @@ void pki_crl::fromPEM_BIO(BIO *bio, QString name)
 	setIntName(rmslashdot(name));
 }
 
-QString pki_crl::getMsg(msg_type msg)
+QString pki_crl::getMsg(msg_type msg) const
 {
 	/*
 	 * We do not construct english sentences from fragments
@@ -54,6 +53,59 @@ QString pki_crl::getMsg(msg_type msg)
 	return pki_base::getMsg(msg);
 }
 
+QSqlError pki_crl::insertSqlData()
+{
+	XSqlQuery q;
+	unsigned name_hash = getSubject().hashNum();
+
+	SQL_PREPARE(q, "SELECT x509super.item FROM x509super "
+		"JOIN certs ON certs.item = x509super.item "
+		"WHERE x509super.subj_hash=? AND certs.ca=1");
+	q.bindValue(0, name_hash);
+	q.exec();
+	if (q.lastError().isValid())
+		return q.lastError();
+	while (q.next()) {
+		pki_x509 *x = db_base::lookupPki<pki_x509>(q.value(0));
+		if (!x) {
+			qDebug("CA certificate with id %d not found",
+				q.value(0).toInt());
+			continue;
+		}
+		verify(x);
+	}
+	SQL_PREPARE(q, "INSERT INTO crls (item, hash, num, iss_hash, issuer, crl) "
+		  "VALUES (?, ?, ?, ?, ?, ?)");
+	q.bindValue(0, sqlItemId);
+	q.bindValue(1, hash());
+	q.bindValue(2, numRev());
+	q.bindValue(3, name_hash);
+	q.bindValue(4, issuer ? issuer->getSqlItemId() : QVariant());
+	q.bindValue(5, i2d_b64());
+	q.exec();
+	return q.lastError();
+}
+
+void pki_crl::restoreSql(const QSqlRecord &rec)
+{
+	pki_base::restoreSql(rec);
+	QByteArray ba = QByteArray::fromBase64(
+				rec.value(VIEW_crls_crl).toByteArray());
+	d2i(ba);
+	setIssuer(db_base::lookupPki<pki_x509>(rec.value(VIEW_crls_issuer)));
+}
+
+QSqlError pki_crl::deleteSqlData()
+{
+	XSqlQuery q;
+	QSqlError e;
+
+	SQL_PREPARE(q, "DELETE FROM crls WHERE item=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	return q.lastError();
+}
+
 void pki_crl::fload(const QString fname)
 {
 	FILE *fp = fopen_read(fname);
@@ -79,7 +131,7 @@ void pki_crl::fload(const QString fname)
 		fopen_error(fname);
 }
 
-QString pki_crl::getSigAlg()
+QString pki_crl::getSigAlg() const
 {
 	return QString(OBJ_nid2ln(X509_CRL_get_signature_nid(crl)));
 }
@@ -132,7 +184,7 @@ void pki_crl::d2i(QByteArray &ba)
 	pki_openssl_error();
 }
 
-QByteArray pki_crl::i2d()
+QByteArray pki_crl::i2d() const
 {
 	return i2d_bytearray(I2D_VOID(i2d_X509_CRL), crl);
 }
@@ -151,13 +203,6 @@ void pki_crl::fromData(const unsigned char *p, db_header_t *head)
 	}
 }
 
-QByteArray pki_crl::toData()
-{
-	QByteArray ba = i2d();
-	pki_openssl_error();
-	return ba;
-}
-
 void pki_crl::addRev(const x509rev &xrev, bool withReason)
 {
 	X509_CRL_add0_revoked(crl, xrev.get(withReason));
@@ -180,7 +225,7 @@ extList pki_crl::extensions() const
 	return el;
 }
 
-bool pki_crl::visible()
+bool pki_crl::visible() const
 {
 	if (pki_x509name::visible())
 		return true;
@@ -229,17 +274,17 @@ BIO *pki_crl::pem(BIO *b, int format)
 	return b;
 }
 
-a1time pki_crl::getLastUpdate()
+a1time pki_crl::getLastUpdate() const
 {
 	return a1time(X509_CRL_get0_lastUpdate(crl));
 }
 
-a1time pki_crl::getNextUpdate()
+a1time pki_crl::getNextUpdate() const
 {
 	return a1time(X509_CRL_get0_nextUpdate(crl));
 }
 
-int pki_crl::numRev()
+int pki_crl::numRev() const
 {
 	STACK_OF(X509_REVOKED) *st = X509_CRL_get_REVOKED(crl);
 
@@ -265,14 +310,27 @@ x509name pki_crl::getSubject() const
 	return x509name(X509_CRL_get_issuer(crl));
 }
 
-bool pki_crl::verify(pki_key *key)
+bool pki_crl::verify(pki_x509 *issuer)
 {
-	bool ret=false;
-	if (key) {
-		ret = (X509_CRL_verify(crl, key->getPubKey()) == 1);
-		pki_ign_openssl_error();
+	if (!issuer)
+		return false;
+	if (getSubject() != issuer->getSubject())
+		return false;
+	pki_key *key = issuer->getPubKey();
+	if (!key)
+		return false;
+	int ret = X509_CRL_verify(crl, key->getPubKey());
+	pki_ign_openssl_error();
+	if (ret != 1) {
+		delete key;
+		return false;
 	}
-	return ret;
+	delete key;
+	pki_x509 *curr = getIssuer();
+	if (curr && curr->getNotAfter() > issuer->getNotAfter())
+		return true;
+	setIssuer(issuer);
+	return true;
 }
 
 void pki_crl::setCrlNumber(a1int num)
@@ -284,7 +342,7 @@ void pki_crl::setCrlNumber(a1int num)
 	pki_openssl_error();
 }
 
-a1int pki_crl::getCrlNumber()
+a1int pki_crl::getCrlNumber() const
 {
 	a1int num;
 	if (!getCrlNumber(&num))
@@ -292,7 +350,7 @@ a1int pki_crl::getCrlNumber()
 	return num;
 }
 
-bool pki_crl::getCrlNumber(a1int *num)
+bool pki_crl::getCrlNumber(a1int *num) const
 {
 	int j;
 	ASN1_INTEGER *i;
@@ -323,12 +381,12 @@ QString pki_crl::printV3ext()
 	return text;
 }
 
-QVariant pki_crl::column_data(dbheader *hd)
+QVariant pki_crl::column_data(const dbheader *hd) const
 {
 	switch (hd->id) {
 		case HD_crl_signer:
 			if (issuer)
-				return QVariant(getIssuer()->getIntName());
+				return QVariant(getIssuerName());
 			else
 				return QVariant(tr("unknown"));
 		case HD_crl_revoked:
@@ -346,17 +404,7 @@ QVariant pki_crl::column_data(dbheader *hd)
 	return pki_x509name::column_data(hd);
 }
 
-QVariant pki_crl::getIcon(dbheader *hd)
+QVariant pki_crl::getIcon(const dbheader *hd) const
 {
 	return hd->id == HD_internal_name ? QVariant(*icon) : QVariant();
 }
-
-void pki_crl::oldFromData(unsigned char *p, int size)
-{
-	QByteArray ba((const char *)p, size);
-	d2i(ba);
-
-	if (ba.count() > 0) {
-		my_error(tr("Wrong Size %1").arg(ba.count()));
-	}
-}
