@@ -16,6 +16,10 @@
 #include "asn1time.h"
 #include "asn1int.h"
 
+#define VIEW_crls_num 6
+#define VIEW_crls_issuer 7
+#define VIEW_crls_crl 8
+
 class pki_crl: public pki_x509name
 {
 		Q_OBJECT
@@ -26,11 +30,10 @@ class pki_crl: public pki_x509name
 		extList extensions() const;
 	public:
 		pki_crl(const QString name = "");
-		/* destructor */
 		~pki_crl();
 		void fromPEM_BIO(BIO *bio, QString name);
 		void fload(const QString fname);
-		QString getSigAlg();
+		QString getSigAlg() const;
 		void writeDefault(const QString fname);
 		static QPixmap *icon;
 		void createCrl(const QString d, pki_x509 *iss);
@@ -44,6 +47,10 @@ class pki_crl: public pki_x509name
 		{
 			return issuer;
 		}
+		QString getIssuerName() const
+		{
+			return issuer->getIntName();
+		}
 		void setIssuer(pki_x509 *iss)
 		{
 			 issuer = iss;
@@ -51,27 +58,28 @@ class pki_crl: public pki_x509name
 		x509name getSubject() const;
 		void setLastUpdate(const a1time &t);
 		void setNextUpdate(const a1time &t);
-		a1time getNextUpdate();
-		a1time getLastUpdate();
+		a1time getNextUpdate() const;
+		a1time getLastUpdate() const;
 		void fromData(const unsigned char *p, db_header_t *head);
-		void oldFromData(unsigned char *p, int size);
-		QByteArray toData();
-		bool verify(pki_key *pkey);
-		int numRev();
+		bool verify(pki_x509 *issuer);
+		int numRev() const;
 		x509revList getRevList();
 		QString printV3ext();
 		x509v3ext getExtByNid(int nid);
 		a1int getVersion();
-		QVariant column_data(dbheader *hd);
-		QVariant getIcon(dbheader *hd);
-		virtual QString getMsg(msg_type msg);
+		QVariant column_data(const dbheader *hd) const;
+		QVariant getIcon(const dbheader *hd) const;
+		QString getMsg(msg_type msg) const;
 		void d2i(QByteArray &ba);
-		QByteArray i2d();
+		QByteArray i2d() const;
 		void setCrlNumber(a1int num);
-		bool getCrlNumber(a1int *num);
-		a1int getCrlNumber();
+		bool getCrlNumber(a1int *num) const;
+		a1int getCrlNumber() const;
 		BIO *pem(BIO *, int);
-		bool visible();
+		bool visible() const;
+		QSqlError insertSqlData();
+		QSqlError deleteSqlData();
+		void restoreSql(const QSqlRecord &rec);
 };
 
 #endif
