@@ -8,7 +8,6 @@
 
 #include "pki_evp.h"
 #include "pass_info.h"
-#include "Passwd.h"
 #include "func.h"
 #include "db.h"
 #include "entropy.h"
@@ -17,6 +16,7 @@
 #include <openssl/rand.h>
 #include <openssl/evp.h>
 #include <openssl/pem.h>
+#include <openssl/pkcs12.h>
 
 #include <QProgressDialog>
 #include <QApplication>
@@ -33,24 +33,14 @@ QPixmap *pki_evp::icon[2]= { NULL, NULL };
 
 void pki_evp::init()
 {
-	class_name = "pki_evp";
 	ownPass = ptCommon;
-	dataVersion=2;
-	pkiType=asym_key;
-}
-
-QString pki_evp::removeTypeFromIntName(QString n)
-{
-	if (n.right(1) != ")" )
-		return n;
-	n.truncate(n.length() - 6);
-	return n;
+	pkiType = asym_key;
 }
 
 void pki_evp::setOwnPass(enum passType x)
 {
 	EVP_PKEY *pk=NULL, *pk_back = key;
-	int oldOwnPass = ownPass;
+	enum passType oldOwnPass = ownPass;
 
 	if (ownPass == x || isPubKey())
 		return;
@@ -71,7 +61,30 @@ void pki_evp::setOwnPass(enum passType x)
 		ownPass = oldOwnPass;
 		throw(err);
 	}
-	EVP_PKEY_free(pk_back);
+}
+
+bool pki_evp::sqlUpdatePrivateKey()
+{
+	Transaction;
+	if (!TransBegin())
+		return false;
+	XSqlQuery q;
+	SQL_PREPARE(q, "UPDATE private_keys SET private=?, ownPass=? "
+		"WHERE item=?");
+	q.bindValue(0, encKey_b64());
+	q.bindValue(1, ownPass);
+	q.bindValue(2, sqlItemId);
+	AffectedItems(sqlItemId);
+	q.exec();
+
+	encKey.fill(0);
+	encKey.clear();
+
+	if (!q.lastError().isValid() && q.numRowsAffected() == 1) {
+		TransCommit();
+		return true;
+	}
+	return false;
 }
 
 void pki_evp::generate(int bits, int type, QProgressBar *progress, int curve_nid)
@@ -141,7 +154,8 @@ void pki_evp::generate(int bits, int type, QProgressBar *progress, int curve_nid
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	BN_GENCB_free(bar);
 #endif
-
+	isPub = false;
+	pkiSource = generated;
 	pki_openssl_error();
 	encryptKey();
 }
@@ -152,7 +166,8 @@ pki_evp::pki_evp(const pki_evp *pk)
 	init();
 	pki_openssl_error();
 	ownPass = pk->ownPass;
-	encKey = pk->encKey;
+	isPub = pk->isPub;
+	encKey = pk->getEncKey();
 }
 
 pki_evp::pki_evp(const QString name, int type)
@@ -163,16 +178,6 @@ pki_evp::pki_evp(const QString name, int type)
 	pki_openssl_error();
 }
 
-pki_evp::pki_evp(EVP_PKEY *pkey)
-	:pki_key()
-{
-	init();
-	if (key) {
-		EVP_PKEY_free(key);
-	}
-	key = pkey;
-}
-
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
 static bool EVP_PKEY_isPrivKey(EVP_PKEY *key)
 {
@@ -217,6 +222,13 @@ static bool EVP_PKEY_isPrivKey(EVP_PKEY *key)
 }
 #endif
 
+pki_evp::pki_evp(EVP_PKEY *pkey)
+	:pki_key()
+{
+	init();
+	set_EVP_PKEY(pkey);
+}
+
 void pki_evp::openssl_pw_error(QString fname)
 {
 	switch (ERR_peek_error() & 0xff000fff) {
@@ -225,10 +237,24 @@ void pki_evp::openssl_pw_error(QString fname)
 	case ERR_PACK(ERR_LIB_EVP, 0, EVP_R_BAD_DECRYPT):
 		pki_ign_openssl_error();
 		throw errorEx(tr("Failed to decrypt the key (bad password) ")+
-				fname, class_name, E_PASSWD);
+				fname, getClassName(), E_PASSWD);
 	}
 }
 
+void pki_evp::set_EVP_PKEY(EVP_PKEY *pkey)
+{
+	if (!pkey)
+		return;
+	if (key)
+		EVP_PKEY_free(key);
+	key = pkey;
+	isPub = !EVP_PKEY_isPrivKey(key);
+	if (!isPub) {
+		bogusEncryptKey();
+	}
+	pki_openssl_error();
+}
+
 void pki_evp::fromPEMbyteArray(QByteArray &ba, QString name)
 {
 	BIO *bio = BIO_new_mem_buf(ba.data(), ba.length());
@@ -245,15 +271,9 @@ void pki_evp::fromPEMbyteArray(QByteArray &ba, QString name)
 		pkey = PEM_read_bio_PUBKEY(bio, NULL, PwDialog::pwCallback, &p);
 	}
 	BIO_free(bio);
-	if (pkey){
-		if (key)
-			EVP_PKEY_free(key);
-		key = pkey;
-		if (EVP_PKEY_isPrivKey(key))
-			bogusEncryptKey();
-		setIntName(rmslashdot(name));
-	}
-	openssl_error(name);
+
+	setIntName(rmslashdot(name));
+	set_EVP_PKEY(pkey);
 }
 
 static void search_ec_oid(EVP_PKEY *pkey)
@@ -383,7 +403,7 @@ void pki_evp::fromData(const unsigned char *p, db_header_t *head)
 	QByteArray ba((const char*)p, size);
 
 	type = db::intFromData(ba);
-	ownPass = db::intFromData(ba);
+	ownPass = (enum passType)db::intFromData(ba);
 	if (version < 2) {
 		d2i_old(ba, type);
 	} else {
@@ -397,9 +417,65 @@ void pki_evp::fromData(const unsigned char *p, db_header_t *head)
 		throw errorEx(tr("Ignoring unsupported private key"));
 
 	encKey = ba;
+	isPub = encKey.size() == 0;
 }
 
 EVP_PKEY *pki_evp::decryptKey() const
+{
+	Passwd ownPassBuf;
+	int ret;
+
+	if (isPubKey()) {
+		QByteArray ba = i2d_bytearray(I2D_VOID(i2d_PUBKEY), key);
+		return (EVP_PKEY*)d2i_bytearray(D2I_VOID(d2i_PUBKEY), ba);
+	}
+	/* This key has its own password */
+	if (ownPass == ptPrivate) {
+		pass_info pi(XCA_TITLE, tr("Please enter the password to decrypt the private key: '%1'").arg(getIntName()));
+		ret = PwDialog::execute(&pi, &ownPassBuf, false);
+		if (ret != 1)
+			throw errorEx(tr("Password input aborted"),
+					getClassName());
+	} else if (ownPass == ptBogus) { // BOGUS pass
+		ownPassBuf = "Bogus";
+	} else {
+		ownPassBuf = passwd;
+		while (sha512passwT(ownPassBuf, passHash) != passHash &&
+			sha512passwd(ownPassBuf, passHash) != passHash)
+		{
+			pass_info p(XCA_TITLE, tr("Please enter the database password for decrypting the key '%1'").arg(getIntName()));
+			ret = PwDialog::execute(&p, &ownPassBuf, false);
+			if (ret != 1)
+				throw errorEx(tr("Password input aborted"),
+						getClassName());
+		}
+	}
+	QByteArray myencKey = getEncKey();
+	qDebug() << "myencKey.count()"<<myencKey.count();
+	if (myencKey.count() == 0)
+		return NULL;
+	BIO *b = BIO_from_QByteArray(myencKey);
+	check_oom(b);
+	EVP_PKEY *priv = NULL;
+	X509_SIG *p8 = d2i_PKCS8_bio(b, NULL);
+	if (p8) {
+		PKCS8_PRIV_KEY_INFO *p8inf = PKCS8_decrypt(p8,
+				ownPassBuf.constData(), ownPassBuf.size());
+		if (p8inf) {
+			priv = EVP_PKCS82PKEY(p8inf);
+			PKCS8_PRIV_KEY_INFO_free(p8inf);
+		}
+		X509_SIG_free(p8);
+	}
+	BIO_free(b);
+	if (priv)
+		return priv;
+	pki_ign_openssl_error();
+	return legacyDecryptKey(myencKey, ownPassBuf);
+}
+
+EVP_PKEY *pki_evp::legacyDecryptKey(QByteArray &myencKey,
+				    Passwd &ownPassBuf) const
 {
 	unsigned char *p;
 	const unsigned char *p1;
@@ -413,54 +489,16 @@ EVP_PKEY *pki_evp::decryptKey() const
 #endif
 	EVP_CIPHER_CTX *ctx;
 	const EVP_CIPHER *cipher = EVP_des_ede3_cbc();
-	Passwd ownPassBuf;
-	int ret;
-
-	if (isPubKey()) {
-		unsigned char *q;
-		outl = i2d_PUBKEY(key, NULL);
-		p = q = (unsigned char *)OPENSSL_malloc(outl);
-		check_oom(q);
-		i2d_PUBKEY(key, &p);
-		p = q;
-		tmpkey = d2i_PUBKEY(NULL, (const unsigned char**)&p, outl);
-		OPENSSL_free(q);
-		return tmpkey;
-	}
-	/* This key has its own password */
-	if (ownPass == ptPrivate) {
-		pass_info pi(XCA_TITLE, tr("Please enter the password to decrypt the private key: '%1'").arg(getIntName()));
-		ret = PwDialog::execute(&pi, &ownPassBuf, false);
-		if (ret != 1)
-			throw errorEx(tr("Password input aborted"), class_name);
-	} else if (ownPass == ptBogus) { // BOGUS pass
-		ownPassBuf = "Bogus";
-	} else {
-		ownPassBuf = passwd;
-		while (md5passwd(ownPassBuf) != passHash &&
-			sha512passwd(ownPassBuf, passHash) != passHash)
-		{
-			pass_info p(XCA_TITLE, tr("Please enter the database password for decrypting the key '%1'").arg(getIntName()));
-			ret = PwDialog::execute(&p, &ownPassBuf, false);
-			if (ret != 1)
-				throw errorEx(tr("Password input aborted"), class_name);
-		}
-	}
-	p = (unsigned char *)OPENSSL_malloc(encKey.count());
+	p = (unsigned char *)OPENSSL_malloc(myencKey.count());
 	check_oom(p);
 	pki_openssl_error();
 	p1 = p;
 	memset(iv, 0, EVP_MAX_IV_LENGTH);
 
-	memcpy(iv, encKey.constData(), 8); /* recover the iv */
+	memcpy(iv, myencKey.constData(), 8); /* recover the iv */
 	/* generate the key */
 	EVP_BytesToKey(cipher, EVP_sha1(), iv,
-		ownPassBuf.constUchar(),
-		ownPassBuf.size(), 1, ckey, NULL);
-	/* we use sha1 as message digest,
-	 * because an md5 version of the password is
-	 * stored in the database...
-	 */
+		ownPassBuf.constUchar(), ownPassBuf.size(), 1, ckey, NULL);
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	ctx = EVP_CIPHER_CTX_new();
 #else
@@ -469,15 +507,16 @@ EVP_PKEY *pki_evp::decryptKey() const
 	EVP_CIPHER_CTX_init(ctx);
 	EVP_DecryptInit(ctx, cipher, ckey, iv);
 	EVP_DecryptUpdate(ctx, p , &outl,
-		(const unsigned char*)encKey.constData() +8, encKey.count() -8);
-
+		(const unsigned char*)myencKey.constData() +8,
+		myencKey.count() -8);
 	decsize = outl;
-	EVP_DecryptFinal(ctx, p + decsize , &outl);
+	EVP_DecryptFinal_ex(ctx, p + decsize , &outl);
+	EVP_CIPHER_CTX_cleanup(ctx);
 	decsize += outl;
-	//printf("Decrypt decsize=%d, encKey_len=%d\n", decsize, encKey.count() -8);
 	pki_openssl_error();
 	tmpkey = d2i_PrivateKey(getKeyType(), NULL, &p1, decsize);
 	pki_openssl_error();
+	OPENSSL_cleanse(p, myencKey.count());
 	OPENSSL_free(p);
 	EVP_CIPHER_CTX_cleanup(ctx);
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
@@ -488,20 +527,10 @@ EVP_PKEY *pki_evp::decryptKey() const
 		RSA *rsa = EVP_PKEY_get0_RSA(tmpkey);
 		RSA_blinding_on(rsa, NULL);
 	}
+	myencKey.fill(0);
 	return tmpkey;
 }
 
-QByteArray pki_evp::toData()
-{
-	QByteArray ba;
-
-	ba += db::intToData(getKeyType());
-	ba += db::intToData(ownPass);
-	ba += i2d();
-	ba += encKey;
-	return ba;
-}
-
 EVP_PKEY *pki_evp::priv2pub(EVP_PKEY* key)
 {
 	int keylen;
@@ -524,17 +553,9 @@ EVP_PKEY *pki_evp::priv2pub(EVP_PKEY* key)
 
 void pki_evp::encryptKey(const char *password)
 {
-	int outl, keylen;
-	EVP_PKEY *pkey1 = NULL;
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-	EVP_CIPHER_CTX ctxbuf;
-#endif
-	EVP_CIPHER_CTX *ctx;
-	const EVP_CIPHER *cipher = EVP_des_ede3_cbc();
-	unsigned char iv[EVP_MAX_IV_LENGTH], *punenc, *punenc1;
-	unsigned char ckey[EVP_MAX_KEY_LENGTH];
 	Passwd ownPassBuf;
 
+	pki_openssl_error();
 	/* This key has its own, private password */
 	if (ownPass == ptPrivate) {
 		int ret;
@@ -542,84 +563,50 @@ void pki_evp::encryptKey(const char *password)
 			arg(getIntName()));
 		ret = PwDialog::execute(&p, &ownPassBuf, true);
 		if (ret != 1)
-			throw errorEx("Password input aborted", class_name);
+			throw errorEx("Password input aborted", getClassName());
+	pki_openssl_error();
 	} else if (ownPass == ptBogus) { // BOGUS password
 		ownPassBuf = "Bogus";
+	pki_openssl_error();
 	} else {
 		if (password) {
 			/* use the password parameter
 			 * if this is a common password */
 			ownPassBuf = password;
+	pki_openssl_error();
 		} else {
 			int ret = 0;
 			ownPassBuf = passwd;
 			pass_info p(XCA_TITLE, tr("Please enter the database password for encrypting the key"));
-			while (md5passwd(ownPassBuf) != passHash &&
-				sha512passwd(ownPassBuf, passHash) != passHash )
+			while (sha512passwT(ownPassBuf, passHash) != passHash &&
+				sha512passwd(ownPassBuf, passHash) != passHash)
 			{
 				ret = PwDialog::execute(&p, &ownPassBuf, false);
 				if (ret != 1)
-					throw errorEx("Password input aborted", class_name);
+					throw errorEx("Password input aborted",
+							getClassName());
 			}
 		}
 	}
 
-	/* Prepare Encryption */
-	memset(iv, 0, EVP_MAX_IV_LENGTH);
-	Entropy::get(iv, 8);      /* Generate a salt */
-	EVP_BytesToKey(cipher, EVP_sha1(), iv,
-			ownPassBuf.constUchar(),
-			ownPassBuf.size(), 1, ckey, NULL);
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-	ctx = EVP_CIPHER_CTX_new();
-#else
-	ctx = &ctxbuf;
-#endif
-	EVP_CIPHER_CTX_init (ctx);
+	/* Convert private key to DER(PKCS8-aes) */
+	const char *p;
+	BIO *bio = BIO_new(BIO_s_mem());
+	i2d_PKCS8PrivateKey_bio(bio, key, EVP_aes_256_cbc(),
+		ownPassBuf.data(), ownPassBuf.size(), NULL, 0);
 	pki_openssl_error();
+	int l = BIO_get_mem_data(bio, &p);
+	encKey = QByteArray(p, l);
+	BIO_free(bio);
 
-	/* reserve space for unencrypted and encrypted key */
-	keylen = i2d_PrivateKey(key, NULL);
-	encKey.resize(keylen + EVP_MAX_KEY_LENGTH + 8);
-	punenc1 = punenc = (unsigned char *)OPENSSL_malloc(keylen);
-	check_oom(punenc);
-	keylen = i2d_PrivateKey(key, &punenc1);
-	pki_openssl_error();
-
-	memcpy(encKey.data(), iv, 8); /* store the iv */
-	/*
-	 * Now DER version of privkey is in punenc
-	 * and privkey is still in key
+	/* Replace private key by public key and
+	   have the encrypted private in "encKey"
 	 */
-
-	/* do the encryption */
-	/* store key right after the iv */
-	EVP_EncryptInit(ctx, cipher, ckey, iv);
-	unsigned char *penc = (unsigned char *)encKey.data() +8;
-	EVP_EncryptUpdate(ctx, penc, &outl, punenc, keylen);
-	int encKey_len = outl;
-	EVP_EncryptFinal(ctx, penc + encKey_len, &outl);
-	encKey.resize(encKey_len + outl +8);
-	/* Cleanup */
-	EVP_CIPHER_CTX_cleanup(ctx);
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-	EVP_CIPHER_CTX_free(ctx);
-#endif
-	/* wipe out the memory */
-	memset(punenc, 0, keylen);
-	OPENSSL_free(punenc);
-	pki_openssl_error();
-
-	pkey1 = priv2pub(key);
+	EVP_PKEY *pkey1 = priv2pub(key);
 	check_oom(pkey1);
 	EVP_PKEY_free(key);
 	key = pkey1;
 	pki_openssl_error();
-
-	//CRYPTO_mem_ctrl(CRYPTO_MEM_CHECK_OFF);
-
-	//printf("Encrypt: encKey_len=%d\n", encKey_len);
-	return;
 }
 
 void pki_evp::set_evp_key(EVP_PKEY *pkey)
@@ -632,20 +619,79 @@ void pki_evp::set_evp_key(EVP_PKEY *pkey)
 void pki_evp::bogusEncryptKey()
 {
 	ownPass = ptBogus;
+	isPub = false;
 	encryptKey();
 }
 
 pki_evp::~pki_evp()
 {
+	encKey.fill(0);
 }
 
+QSqlError pki_evp::insertSqlData()
+{
+	XSqlQuery q;
+	QSqlError e = pki_key::insertSqlData();
+	if (e.isValid())
+		return e;
+	if (isPubKey())
+		return QSqlError();
+
+	SQL_PREPARE(q, "INSERT INTO private_keys (item, ownPass, private) "
+		  "VALUES (?, ?, ?)");
+	q.bindValue(0, sqlItemId);
+	q.bindValue(1, ownPass);
+	q.bindValue(2, encKey_b64());
+	q.exec();
+	encKey.fill(0);
+	encKey.clear();
+	return q.lastError();
+}
+
+void pki_evp::restoreSql(const QSqlRecord &rec)
+{
+	pki_key::restoreSql(rec);
+	isPub = rec.isNull(VIEW_private_ownpass);
+	if (!isPub)
+		ownPass =(enum passType)rec.value(VIEW_private_ownpass).toInt();
+}
+
+QByteArray pki_evp::getEncKey() const
+{
+	XSqlQuery q;
+	QSqlError e;
+	QByteArray ba;
+
+	if (encKey.count() > 0 || !sqlItemId.isValid())
+		return encKey;
+
+	SQL_PREPARE(q, "SELECT private FROM private_keys WHERE item=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	e = q.lastError();
+	if (e.isValid() || !q.first())
+		return QByteArray();
+	return QByteArray::fromBase64(q.value(0).toByteArray().trimmed());
+}
+
+QSqlError pki_evp::deleteSqlData()
+{
+	XSqlQuery q;
+	QSqlError e = pki_key::deleteSqlData();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "DELETE FROM private_keys WHERE item=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	return q.lastError();
+}
 
 void pki_evp::writePKCS8(const QString fname, const EVP_CIPHER *enc,
 		pem_password_cb *cb, bool pem)
 {
 	EVP_PKEY *pkey;
 	pass_info p(XCA_TITLE, tr("Please enter the password protecting the PKCS#8 key '%1'").arg(getIntName()));
-	FILE *fp = fopen_write(fname);
+	FILE *fp = fopen_write_key(fname);
 	if (fp != NULL) {
 		if (key) {
 			pkey = decryptKey();
@@ -671,70 +717,68 @@ static int mycb(char *buf, int size, int, void *)
 
 void pki_evp::writeDefault(const QString fname)
 {
-	writeKey(fname + QDir::separator() + getIntName() + ".pem",
+	writeKey(get_dump_filename(fname, ".pem"),
 		pki_evp::passwd[0] ? EVP_des_ede3_cbc() : NULL,
 		mycb, true);
 }
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined LIBRESSL_VERSION_NUMBER
+int PEM_write_bio_PrivateKey_traditional(BIO *bp, EVP_PKEY *x,
+                                         const EVP_CIPHER *enc,
+                                         unsigned char *kstr, int klen,
+                                         pem_password_cb *cb, void *u)
+{
+	const char *t = "";
+	int keytype = EVP_PKEY_id(x);
+
+	switch (keytype) {
+		case EVP_PKEY_RSA: t = "RSA PRIVATE KEY"; break;
+		case EVP_PKEY_DSA: t = "DSA PRIVATE KEY"; break;
+#ifndef OPENSSL_NO_EC
+		case EVP_PKEY_EC: t = "EC PRIVATE KEY"; break;
+#endif
+	}
+	return PEM_ASN1_write_bio((i2d_of_void *)i2d_PrivateKey,
+				t, bp, (char*)x, enc, kstr, klen, cb, u);
+}
+#endif
+
 void pki_evp::writeKey(const QString fname, const EVP_CIPHER *enc,
 			pem_password_cb *cb, bool pem)
 {
-	EVP_PKEY *pkey;
-	int keytype;
 	pass_info p(XCA_TITLE, tr("Please enter the export password for the private key '%1'").arg(getIntName()));
+
 	if (isPubKey()) {
 		writePublic(fname, pem);
 		return;
 	}
-	FILE *fp = fopen_write(fname);
+	FILE *fp = fopen_write_key(fname);
 	if (!fp) {
 		fopen_error(fname);
 		return;
 	}
-	if (key){
-		pkey = decryptKey();
-		if (pkey) {
-			if (pem) {
-				keytype = EVP_PKEY_id(pkey);
-				switch (keytype) {
-				case EVP_PKEY_RSA:
-					PEM_write_RSAPrivateKey(fp,
-						EVP_PKEY_get0_RSA(pkey),
-						enc, NULL, 0, cb, &p);
-					break;
-				case EVP_PKEY_DSA:
-					PEM_write_DSAPrivateKey(fp,
-						EVP_PKEY_get0_DSA(pkey),
-						enc, NULL, 0, cb, &p);
-					break;
-#ifndef OPENSSL_NO_EC
-				case EVP_PKEY_EC:
-					PEM_write_ECPrivateKey(fp,
-						EVP_PKEY_get0_EC_KEY(pkey),
-						enc, NULL, 0, cb, &p);
-					break;
-#endif
-				default:
-					PEM_write_PrivateKey(fp,
-						pkey,
-						enc, NULL, 0, cb, &p);
-				}
-			} else {
-				i2d_PrivateKey_fp(fp, pkey);
-			}
+	EVP_PKEY *pkey = key ? decryptKey() : NULL;
+	if (!pkey) {
+		fclose(fp);
+	        pki_openssl_error();
+		return;
+	}
+	if (pem) {
+		BIO *b = BIO_new_fp(fp, BIO_NOCLOSE);
+		if (!b) {
 			EVP_PKEY_free(pkey);
+			fclose(fp);
+			return;
 		}
+		PEM_write_bio_PrivateKey_traditional(b, pkey, enc,
+						NULL, 0, cb, &p);
+		BIO_free(b);
+	} else {
+		i2d_PrivateKey_fp(fp, pkey);
 	}
 	fclose(fp);
 	pki_openssl_error();
-}
-
-bool pki_evp::isPubKey() const
-{
-	if (encKey.count() == 0) {
-		return true;
-	}
-	return false;
+	EVP_PKEY_free(pkey);
 }
 
 int pki_evp::verify()
@@ -752,25 +796,7 @@ int pki_evp::verify()
 	return veri;
 }
 
-const EVP_MD *pki_evp::getDefaultMD()
-{
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-	return EVP_sha1();
-#else
-	const EVP_MD *md;
-	switch (getKeyType()) {
-		case EVP_PKEY_RSA: md = EVP_sha1(); break;
-		case EVP_PKEY_DSA: md = EVP_dss1(); break;
-#ifndef OPENSSL_NO_EC
-		case EVP_PKEY_EC:  md = EVP_ecdsa(); break;
-#endif
-		default: md = NULL; break;
-	}
-	return md;
-#endif
-}
-
-QVariant pki_evp::getIcon(dbheader *hd)
+QVariant pki_evp::getIcon(const dbheader *hd) const
 {
 	if (hd->id != HD_internal_name)
 		return QVariant();
@@ -785,9 +811,7 @@ QString pki_evp::md5passwd(QByteArray pass)
 	EVP_MD_CTX mdctxbuf;
 #endif
 	EVP_MD_CTX *mdctx;
-	QString str;
 	int n;
-	int j;
 	unsigned char m[EVP_MAX_MD_SIZE];
 
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
@@ -803,16 +827,11 @@ QString pki_evp::md5passwd(QByteArray pass)
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	EVP_MD_CTX_free(mdctx);
 #endif
-
-	for (j=0; j<n; j++) {
-		char zs[4];
-		sprintf(zs, "%02X%c",m[j], (j+1 == n) ?'\0':':');
-		str += zs;
-	}
-	return str;
+	return formatHash(m, n);
 }
 
-QString pki_evp::sha512passwd(QByteArray pass, QString salt)
+QString pki_evp::_sha512passwd(QByteArray pass, QString salt,
+				int size, int repeat)
 {
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
 	EVP_MD_CTX mdctxbuf;
@@ -820,13 +839,12 @@ QString pki_evp::sha512passwd(QByteArray pass, QString salt)
 	EVP_MD_CTX *mdctx;
 	QString str;
 	int n;
-	int j;
 	unsigned char m[EVP_MAX_MD_SIZE];
 
-	if (salt.length() <5)
+	if (salt.length() < size) {
 		abort();
-
-	str = salt.left(5);
+	}
+	str = salt.left(size);
 	pass = str.toLatin1() + pass;
 
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
@@ -834,107 +852,25 @@ QString pki_evp::sha512passwd(QByteArray pass, QString salt)
 #else
 	mdctx = &mdctxbuf;
 #endif
+	while (repeat--) {
+		EVP_DigestInit(mdctx, EVP_sha512());
+		EVP_DigestUpdate(mdctx, pass.constData(), pass.size());
+		EVP_DigestFinal(mdctx, m, (unsigned*)&n);
+		pass = QByteArray((char*)m, n);
 
-	EVP_DigestInit(mdctx, EVP_sha512());
-	EVP_DigestUpdate(mdctx, pass.constData(), pass.size());
-	EVP_DigestFinal(mdctx, m, (unsigned*)&n);
-
+	}
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	EVP_MD_CTX_free(mdctx);
 #endif
-
-	for (j=0; j<n; j++) {
-		char zs[4];
-		sprintf(zs, "%02X",m[j]);
-		str += zs;
-	}
-	return str;
+	return str + formatHash(m, n, false);
 }
 
-void pki_evp::veryOldFromData(unsigned char *p, int size )
+QString pki_evp::sha512passwd(QByteArray pass, QString salt)
 {
-	unsigned char *sik, *pdec, *pdec1, *sik1;
-	int outl, decsize;
-	unsigned char iv[EVP_MAX_IV_LENGTH];
-	unsigned char ckey[EVP_MAX_KEY_LENGTH];
-	memset(iv, 0, EVP_MAX_IV_LENGTH);
-	RSA *rsakey;
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-	EVP_CIPHER_CTX ctxbuf;
-#endif
-	EVP_CIPHER_CTX *ctx;
-	const EVP_CIPHER *cipher = EVP_des_ede3_cbc();
-	sik = (unsigned char *)OPENSSL_malloc(size);
-	check_oom(sik);
-	pki_openssl_error();
-	pdec = (unsigned char *)OPENSSL_malloc(size);
-	if (pdec == NULL ) {
-		OPENSSL_free(sik);
-		check_oom(pdec);
-	}
-	pdec1=pdec;
-	sik1=sik;
-	memcpy(iv, p, 8); /* recover the iv */
-	/* generate the key */
-	EVP_BytesToKey(cipher, EVP_sha1(), iv, oldpasswd.constUchar(),
-		oldpasswd.size(), 1, ckey, NULL);
-	/* we use sha1 as message digest,
-	 * because an md5 version of the password is
-	 * stored in the database...
-	 */
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-	ctx = EVP_CIPHER_CTX_new();
-#else
-	ctx = &ctxbuf;
-#endif
-
-	EVP_CIPHER_CTX_init(ctx);
-	EVP_DecryptInit(ctx, cipher, ckey, iv);
-	EVP_DecryptUpdate(ctx, pdec , &outl, p + 8, size - 8);
-	decsize = outl;
-	EVP_DecryptFinal(ctx, pdec + decsize, &outl);
-	decsize += outl;
-	pki_openssl_error();
-	memcpy(sik, pdec, decsize);
-	if (getKeyType() == EVP_PKEY_RSA) {
-		rsakey=d2i_RSAPrivateKey(NULL,(const unsigned char **)&pdec, decsize);
-		if (pki_ign_openssl_error()) {
-			rsakey = d2i_RSA_PUBKEY(NULL, (const unsigned char **)&sik, decsize);
-		}
-		pki_openssl_error();
-		if (rsakey) EVP_PKEY_assign_RSA(key, rsakey);
-	}
-	OPENSSL_free(sik1);
-	OPENSSL_free(pdec1);
-	EVP_CIPHER_CTX_cleanup(ctx);
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-	EVP_CIPHER_CTX_free(ctx);
-#endif
-	pki_openssl_error();
-	encryptKey();
+	return _sha512passwd(pass, salt, 5, 1);
 }
 
-void pki_evp::oldFromData(unsigned char *p, int size )
+QString pki_evp::sha512passwT(QByteArray pass, QString salt)
 {
-	int version, type;
-
-	QByteArray ba;
-
-	version = intFromData(ba);
-	if (version != 1) { // backward compatibility
-		veryOldFromData(p, size);
-		return;
-	}
-	if (key)
-		EVP_PKEY_free(key);
-
-	key = NULL;
-	type = intFromData(ba);
-	ownPass = intFromData(ba);
-
-	d2i_old(ba, type);
-	pki_openssl_error();
-
-	encKey = ba;
+	return _sha512passwd(pass, salt, 17, 8000);
 }
-
