@@ -17,13 +17,23 @@
 #include "pki_key.h"
 #include "Passwd.h"
 
+#define VIEW_private_ownpass 9
+
 class pki_evp: public pki_key
 {
 		Q_OBJECT
-	protected:
 		QByteArray encKey;
 		void init();
-		void veryOldFromData(unsigned char *p, int size);
+		QByteArray getEncKey() const;
+		QString encKey_b64()
+		{
+			return QString::fromLatin1(encKey.toBase64());
+		}
+		static QString _sha512passwd(QByteArray pass, QString salt,
+						int size, int repeat);
+		void set_EVP_PKEY(EVP_PKEY *pkey);
+
+	protected:
 		void openssl_pw_error(QString fname);
 	public:
 		static QPixmap *icon[2];
@@ -32,6 +42,7 @@ class pki_evp: public pki_key
 		static Passwd oldpasswd;
 		static QString md5passwd(QByteArray pass);
 		static QString sha512passwd(QByteArray pass, QString salt);
+		static QString sha512passwT(QByteArray pass, QString salt);
 		void generate(int bits, int type, QProgressBar *progress);
 		void generate(int bits, int type, QProgressBar *progress,
 				int curve_nid);
@@ -42,6 +53,8 @@ class pki_evp: public pki_key
 		void encryptKey(const char *password = NULL);
 		void bogusEncryptKey();
 		EVP_PKEY *decryptKey() const;
+		EVP_PKEY *legacyDecryptKey(QByteArray &myencKey,
+					   Passwd &ownPassBuf) const;
 		pki_evp(const pki_evp *pk);
 		/* destructor */
 		virtual ~pki_evp();
@@ -52,16 +65,16 @@ class pki_evp: public pki_key
 		void fload(const QString fname);
 		void writeDefault(const QString fname);
 		void fromData(const unsigned char *p, db_header_t *head);
-		void oldFromData(unsigned char *p, int size);
-		QByteArray toData();
 		void writeKey(const QString fname, const EVP_CIPHER *enc,
 		pem_password_cb *cb, bool pem);
 		void writePKCS8(const QString fname, const EVP_CIPHER *enc,
 		pem_password_cb *cb, bool pem);
-		bool isPubKey() const;
 		int verify();
-		const EVP_MD *getDefaultMD();
-		QVariant getIcon(dbheader *hd);
+		QVariant getIcon(const dbheader *hd) const;
+		bool sqlUpdatePrivateKey();
+		QSqlError insertSqlData();
+		QSqlError deleteSqlData();
+		void restoreSql(const QSqlRecord &rec);
 };
 
 #endif
