@@ -18,11 +18,20 @@
 
 #define MAX_KEY_LENGTH 4096
 
+#define VIEW_public_keys_type 6
+#define VIEW_public_keys_len 7
+#define VIEW_public_keys_public 8
+
 class pki_key: public pki_base
 {
 		Q_OBJECT
+	public:
+		enum passType { ptCommon, ptPrivate, ptBogus, ptPin };
+
 	protected:
-		int ownPass;
+		enum passType ownPass;
+		int key_size;
+		bool isPub;
 		EVP_PKEY *key;
 		QString BN2QString(const BIGNUM *bn) const;
 		QString BNoneLine(BIGNUM *bn) const;
@@ -32,46 +41,31 @@ class pki_key: public pki_base
 		BIGNUM *ssh_key_data2bn(QByteArray *ba, bool skip = false);
 		void ssh_key_QBA2data(QByteArray &ba, QByteArray *data);
 		void ssh_key_bn2data(const BIGNUM *bn, QByteArray *data);
-		int ucount; // usage counter
+		mutable int useCount; // usage counter
 	public:
 		pki_key(const QString name = "");
 		pki_key(const pki_key *pk);
 		virtual ~pki_key();
 		static builtin_curves builtinCurves;
-		enum passType { ptCommon, ptPrivate, ptBogus, ptPin };
-
-		virtual EVP_PKEY *decryptKey() const
-		{
-			return NULL;
-		}
-		virtual QString length() const
-		{
-			return QString();
-		}
-		virtual bool isPubKey() const
-		{
-			return true;
-		}
-		virtual const EVP_MD *getDefaultMD()
-		{
-			return NULL;
-		}
+		QString length() const;
+		QString comboText() const;
+		QString getKeyTypeString(void) const;
+		virtual EVP_PKEY *decryptKey() const = 0;
 		virtual bool isToken();
 		virtual QString getTypeString(void) const;
-		virtual QString getIntNameWithType(void);
 		virtual QList<int> possibleHashNids();
-		virtual QString getMsg(msg_type msg);
-		virtual QString length();
+		QString getMsg(msg_type msg) const;
 
 		void writePublic(const QString fname, bool pem);
 		bool compare(pki_base *ref);
 		int getKeyType() const;
-		static QString removeTypeFromIntName(QString n);
 		bool isPrivKey() const;
-		int incUcount();
-		int decUcount();
-		int getUcount();
-		int getOwnPass(void)
+		int getUcount() const;
+		void setUcount(int c)
+		{
+			useCount = c;
+		}
+		enum passType getOwnPass(void)
 		{
 			return ownPass;
 		}
@@ -79,19 +73,31 @@ class pki_key: public pki_base
 		{
 			return key;
 		}
+		bool isPubKey() const
+		{
+			return isPub;
+		}
 		BIO *pem(BIO *, int);
-		QVariant column_data(dbheader *hd);
-		QString modulus();
-		QString pubEx();
-		QString subprime();
-		QString pubkey();
-		int ecParamNid();
-		QString ecPubKey();
+		QVariant column_data(const dbheader *hd) const;
+		QString modulus() const;
+		QString pubEx() const;
+		QString subprime() const;
+		QString pubkey() const;
+		int ecParamNid() const;
+		QString ecPubKey() const;
 		void d2i(QByteArray &ba);
 		void d2i_old(QByteArray &ba, int type);
-		QByteArray i2d();
+		QByteArray i2d() const;
 		EVP_PKEY *load_ssh2_key(FILE *fp);
 		void writeSSH2public(QString fname);
+		void resetUcount()
+		{
+			useCount = -1;
+		}
+		QSqlError insertSqlData();
+		QSqlError deleteSqlData();
+		void restoreSql(const QSqlRecord &rec);
 };
 
+Q_DECLARE_METATYPE(pki_key *);
 #endif
