@@ -30,14 +30,12 @@
 #include "openssl_compat.h"
 
 QPixmap *pki_scard::icon[1] = { NULL };
-bool pki_scard::only_token_hashes = false;
 
 void pki_scard::init(void)
 {
-	class_name = "pki_scard";
 	ownPass = ptPin;
-	dataVersion = 2;
 	pkiType = smartCard;
+	isPub = false;
 
 	card_serial = card_manufacturer = card_label = "";
 	card_model = slot_label = "";
@@ -49,7 +47,7 @@ pki_scard::pki_scard(const QString name)
 	init();
 }
 
-QString pki_scard::getMsg(msg_type msg)
+QString pki_scard::getMsg(msg_type msg) const
 {
 	/*
 	 * We do not construct english sentences from fragments
@@ -67,6 +65,70 @@ QString pki_scard::getMsg(msg_type msg)
 	return pki_base::getMsg(msg);
 }
 
+QSqlError pki_scard::insertSqlData()
+{
+	XSqlQuery q;
+	QSqlError e = pki_key::insertSqlData();
+	if (e.isValid())
+		return e;
+
+	SQL_PREPARE(q, "INSERT INTO tokens (item, card_manufacturer, card_serial, "
+					"card_model, card_label, slot_label, "
+					"object_id) "
+		  "VALUES (?, ?, ?, ?, ?, ?, ?)");
+	q.bindValue(0, sqlItemId);
+	q.bindValue(1, card_manufacturer);
+	q.bindValue(2, card_serial);
+	q.bindValue(3, card_model);
+	q.bindValue(4, card_label);
+	q.bindValue(5, slot_label);
+	q.bindValue(6, object_id);
+	q.exec();
+	e = q.lastError();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "INSERT INTO token_mechanism (item, mechanism) "
+		  "VALUES (?, ?)");
+	q.bindValue(0, sqlItemId);
+	foreach(CK_MECHANISM_TYPE m, mech_list) {
+		q.bindValue(1, QVariant((uint)m));
+		q.exec();
+	}
+	return q.lastError();
+}
+
+void pki_scard::restoreSql(const QSqlRecord &rec)
+{
+	pki_key::restoreSql(rec);
+	card_manufacturer = rec.value(VIEW_tokens_card_manufacturer).toString();
+	card_serial = rec.value(VIEW_tokens_card_serial).toString();
+	card_model = rec.value(VIEW_tokens_card_model).toString();
+	card_label = rec.value(VIEW_tokens_card_label).toString();
+	slot_label = rec.value(VIEW_tokens_slot_label).toString();
+	object_id = rec.value(VIEW_tokens_object_id).toString();
+	card_manufacturer = rec.value(VIEW_tokens_card_manufacturer).toString();
+	isPub = false;
+	qDebug() << card_manufacturer <<card_serial<<card_model<<card_label<<slot_label<<object_id;
+}
+
+QSqlError pki_scard::deleteSqlData()
+{
+	XSqlQuery q;
+	QSqlError e = pki_key::deleteSqlData();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "DELETE FROM tokens WHERE item=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	e = q.lastError();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "DELETE FROM token_mechanism WHERE item=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	return q.lastError();
+}
+
 EVP_PKEY *pki_scard::load_pubkey(pkcs11 &p11, CK_OBJECT_HANDLE object) const
 {
 	unsigned long keytype;
@@ -171,6 +233,8 @@ void pki_scard::load_token(pkcs11 &p11, CK_OBJECT_HANDLE object)
 	card_manufacturer = ti.manufacturerID();
 	card_serial = ti.serial();
 	card_model = ti.model();
+	pkiSource = token;
+	isPub = false;
 
 	pk11_attr_data id(CKA_ID);
 	p11.loadAttribute(id, object);
@@ -185,7 +249,7 @@ void pki_scard::load_token(pkcs11 &p11, CK_OBJECT_HANDLE object)
 		p11.loadAttribute(label, object);
 		slot_label = label.getText();
 	} catch (errorEx &err) {
-		printf("No PubKey Label: %s\n", err.getCString());
+		qDebug() << "No PubKey Label:" << err.getString();
 		// ignore
 	}
 	if (slot_label.isEmpty()) {
@@ -199,7 +263,7 @@ void pki_scard::load_token(pkcs11 &p11, CK_OBJECT_HANDLE object)
 			slot_label = xn.getMostPopular();
 			pki_openssl_error();
 		} catch (errorEx &err) {
-			printf("No Pubkey Subject: %s\n", err.getCString());
+			qDebug() << "No Pubkey Subject:" << err.getString();
 			// ignore
 		}
 	}
@@ -478,7 +542,7 @@ QList<int> pki_scard::possibleHashNids()
 {
 	QList<int> nids;
 
-	if (!only_token_hashes)
+	if (!Settings["only_token_hashes"])
 		return pki_key::possibleHashNids();
 
 	foreach(CK_MECHANISM_TYPE mechanism, mech_list) {
@@ -524,39 +588,6 @@ QList<int> pki_scard::possibleHashNids()
 	return nids;
 }
 
-const EVP_MD *pki_scard::getDefaultMD()
-{
-	if (mech_list.contains(CKM_SHA1_RSA_PKCS))
-		return EVP_sha1();
-	if (mech_list.contains(CKM_DSA_SHA1))
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-		return EVP_sha1();
-#else
-		return EVP_dss1();
-#endif
-#ifndef OPENSSL_NO_EC
-	if (mech_list.contains(CKM_ECDSA_SHA1))
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L
-		return EVP_sha1();
-#else
-		return EVP_ecdsa();
-#endif
-#endif
-	if (mech_list.contains(CKM_SHA512_RSA_PKCS))
-		return EVP_sha512();
-	if (mech_list.contains(CKM_SHA384_RSA_PKCS))
-		return EVP_sha384();
-	if (mech_list.contains(CKM_SHA256_RSA_PKCS))
-		return EVP_sha256();
-	if (mech_list.contains(CKM_RIPEMD160_RSA_PKCS))
-		return EVP_ripemd160();
-	if (mech_list.contains(CKM_MD5_RSA_PKCS))
-		return EVP_md5();
-
-	/* Last resort */
-	return EVP_sha1();
-}
-
 /* Assures the correct card is inserted and
  * returns the slot ID in slot true on success */
 bool pki_scard::prepare_card(slotid *slot, bool verifyPubkey) const
@@ -678,7 +709,7 @@ void pki_scard::generateKey_card(int type, slotid slot, int size,
 	atts << pk11_attr_ulong(CKA_CLASS, CKO_PUBLIC_KEY) << kt.id;
 	QList<CK_OBJECT_HANDLE> objects = p11.objectList(atts);
 	if (objects.count() != 1)
-		printf("OBJECTS found: %d\n",objects.count());
+		qCritical() << "OBJECTS found:" << objects.count();
 
 	if (objects.count() == 0)
 		throw errorEx(tr("Unable to find generated key on card"));
@@ -690,24 +721,6 @@ pki_scard::~pki_scard()
 {
 }
 
-QByteArray pki_scard::toData()
-{
-	QByteArray ba;
-
-	ba += db::stringToData(card_serial);
-	ba += db::stringToData(card_manufacturer);
-	ba += db::stringToData(card_label);
-	ba += db::stringToData(slot_label);
-	ba += db::stringToData(card_model);
-	ba += db::stringToData(object_id);
-	ba += db::intToData(mech_list.count());
-	for (int i=0; i<mech_list.count(); i++)
-		ba += db::intToData(mech_list[i]);
-
-	ba += i2d();
-	return ba;
-}
-
 void pki_scard::fromData(const unsigned char *p, db_header_t *head )
 {
 	int version, size;
@@ -744,11 +757,6 @@ void pki_scard::fromData(const unsigned char *p, db_header_t *head )
 	}
 }
 
-bool pki_scard::isPubKey() const
-{
-	return false;
-}
-
 QString pki_scard::getTypeString(void) const
 {
 	return tr("Token %1").arg(pki_key::getTypeString());
@@ -828,12 +836,12 @@ bool pki_scard::isToken()
 	return true;
 }
 
-QVariant pki_scard::getIcon(dbheader *hd)
+QVariant pki_scard::getIcon(const dbheader *hd) const
 {
 	return hd->id == HD_internal_name ? QVariant(*icon[0]) : QVariant();
 }
 
-bool pki_scard::visible()
+bool pki_scard::visible() const
 {
 	QStringList sl;
 	if (pki_base::visible())
@@ -847,3 +855,22 @@ bool pki_scard::visible()
 	}
 	return false;
 }
+
+void pki_scard::updateLabel(QString label)
+{
+	XSqlQuery q;
+	Transaction;
+
+	if (slot_label == label)
+		return;
+	if (!TransBegin())
+		return;
+	slot_label = label;
+
+	SQL_PREPARE(q, "UPDATE tokens SET slot_label=? WHERE item=?");
+	q.bindValue(0, slot_label);
+	q.bindValue(1, sqlItemId);
+	q.exec();
+	AffectedItems(sqlItemId);
+	TransCommit();
+}
