@@ -15,6 +15,13 @@
 #include "pkcs11.h"
 #include "pki_key.h"
 
+#define VIEW_tokens_card_manufacturer 10
+#define VIEW_tokens_card_serial 11
+#define VIEW_tokens_card_model 12
+#define VIEW_tokens_card_label 13
+#define VIEW_tokens_slot_label 14
+#define VIEW_tokens_object_id  15
+
 class pki_scard: public pki_key
 {
 		Q_OBJECT
@@ -32,12 +39,9 @@ class pki_scard: public pki_key
 		pki_scard(const QString name);
 		virtual ~pki_scard();
 		static QPixmap *icon[1];
-		static bool only_token_hashes;
 		void load_token(pkcs11 &p11, CK_OBJECT_HANDLE object);
 		bool prepare_card(slotid *slot, bool verifyPubkey=true) const;
 		void fromData(const unsigned char *p, db_header_t *head);
-		QByteArray toData();
-		bool isPubKey() const;
 		QString getTypeString(void) const;
 		QString getManufacturer() const
 		{
@@ -55,6 +59,7 @@ class pki_scard: public pki_key
 		{
 			return slot_label;
 		}
+		void updateLabel(QString label);
 		QString getId() const
 		{
 			return object_id;
@@ -71,7 +76,7 @@ class pki_scard: public pki_key
 		void changeSoPin();
 		int verify();
 		bool isToken();
-		QVariant getIcon(dbheader *hd);
+		QVariant getIcon(const dbheader *hd) const;
 		QList<CK_MECHANISM_TYPE> getMech_list()
 		{
 			return mech_list;
@@ -81,15 +86,17 @@ class pki_scard: public pki_key
 		void setMech_list(QList<CK_MECHANISM_TYPE> ml) { mech_list = ml; };
 		QList<int> possibleHashNids();
 		EVP_PKEY *load_pubkey(pkcs11 &p11, CK_OBJECT_HANDLE object) const;
-		const EVP_MD *getDefaultMD();
 		void generateKey_card(int type, slotid slot, int size,
 					int curve_nid, QProgressBar *bar);
 		void deleteFromToken();
 		void deleteFromToken(slotid slot);
 		void store_token(slotid slot, EVP_PKEY *pkey);
 		int renameOnToken(slotid slot, QString name);
-		QString getMsg(msg_type msg);
-		bool visible();
+		QString getMsg(msg_type msg) const;
+		bool visible() const;
+		QSqlError insertSqlData();
+		QSqlError deleteSqlData();
+		void restoreSql(const QSqlRecord &rec);
 };
 
 #endif
