@@ -9,6 +9,7 @@
 #include "pki_x509.h"
 #include "pki_evp.h"
 #include "pki_scard.h"
+#include "db_base.h"
 #include "func.h"
 #include "base.h"
 #include "exception.h"
@@ -20,9 +21,6 @@
 
 #include "openssl_compat.h"
 
-bool pki_x509::dont_colorize_expiries = false;
-bool pki_x509::disable_netscape = false;
-
 QPixmap *pki_x509::icon[5];
 
 pki_x509::pki_x509(X509 *c)
@@ -41,13 +39,10 @@ pki_x509::pki_x509(const pki_x509 *crt)
 	pki_openssl_error();
 	psigner = crt->psigner;
 	setRefKey(crt->getRefKey());
-	trust = crt->trust;
-	efftrust = crt->efftrust;
-	caSerial = crt->caSerial;
-	caTemplate = crt->caTemplate;
+	caTemplateSqlId = crt->caTemplateSqlId;
 	revocation = crt->revocation;
 	crlDays = crt->crlDays;
-	crlExpiry = crt->crlExpiry;
+	crlExpire = crt->crlExpire;
 	pki_openssl_error();
 }
 
@@ -60,7 +55,7 @@ pki_x509::pki_x509(const QString name)
 	pki_openssl_error();
 }
 
-QString pki_x509::getMsg(msg_type msg)
+QString pki_x509::getMsg(msg_type msg) const
 {
 	/*
 	 * We do not construct english sentences from fragments
@@ -73,12 +68,144 @@ QString pki_x509::getMsg(msg_type msg)
 	case msg_import: return tr("Successfully imported the certificate '%1'");
 	case msg_delete: return tr("Delete the certificate '%1'?");
 	case msg_create: return tr("Successfully created the certificate '%1'");
-	/* %1: Number of certs; %2: list of cert ames */
+	/* %1: Number of certs; %2: list of cert names */
 	case msg_delete_multi: return tr("Delete the %1 certificates: %2?");
 	}
 	return pki_base::getMsg(msg);
 }
 
+QSqlError pki_x509::insertSqlData()
+{
+	XSqlQuery q;
+	a1time now;
+	pki_x509 *signer = findIssuer();
+	QSqlError e = pki_x509super::insertSqlData();
+	if (e.isValid())
+		return e;
+
+	SQL_PREPARE(q, "INSERT INTO certs (item, hash, iss_hash, serial, issuer, "
+				"ca, cert) "
+		  "VALUES (?, ?, ?, ?, ?, ?, ?)");
+	q.bindValue(0, sqlItemId);
+	q.bindValue(1, hash());
+	q.bindValue(2, (uint)getIssuerName().hashNum());
+	q.bindValue(3, getSerial().toHex());
+	q.bindValue(4, signer ? signer->getSqlItemId() : QVariant());
+	q.bindValue(5, (int)isCA());
+	q.bindValue(6, i2d_b64());
+	q.exec();
+	if (!isCA())
+		return q.lastError();
+
+	SQL_PREPARE(q, "INSERT INTO authority (item, template, crlExpire, crlNo, crlDays) "
+			"VALUES (?, ?, ?, 0, ?)");
+	q.bindValue(0, sqlItemId);
+	q.bindValue(1, caTemplateSqlId);
+	q.bindValue(2, crlExpire.toPlain());
+	q.bindValue(3, crlDays);
+	q.exec();
+	if (fromDataRevList.size() > 0)
+		fromDataRevList.sqlUpdate(sqlItemId);
+	return q.lastError();
+}
+
+void pki_x509::restoreSql(const QSqlRecord &rec)
+{
+	pki_x509super::restoreSql(rec);
+	QByteArray ba = QByteArray::fromBase64(
+				rec.value(VIEW_x509_cert).toByteArray());
+	d2i(ba);
+	signerSqlId = rec.value(VIEW_x509_issuer);
+	crlNumber.set(rec.value(VIEW_x509_auth_crlNo).toUInt());
+	crlExpire.fromPlain(rec.value(VIEW_x509_auth_crlExpire).toString());
+	caTemplateSqlId = rec.value(VIEW_x509_auth_template);
+	if (!rec.isNull(VIEW_x509_auth_crlDays))
+		crlDays = rec.value(VIEW_x509_auth_crlDays).toInt();
+	else
+		crlDays = 30;
+	if (!rec.isNull(VIEW_x509_revocation))
+		revocation = x509rev(rec, VIEW_x509_revocation);
+}
+
+QSqlError pki_x509::deleteSqlData()
+{
+	XSqlQuery q;
+	QSqlError e = pki_x509super::deleteSqlData();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "DELETE FROM certs WHERE item=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	e = q.lastError();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "DELETE FROM authority WHERE item=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	e = q.lastError();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "UPDATE crls SET issuer=NULL WHERE issuer=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	e = q.lastError();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "UPDATE certs SET issuer=NULL WHERE issuer=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	e = q.lastError();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "DELETE FROM revocations WHERE caId=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	// Select affected items
+	QList<pki_base*> list = db_base::sqlSELECTpki<pki_base>(
+		"SELECT DISTINCT items.id FROM items, certs, crls "
+		"WHERE (items.id = certs.item OR items.id = crls.item) "
+		"AND crls.issuer = ? AND certs.issuer = ?",
+		QList<QVariant>() << QVariant(sqlItemId)
+				  << QVariant(sqlItemId));
+
+	foreach(pki_base *pki, list)
+		AffectedItems(pki->getSqlItemId());
+
+	return q.lastError();
+}
+
+pki_x509 *pki_x509::findIssuer()
+{
+	XSqlQuery q;
+	pki_x509 *issuer;
+	unsigned hash;
+
+	if ((issuer = getSigner()) != NULL)
+		return issuer;
+	// first check for self-signed
+	if (verify(this))
+		return this;
+
+	hash = getIssuerName().hashNum();
+	/* Select X509 CA certificates with subject-hash == hash */
+	SQL_PREPARE(q, "SELECT x509super.item from x509super "
+		"JOIN certs ON certs.item = x509super.item "
+		"WHERE certs.ca=1 AND x509super.subj_hash=?");
+	q.bindValue(0, hash);
+	q.exec();
+	while (q.next()) {
+		issuer = db_base::lookupPki<pki_x509>(q.value(0));
+		if (!issuer) {
+			qDebug("Certificate with id %d not found",
+                                q.value(0).toInt());
+		}
+		if (verify(issuer)) {
+			return issuer;
+		}
+	}
+	return NULL;
+}
+
 void pki_x509::fromPEM_BIO(BIO *bio, QString name)
 {
 	X509 *_cert;
@@ -89,8 +216,6 @@ void pki_x509::fromPEM_BIO(BIO *bio, QString name)
 	autoIntName();
 	if (getIntName().isEmpty())
 		setIntName(rmslashdot(name));
-	trust = 1;
-	efftrust = 1;
 }
 
 void pki_x509::fload(const QString fname)
@@ -118,8 +243,6 @@ void pki_x509::fload(const QString fname)
 	autoIntName();
 	if (getIntName().isEmpty())
 		setIntName(rmslashdot(fname));
-	trust = 1;
-	efftrust = 1;
 }
 
 pki_x509::~pki_x509()
@@ -133,17 +256,11 @@ pki_x509::~pki_x509()
 void pki_x509::init()
 {
 	psigner = NULL;
-	trust = 0;
-	efftrust = 0;
-	caSerial = 1;
-	caTemplate = "";
+	caTemplateSqlId = QVariant();
 	crlDays = 30;
-	crlExpiry.setUndefined();
-	class_name = "pki_x509";
+	crlExpire.setUndefined();
 	cert = NULL;
-	dataVersion = 4;
 	pkiType = x509;
-	randomSerial = false;
 }
 
 void pki_x509::setSerial(const a1int &serial)
@@ -169,19 +286,6 @@ pki_x509 *pki_x509::getBySerial(const a1int &a) const
 	return NULL;
 }
 
-#define SERIAL_LEN 8
-a1int pki_x509::getIncCaSerial()
-{
-	unsigned char buf[SERIAL_LEN];
-	if (!randomSerial)
-		return caSerial++;
-
-	RAND_bytes(buf, SERIAL_LEN);
-	a1int serial;
-	serial.setRaw(buf, SERIAL_LEN);
-	return serial;
-}
-
 a1int pki_x509::hashInfo(const EVP_MD *md) const
 {
 	unsigned char digest[EVP_MAX_MD_SIZE];
@@ -209,7 +313,7 @@ void pki_x509::load_token(pkcs11 &p11, CK_OBJECT_HANDLE object)
 		p11.loadAttribute(label, object);
 		desc = label.getText();
 	} catch(errorEx &err) {
-		printf("No Cert Label: %s\n", err.getCString());
+		qDebug("No Cert Label: %s", err.getCString());
 		// IGNORE
 	}
 	pk11_attr_data x509(CKA_VALUE);
@@ -228,11 +332,12 @@ void pki_x509::load_token(pkcs11 &p11, CK_OBJECT_HANDLE object)
 			desc = xn.getMostPopular();
 			pki_openssl_error();
 		} catch(errorEx &err) {
-			printf("No Cert Subject: %s\n", err.getCString());
+			qDebug("No Cert Subject: %s", err.getCString());
 			// IGNORE
 		}
 	}
 	setIntName(desc);
+	pkiSource = token;
 	pki_openssl_error();
 }
 
@@ -246,7 +351,7 @@ void pki_x509::d2i(QByteArray &ba)
 	pki_openssl_error();
 }
 
-QByteArray pki_x509::i2d()
+QByteArray pki_x509::i2d() const
 {
 	return i2d_bytearray(I2D_VOID(i2d_X509), cert);
 }
@@ -264,8 +369,8 @@ void pki_x509::store_token(bool alwaysSelect)
 		if (!p11.selectToken(&slot, NULL))
 			return;
 	} else {
-		card = (pki_scard *)privkey;
-		if (!card->prepare_card(&slot))
+		card = dynamic_cast<pki_scard *>(privkey);
+		if (!card || !card->prepare_card(&slot))
 			return;
 	}
 
@@ -287,7 +392,7 @@ void pki_x509::store_token(bool alwaysSelect)
 		pk11_attr_bool(CKA_TOKEN, true) <<
 		pk11_attr_bool(CKA_PRIVATE, false) <<
 		pk11_attr_data(CKA_SUBJECT, getSubject().i2d()) <<
-		pk11_attr_data(CKA_ISSUER, getIssuer().i2d()) <<
+		pk11_attr_data(CKA_ISSUER, getIssuerName().i2d()) <<
 		pk11_attr_data(CKA_SERIAL_NUMBER, getSerial().i2d()) <<
 		pk11_attr_data(CKA_LABEL, desc.toUtf8()) <<
 		(card ? card->getIdAttr() : p11.findUniqueID(CKO_CERTIFICATE));
@@ -300,10 +405,10 @@ void pki_x509::store_token(bool alwaysSelect)
 
 void pki_x509::deleteFromToken()
 {
-	pki_scard *card = (pki_scard *)privkey;
+	pki_scard *card = dynamic_cast<pki_scard *>(privkey);
 	slotidList p11_slots;
 
-	if (!pkcs11::loaded())
+	if (!card || !pkcs11::loaded())
 		return;
 
 	if (privkey && privkey->isToken()) {
@@ -404,7 +509,7 @@ x509name pki_x509::getSubject() const
 	return x;
 }
 
-x509name pki_x509::getIssuer() const
+x509name pki_x509::getIssuerName() const
 {
 	x509name x(X509_get_issuer_name(cert));
 	pki_openssl_error();
@@ -442,22 +547,29 @@ void pki_x509::delSigner(pki_base *s)
 		psigner = NULL;
 }
 
-bool pki_x509::canSign()
+bool pki_x509::isCA() const
 {
-	BASIC_CONSTRAINTS *bc;
+	bool ca;
 	int crit;
+	BASIC_CONSTRAINTS *bc = (BASIC_CONSTRAINTS *)
+		X509_get_ext_d2i(cert, NID_basic_constraints, &crit, NULL);
+	pki_openssl_error();
+	ca = bc && bc->ca;
+	if (bc)
+		BASIC_CONSTRAINTS_free(bc);
+	return ca;
+}
+
+bool pki_x509::canSign() const
+{
 	if (!privkey || privkey->isPubKey())
 		return false;
 	if (privkey->isToken() && !pkcs11::loaded())
 		return false;
-	bc = (BASIC_CONSTRAINTS *)X509_get_ext_d2i(cert, NID_basic_constraints, &crit, NULL);
-	pki_openssl_error();
-	if (!bc || !bc->ca)
-		return false;
-	return true;
+	return isCA();
 }
 
-bool pki_x509::hasExtension(int nid)
+bool pki_x509::hasExtension(int nid) const
 {
 	return getV3ext().idxByNid(nid) != -1;
 }
@@ -488,7 +600,7 @@ void pki_x509::fromData(const unsigned char *p, db_header_t *head)
 
 	d2i(ba);
 	pki_openssl_error();
-	trust = db::intFromData(ba);
+	/* trust = */ db::intFromData(ba);
 	if (version < 4) {
 		a1time revoked;
 		isRevoked = db::boolFromData(ba);
@@ -499,16 +611,14 @@ void pki_x509::fromData(const unsigned char *p, db_header_t *head)
 			revocation.setSerial(getSerial());
 		}
 	}
-	caSerial.setHex(db::stringFromData(ba));
 	pki_openssl_error();
-	caTemplate = db::stringFromData(ba);
+	/* Superflous CaSerial = */db::stringFromData(ba);
+	QString caTemplate = db::stringFromData(ba);
 	crlDays = db::intFromData(ba);
-	crlExpiry.d2i(ba);
+	crlExpire.d2i(ba);
 	pki_openssl_error();
 	if (version > 1)
-		randomSerial = db::boolFromData(ba);
-	else
-		randomSerial = false;
+		/* randomSerial = */ db::boolFromData(ba);
 	if (version > 2)
 		crlNumber.setHex(db::stringFromData(ba));
 	pki_openssl_error();
@@ -525,44 +635,27 @@ void pki_x509::fromData(const unsigned char *p, db_header_t *head)
 	}
 	pki_openssl_error();
 	if (version > 3) {
-		x509revList curr(revList);
-		revList.fromBA(ba);
+		fromDataRevList.fromBA(ba);
 		pki_openssl_error();
-		revList.merge(curr);
 	}
 	if (ba.count() > 0) {
 		my_error(tr("Wrong Size %1").arg(ba.count()));
 	}
 	pki_openssl_error();
+
+	XSqlQuery q;
+	SQL_PREPARE(q, "SELECT id FROM items WHERE name=? AND type=?");
+	q.bindValue(0, caTemplate);
+	q.bindValue(1, tmpl);
+	q.exec();
+	if (q.next())
+		caTemplateSqlId = q.value(0);
 }
 
-QByteArray pki_x509::toData()
-{
-	QByteArray ba;
-
-	ba += i2d(); // cert
-	ba += db::intToData(trust);
-	// version 4: don't store isrevoked, revoked
-
-	// the serial if this is a CA
-	ba += db::stringToData(caSerial.toHex());
-	// the name of the template to use for signing
-	ba += db::stringToData(caTemplate);
-	// version 3
-	ba += db::intToData(crlDays); // the CRL period
-	ba += crlExpiry.i2d(); // last CRL date
-	ba += db::boolToData(randomSerial);
-	ba += db::stringToData(crlNumber.toHex());
-	// version 4: don't store own revocation but client revocations
-	ba += revList.toBA();
-	pki_openssl_error();
-	return ba;
-}
 
 void pki_x509::writeDefault(const QString fname)
 {
-	writeCert(fname + QDir::separator() + getIntName() + ".crt",
-			true, false);
+	writeCert(get_dump_filename(fname, ".crt"), true, false);
 }
 
 void pki_x509::writeCert(const QString fname, bool PEM, bool append)
@@ -585,7 +678,7 @@ void pki_x509::writeCert(const QString fname, bool PEM, bool append)
 		fopen_error(fname);
 }
 
-void pki_x509::writeIndexEntry(FILE *fp)
+QString pki_x509::getIndexEntry()
 {
 	QString flag = NULL;
 	bool revoked = isRevoked();
@@ -597,28 +690,11 @@ void pki_x509::writeIndexEntry(FILE *fp)
 	else
 		flag = "E";
 
-	QString line = QString("%1\t%2\t%3\t%4\tunknown\t%5\n").arg(
-		flag, getNotAfter().toPlainUTC(), (revoked ? revocation.getDate().toPlainUTC() : ""),
-		getSerial().toHex(), QString(X509_NAME_oneline(getSubject().get(), NULL, 0)));
-
-	QByteArray ba = line.toUtf8();
-	fwrite(ba.constData(), ba.size(), 1, fp);
-}
-
-void pki_x509::writeIndexEntry(const QString fname, bool append)
-{
-	FILE *fp;
-	const char *p = "w";
-	if (append)
-		p = "a";
-	fp = fopen(QString2filename(fname), p);
-	if (fp != NULL) {
-		if (cert)
-			writeIndexEntry(fp);
-		fclose(fp);
-		pki_openssl_error();
-	} else
-		fopen_error(fname);
+	return QString("%1\t%2\t%3\t%4\tunknown\t%5\n").arg(
+		flag, getNotAfter().toPlainUTC(),
+		revoked ? revocation.getDate().toPlainUTC() : "",
+		getSerial(),
+		QString(X509_NAME_oneline(getSubject().get(), NULL, 0)));
 }
 
 BIO *pki_x509::pem(BIO *b, int format)
@@ -662,24 +738,44 @@ bool pki_x509::verify(pki_x509 *signer)
 		return true;
 	if ((psigner != NULL) || (signer == NULL))
 		return false;
+	if (signer == this && signerSqlId == sqlItemId)
+		return true;
+
 	if (verify_only(signer)) {
 		int idx;
 		x509rev r;
+		x509revList rl(revocation);
 		r.setSerial(getSerial());
 		psigner = signer;
-		psigner->revList.merge(x509revList(revocation));
-		idx = psigner->revList.indexOf(r);
+		signerSqlId = psigner->sqlItemId;
+		psigner->mergeRevList(rl);
+		rl = psigner->getRevList();
+		idx = rl.indexOf(r);
 		if (idx != -1)
-			revocation = psigner->revList[idx];
+			revocation = rl[idx];
 		return true;
 	}
 	return false;
 }
 
+x509revList pki_x509::getRevList() const
+{
+	return isCA() ? x509revList::fromSql(sqlItemId) : x509revList();
+}
+
+void pki_x509::mergeRevList(x509revList &l)
+{
+	x509revList revList = getRevList();
+	revList.merge(l);
+
+	if (revList.merged)
+		revList.sqlUpdate(sqlItemId);
+}
+
 void pki_x509::setRevocations(const x509revList &rl)
 {
-	revList = rl;
 	x509rev rev;
+	x509revList revList = rl;
 
 	foreach(pki_base *p, childItems) {
 		pki_x509 *pki = static_cast<pki_x509 *>(p);
@@ -690,6 +786,7 @@ void pki_x509::setRevocations(const x509revList &rl)
 		else
 			pki->revocation = x509rev();
 	}
+	revList.sqlUpdate(sqlItemId);
 }
 
 pki_key *pki_x509::getPubKey() const
@@ -738,22 +835,15 @@ void pki_x509::setPubKey(pki_key *key)
 	pki_openssl_error();
 }
 
-QString pki_x509::fingerprint(const EVP_MD *digest)
+QString pki_x509::fingerprint(const EVP_MD *digest) const
 {
-	QString fp="";
-	char zs[4];
-	unsigned int n, j;
+	unsigned int n;
 	unsigned char md[EVP_MAX_MD_SIZE];
 
 	pki_openssl_error();
-	if (X509_digest(cert, digest, md, &n)) {
-		for (j=0; j<n; j++) {
-			sprintf(zs, "%02X%c",md[j], (j+1 == n) ?'\0':':');
-			fp += zs;
-		}
-	}
+	X509_digest(cert, digest, md, &n);
 	pki_openssl_error();
-	return fp;
+	return formatHash(md, n);
 }
 
 bool pki_x509::checkDate()
@@ -775,14 +865,14 @@ bool pki_x509::checkDate()
 	return true;
 }
 
-extList pki_x509::getV3ext()
+extList pki_x509::getV3ext() const
 {
 	extList el;
 	el.setStack(X509_get0_extensions(cert));
 	return el;
 }
 
-x509v3ext pki_x509::getExtByNid(int nid)
+x509v3ext pki_x509::getExtByNid(int nid) const
 {
 	extList el = getV3ext();
 	int i = el.idxByNid(nid);
@@ -797,43 +887,17 @@ x509v3ext pki_x509::getExtByNid(int nid)
 	return el[i];
 }
 
-int pki_x509::sigAlg()
+int pki_x509::sigAlg() const
 {
 	return X509_get_signature_nid(cert);
 }
 
 pki_x509 *pki_x509::getSigner()
 {
-	return (pki_x509 *)psigner;
+	return static_cast<pki_x509 *>(psigner);
 }
 
-int pki_x509::getTrust()
-{
-	if (trust > 2) trust = 2;
-	if (trust < 0) trust = 0;
-	return trust;
-}
-
-void pki_x509::setTrust(int t)
-{
-	if (t>=0 && t<=2)
-		trust = t;
-}
-
-int pki_x509::getEffTrust()
-{
-	if (efftrust > 2) efftrust = 2;
-	if (efftrust < 0) efftrust = 0;
-	return efftrust;
-}
-
-void pki_x509::setEffTrust(int t)
-{
-	if (t>= 0 && t<= 2)
-		efftrust = t;
-}
-
-bool pki_x509::isRevoked()
+bool pki_x509::isRevoked() const
 {
 	return revocation.isValid();
 }
@@ -841,47 +905,9 @@ bool pki_x509::isRevoked()
 void pki_x509::setRevoked(const x509rev &revok)
 {
 	revocation = revok;
-	if (revok.isValid()) {
-		setEffTrust(0);
-		setTrust(0);
-	}
 }
 
-int pki_x509::calcEffTrust()
-{
-	int mytrust = trust;
-	if (mytrust != 1) {
-		efftrust = mytrust;
-		return mytrust;
-	}
-	if (isRevoked()) {
-		efftrust = 0;
-		return 0;
-	}
-	if (getSigner() == this && trust == 1) { // inherit trust, but self signed
-		trust=0;
-		efftrust=0;
-		return 0;
-	}
-	//we must look at the parent certs
-	pki_x509 *signer = getSigner();
-	while (mytrust == 1 && signer && signer != this) {
-		mytrust = signer->getTrust();
-		signer = signer->getSigner();
-	}
-
-	if (mytrust == 1) mytrust = 0;
-	efftrust = mytrust;
-	return mytrust;
-}
-
-void pki_x509::setCrlExpiry(const a1time &time)
-{
-	crlExpiry = time;
-	pki_openssl_error();
-}
-
-bool pki_x509::caAndPathLen(bool *ca, a1int *pathlen, bool *hasLen)
+bool pki_x509::caAndPathLen(bool *ca, a1int *pathlen, bool *hasLen) const
 {
 	x509v3ext e = getExtByNid(NID_basic_constraints);
 	if (e.nid() != NID_basic_constraints)
@@ -898,11 +924,8 @@ bool pki_x509::caAndPathLen(bool *ca, a1int *pathlen, bool *hasLen)
 	return true;
 }
 
-QVariant pki_x509::column_data(dbheader *hd)
+QVariant pki_x509::column_data(const dbheader *hd) const
 {
-	QString truststatus[] =
-		{ tr("Not trusted"), tr("Trust inherited"), tr("Always Trusted") };
-
 	switch (hd->id) {
 		case HD_cert_serial:
 			return QVariant(getSerial().toHex());
@@ -910,14 +933,12 @@ QVariant pki_x509::column_data(dbheader *hd)
 			return QVariant(getNotBefore().toSortable());
 		case HD_cert_notAfter:
 			return QVariant(getNotAfter().toSortable());
-		case HD_cert_trust:
-			return QVariant(truststatus[getTrust()]);
 		case HD_cert_revocation:
 			return QVariant(isRevoked() ?
 				revocation.getDate().toSortable() : "");
 		case HD_cert_crl_expire:
-			if (canSign() && !crlExpiry.isUndefined())
-				return QVariant(crlExpiry.toSortable());
+			if (canSign() && !crlExpire.isUndefined())
+				return QVariant(crlExpire.toSortable());
 			else
 				return QVariant();
 		case HD_cert_md5fp:
@@ -943,7 +964,7 @@ QVariant pki_x509::column_data(dbheader *hd)
 	return pki_x509super::column_data(hd);
 }
 
-QVariant pki_x509::getIcon(dbheader *hd)
+QVariant pki_x509::getIcon(const dbheader *hd) const
 {
 	int pixnum = 0;
 	bool ca;
@@ -962,7 +983,7 @@ QVariant pki_x509::getIcon(dbheader *hd)
 		if (k && k->isPrivKey()) {
 			pixnum += 1;
 		}
-		if (calcEffTrust() == 0){
+		if (isRevoked()){
 			pixnum += 2;
 		}
 		break;
@@ -972,11 +993,11 @@ QVariant pki_x509::getIcon(dbheader *hd)
 	return QVariant(*icon[pixnum]);
 }
 
-bool pki_x509::visible()
+bool pki_x509::visible() const
 {
 	if (pki_x509super::visible())
 		return true;
-	if (getIssuer().search(limitPattern))
+	if (getIssuerName().search(limitPattern))
 		return true;
 	if (fingerprint(EVP_md5()).contains(limitPattern))
 		return true;
@@ -989,13 +1010,13 @@ bool pki_x509::visible()
 	return false;
 }
 
-QVariant pki_x509::bg_color(dbheader *hd)
+QVariant pki_x509::bg_color(const dbheader *hd) const
 {
 #define BG_RED     QBrush(QColor(255,  0,  0))
 #define BG_YELLOW  QBrush(QColor(255,255,  0))
 #define BG_CYAN    QBrush(QColor(127,255,212))
 
-	if (dont_colorize_expiries)
+	if (Settings["no_expire_colors"])
 		return QVariant();
 
 	a1time nb, na, now, certwarn;
@@ -1026,8 +1047,8 @@ QVariant pki_x509::bg_color(dbheader *hd)
 		case HD_cert_crl_expire:
 			if (canSign()) {
 				QDateTime crlwarn, crlex;
-				crlex = crlExpiry;
-				if (!crlExpiry.isUndefined()) {
+				crlex = crlExpire;
+				if (!crlExpire.isUndefined()) {
 					crlwarn = crlex.addSecs(-2 *60*60*24);
 					if (crlex < now)
 						return QVariant(BG_RED);
@@ -1038,72 +1059,3 @@ QVariant pki_x509::bg_color(dbheader *hd)
 	}
 	return QVariant();
 }
-
-void pki_x509::oldFromData(unsigned char *p, int size)
-{
-	int version, sRev, sLastCrl;
-	QByteArray ba((char*)p, size);
-	X509 *cert_sik = cert;
-	cert = NULL;
-	version = intFromData(ba);
-	if (version >=1 && version <= 5) {
-		intFromData(ba); /* sCert */
-		d2i(ba);
-		trust = intFromData(ba);
-		sRev = intFromData(ba);
-		if (sRev) {
-			a1time r;
-			r.d2i(ba);
-			if (version != 3) {
-				revocation.setSerial(getSerial());
-				revocation.setDate(r);
-			}
-		} else {
-			revocation = x509rev();
-		}
-
-		if (version == 1) {
-			caTemplate="";
-			caSerial=1;
-			crlExpiry.setUndefined();
-			crlDays=30;
-		}
-
-		if (version >= 2 ) {
-			if (version >= 5)
-				caSerial.setHex(db::stringFromData(ba));
-			else {
-				int i = intFromData(ba);
-				if (i>=0)
-					caSerial = i;
-				else {
-					caSerial = getSerial();
-					++caSerial;
-				}
-			}
-			caTemplate = db::stringFromData(ba);
-		}
-		if (version >= 3 ) {
-			crlDays = intFromData(ba);
-			sLastCrl = intFromData(ba);
-			if (sLastCrl) {
-			   crlExpiry.d2i(ba);
-			}
-		}
-		// version 4 saves a NULL as revoked
-		// version 3 did save a recent date :-((
-	}
-	else { // old version
-		d2i(ba);
-		trust = 1;
-		efftrust = 1;
-	}
-	if (cert)
-		X509_free(cert_sik);
-	else
-		cert = cert_sik;
-
-	if (ba.count() > 0) {
-		my_error(tr("Wrong Size %1").arg(ba.count()));
-	}
-}
