@@ -12,6 +12,7 @@
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 #include <openssl/pem.h>
+#include "pki_temp.h"
 #include "pki_key.h"
 #include "pki_x509req.h"
 #include "pki_x509super.h"
@@ -21,36 +22,46 @@
 #include "x509v3ext.h"
 #include "pkcs11.h"
 
+#define VIEW_x509_serial 7
+#define VIEW_x509_issuer 8
+#define VIEW_x509_ca 9
+#define VIEW_x509_cert 10
+#define VIEW_x509_auth_template 11
+#define VIEW_x509_auth_crlExpire 12
+#define VIEW_x509_auth_crlNo 13
+#define VIEW_x509_auth_crlDays 14
+#define VIEW_x509_auth_dnPolicy_UNUSED 15
+#define VIEW_x509_revocation 16
+
 class pki_x509 : public pki_x509super
 {
 		Q_OBJECT
 	private:
 		pki_x509 *psigner;
-		a1time crlExpiry;
-		bool randomSerial;
-		int trust;
-		int efftrust;
-		a1int caSerial;
+		QVariant signerSqlId;
+		a1time crlExpire;
 		a1int crlNumber;
 		int crlDays;
-		QString caTemplate;
+		QVariant caTemplateSqlId;
 		X509 *cert;
 		void init();
 		x509rev revocation;
+		x509revList fromDataRevList;
 
 	protected:
-		int sigAlg();
+		int sigAlg() const;
 
 	public:
 		static QPixmap *icon[5];
-		static bool dont_colorize_expiries;
-		static bool disable_netscape;
-		x509revList revList;
 		pki_x509(X509 *c);
 		pki_x509(const pki_x509 *crt);
 		pki_x509(const QString name = "");
 		~pki_x509();
 
+		void setSigner(pki_x509 *signer)
+		{
+			psigner = signer;
+		}
 		void fload(const QString fname);
 		void load_token(pkcs11 &p11, CK_OBJECT_HANDLE object);
 		void store_token(bool alwaysSelect);
@@ -64,53 +75,38 @@ class pki_x509 : public pki_x509super
 		a1time getNotBefore() const;
 		a1time getNotAfter() const;
 		x509name getSubject() const;
-		x509name getIssuer() const;
+		x509name getIssuerName() const;
 		void setSubject(const x509name &n);
 		void setIssuer(const x509name &n);
-		bool caAndPathLen(bool *ca, a1int *pathlen, bool *hasLen);
+		bool caAndPathLen(bool *ca, a1int *pathlen, bool *hasLen) const;
 
-		QByteArray toData();
 		void fromData(const unsigned char *p, db_header_t *head);
-		void oldFromData(unsigned char *p, int size);
-		bool canSign();
+		bool isCA() const;
+		bool canSign() const;
 		void writeCert(const QString fname, bool PEM, bool append = false);
-		void writeIndexEntry(FILE *fp);
-		void writeIndexEntry(const QString fname, bool append = false);
+		QString getIndexEntry();
 		bool verify(pki_x509 *signer);
 		bool verify_only(pki_x509 *signer);
 		pki_key *getPubKey() const;
 		void setPubKey(pki_key *key);
 		pki_x509 *getSigner();
 		void delSigner(pki_base *s);
-		QString fingerprint(const EVP_MD *digest);
-		extList getV3ext();
+		QString fingerprint(const EVP_MD *digest) const;
+		extList getV3ext() const;
 		bool checkDate();
 		bool addV3ext(const x509v3ext &e, bool skip_existing = false);
 		void sign(pki_key *signkey, const EVP_MD *digest);
+		pki_x509 *findIssuer();
 		X509 *getCert()
 		{
 			return cert;
 		}
-		int getTrust();
-		void setTrust(int t);
-		int getEffTrust();
-		void setEffTrust(int t);
 		void setRevoked(bool rev, a1time inval = a1time(),
 				QString reason = QString());
 		void setRevoked(const x509rev &revok);
-		bool isRevoked();
+		bool isRevoked() const;
 		pki_x509 *getBySerial(const a1int &a) const;
-		int calcEffTrust();
-		a1int getIncCaSerial();
-		a1int getCaSerial()
-		{
-			return caSerial;
-		}
-		void setCaSerial(a1int s)
-		{
-			caSerial = s;
-		}
-		a1int getCrlNumber()
+		a1int getCrlNumber() const
 		{
 			return crlNumber;
 		}
@@ -119,14 +115,13 @@ class pki_x509 : public pki_x509super
 			if (n > crlNumber)
 				crlNumber = n;
 		}
-		void setTemplate(QString s)
+		void setTemplateSqlId(QVariant sqlId)
 		{
-			if (s.length() > 0)
-				caTemplate = s;
+			caTemplateSqlId = sqlId;
 		}
-		QString getTemplate()
+		QVariant getTemplateSqlId()
 		{
-			return caTemplate;
+			return caTemplateSqlId;
 		}
 		void setCrlDays(int s)
 		{
@@ -137,40 +132,38 @@ class pki_x509 : public pki_x509super
 		{
 			return crlDays;
 		}
-		bool usesRandomSerial()
-		{
-			return randomSerial;
-		}
-		void setUseRandomSerial(bool r)
-		{
-			randomSerial = r;
-		}
 		x509rev getRevocation()
 		{
 			return revocation;
 		}
 		pk11_attlist objectAttributes();
-		void setCrlExpiry(const a1time &time);
-		bool hasExtension(int nid);
+		bool hasExtension(int nid) const;
 		bool cmpIssuerAndSerial(pki_x509 *refcert);
-		bool visible();
+		bool visible() const;
 		void updateView();
-		x509v3ext getExtByNid(int nid);
-		QVariant column_data(dbheader *hd);
-		QVariant getIcon(dbheader *hd);
-		QByteArray i2d();
+		x509v3ext getExtByNid(int nid) const;
+		QVariant column_data(const dbheader *hd) const;
+		QVariant getIcon(const dbheader *hd) const;
+		QByteArray i2d() const;
 		void d2i(QByteArray &ba);
 		void deleteFromToken();
 		void deleteFromToken(slotid slot);
-		virtual QString getMsg(msg_type msg);
-		virtual int renameOnToken(slotid slot, QString name);
+		QString getMsg(msg_type msg) const;
+		int renameOnToken(slotid slot, QString name);
 		BIO *pem(BIO *, int);
-		virtual QVariant bg_color(dbheader *hd);
-		void mergeRevList(x509revList l) {
-			revList.merge(l);
-		}
+		QVariant bg_color(const dbheader *hd) const;
+		void mergeRevList(x509revList &l);
 		void setRevocations(const x509revList &rl);
+		x509revList getRevList() const;
 		bool compareNameAndKey(pki_x509 *other);
+		void setCrlExpire(a1time a)
+		{
+			crlExpire = a;
+		}
+		QSqlError insertSqlData();
+		QSqlError deleteSqlData();
+		void restoreSql(const QSqlRecord &rec);
 };
 
+Q_DECLARE_METATYPE(pki_x509 *);
 #endif
