@@ -24,10 +24,8 @@ pki_x509req::pki_x509req(const QString name)
 	: pki_x509super(name)
 {
 	privkey = NULL;
-	class_name = "pki_x509req";
 	request = X509_REQ_new();
 	pki_openssl_error();
-	dataVersion=1;
 	pkiType=x509_req;
 	done = false;
 }
@@ -38,6 +36,42 @@ pki_x509req::~pki_x509req()
 		X509_REQ_free(request);
 }
 
+QSqlError pki_x509req::insertSqlData()
+{
+	XSqlQuery q;
+	QSqlError e = pki_x509super::insertSqlData();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "INSERT INTO requests (item, hash, signed, request) "
+		  "VALUES (?, ?, 0, ?)");
+	q.bindValue(0, sqlItemId);
+	q.bindValue(1, hash());
+	q.bindValue(2, i2d_b64());
+	q.exec();
+	return q.lastError();
+}
+
+void pki_x509req::restoreSql(const QSqlRecord &rec)
+{
+	pki_x509super::restoreSql(rec);
+	QByteArray ba = QByteArray::fromBase64(
+				rec.value(VIEW_x509req_request).toByteArray());
+	d2i(ba);
+	done = rec.value(VIEW_x509req_signed).toBool();
+}
+
+QSqlError pki_x509req::deleteSqlData()
+{
+	XSqlQuery q;
+	QSqlError e = pki_x509super::deleteSqlData();
+	if (e.isValid())
+		return e;
+	SQL_PREPARE(q, "DELETE FROM requests WHERE item=?");
+	q.bindValue(0, sqlItemId);
+	q.exec();
+	return q.lastError();
+}
+
 void pki_x509req::createReq(pki_key *key, const x509name &dn, const EVP_MD *md, extList el)
 {
 	int bad_nids[] = { NID_subject_key_identifier, NID_authority_key_identifier,
@@ -74,7 +108,7 @@ void pki_x509req::createReq(pki_key *key, const x509name &dn, const EVP_MD *md,
 	EVP_PKEY_free(privkey);
 }
 
-QString pki_x509req::getMsg(msg_type msg)
+QString pki_x509req::getMsg(msg_type msg) const
 {
 	/*
 	 * We do not construct english sentences from fragments
@@ -152,7 +186,7 @@ void pki_x509req::d2i(QByteArray &ba)
 	}
 }
 
-QByteArray pki_x509req::i2d()
+QByteArray pki_x509req::i2d() const
 {
 	return i2d_bytearray(I2D_VOID(i2d_X509_REQ), request);
 }
@@ -163,7 +197,15 @@ void pki_x509req::fromData(const unsigned char *p, db_header_t *head )
 
 	size = head->len - sizeof(db_header_t);
 
-	oldFromData((unsigned char *)p, size);
+	QByteArray ba((const char *)p, size);
+	privkey = NULL;
+
+	d2i(ba);
+	pki_openssl_error();
+
+	if (ba.count() > 0) {
+		my_error(tr("Wrong Size %1").arg(ba.count()));
+	}
 }
 
 void pki_x509req::addAttribute(int nid, QString content)
@@ -184,7 +226,7 @@ x509name pki_x509req::getSubject() const
 	return x;
 }
 
-int pki_x509req::sigAlg()
+int pki_x509req::sigAlg() const
 {
 	return X509_REQ_get_signature_nid(request);
 }
@@ -194,16 +236,9 @@ void pki_x509req::setSubject(const x509name &n)
 	X509_REQ_set_subject_name(request, n.get());
 }
 
-QByteArray pki_x509req::toData()
-{
-	QByteArray ba = i2d();
-	pki_openssl_error();
-	return ba;
-}
-
 void pki_x509req::writeDefault(const QString fname)
 {
-	writeReq(fname + QDir::separator() + getIntName() + ".csr", true);
+	writeReq(get_dump_filename(fname, ".csr"), true);
 }
 
 void pki_x509req::writeReq(const QString fname, bool pem)
@@ -231,7 +266,7 @@ BIO *pki_x509req::pem(BIO *b, int format)
 	return b;
 }
 
-int pki_x509req::verify()
+int pki_x509req::verify() const
 {
 	EVP_PKEY *pkey = X509_REQ_get_pubkey(request);
 	bool x = X509_REQ_verify(request,pkey) > 0;
@@ -251,7 +286,7 @@ pki_key *pki_x509req::getPubKey() const
 	 return key;
 }
 
-extList pki_x509req::getV3ext()
+extList pki_x509req::getV3ext() const
 {
 	extList el;
 	STACK_OF(X509_EXTENSION) *sk;
@@ -261,7 +296,7 @@ extList pki_x509req::getV3ext()
 	return el;
 }
 
-QString pki_x509req::getAttribute(int nid)
+QString pki_x509req::getAttribute(int nid) const
 {
 	int n;
 	int count;
@@ -280,7 +315,7 @@ QString pki_x509req::getAttribute(int nid)
 	return ret.join(", ");
 }
 
-QVariant pki_x509req::column_data(dbheader *hd)
+QVariant pki_x509req::column_data(const dbheader *hd) const
 {
 	switch (hd->id) {
 	case HD_req_signed:
@@ -293,7 +328,7 @@ QVariant pki_x509req::column_data(dbheader *hd)
 	return pki_x509super::column_data(hd);
 }
 
-QVariant pki_x509req::getIcon(dbheader *hd)
+QVariant pki_x509req::getIcon(const dbheader *hd) const
 {
 	int pixnum = -1;
 	pki_key *k;
@@ -315,7 +350,7 @@ QVariant pki_x509req::getIcon(dbheader *hd)
 	return QVariant(*icon[pixnum]);
 }
 
-bool pki_x509req::visible()
+bool pki_x509req::visible() const
 {
 	if (pki_x509super::visible())
 		return true;
@@ -325,11 +360,3 @@ bool pki_x509req::visible()
 		return true;
 	return false;
 }
-
-void pki_x509req::oldFromData(unsigned char *p, int size)
-{
-	QByteArray ba((const char *)p, size);
-	privkey = NULL;
-
-	d2i(ba);
-}
