@@ -15,6 +15,8 @@
 #include "pki_x509super.h"
 #include "x509name.h"
 
+#define VIEW_x509req_request 7
+#define VIEW_x509req_signed 8
 
 class pki_x509;
 
@@ -25,10 +27,10 @@ class pki_x509req : public pki_x509super
 	protected:
 		X509_REQ *request;
 		bool done;
-		int sigAlg();
+		int sigAlg() const;
 
 	public:
-		extList getV3ext();
+		extList getV3ext() const;
 		static QPixmap *icon[3];
 		pki_x509req(QString name = "");
 		void fromPEM_BIO(BIO *bio, QString name);
@@ -36,8 +38,6 @@ class pki_x509req : public pki_x509super
 		void writeDefault(const QString fname);
 		~pki_x509req();
 		void fromData(const unsigned char *p, db_header_t *head);
-		void oldFromData(unsigned char *p, int size);
-		QByteArray toData();
 		x509name getSubject() const;
 		void writeReq(const QString fname, bool pem);
 		X509_REQ *getReq()
@@ -45,16 +45,15 @@ class pki_x509req : public pki_x509super
 			return request;
 		}
 		void addAttribute(int nid, QString content);
-		QString getAttribute(int nid);
+		QString getAttribute(int nid) const;
 
-		int verify();
+		int verify() const;
 		pki_key *getPubKey() const;
-		void createReq(pki_key *key, const x509name &dn, const EVP_MD *md,
-		   extList el);
+		void createReq(pki_key *key, const x509name &dn,
+				const EVP_MD *md, extList el);
 		void setSubject(const x509name &n);
-		/* SPKAC special functions */
-		QVariant column_data(dbheader *hd);
-		QVariant getIcon(dbheader *hd);
+		QVariant column_data(const dbheader *hd) const;
+		QVariant getIcon(const dbheader *hd) const;
 		void setDone(bool d = true)
 		{
 			done = d;
@@ -63,11 +62,15 @@ class pki_x509req : public pki_x509super
 		{
 			return done;
 		}
-		virtual QString getMsg(msg_type msg);
+		virtual QString getMsg(msg_type msg) const;
 		void d2i(QByteArray &ba);
-		QByteArray i2d();
+		QByteArray i2d() const;
 		BIO *pem(BIO *, int);
-		bool visible();
+		bool visible() const;
+		QSqlError insertSqlData();
+		QSqlError deleteSqlData();
+		void restoreSql(const QSqlRecord &rec);
 };
 
+Q_DECLARE_METATYPE(pki_x509req *);
 #endif
