@@ -27,15 +27,16 @@ class x509name
 		QString oneLine(unsigned long flags = XN_FLAG_ONELINE) const;
 		int nid(int i) const;
 		QString getOid(int i) const;
-		QByteArray i2d();
+		QByteArray i2d() const;
 		void d2i(QByteArray &ba);
 		QStringList entryList(int i) const;
-		QString getEntryByNid(int nid ) const;
+		QString getEntryByNid(int nid) const;
 		QString getEntry(int i) const;
 		QString getEntryTag(int i) const;
 		int entryCount() const;
 		x509name &operator = (const x509name &x);
 		bool operator == (const x509name &x) const;
+		bool operator != (const x509name &x) const;
 		static int getNidByName(const QString &nid_name);
 		void addEntryByNid(int nid, const QString entry);
 		QString checkLength() const;
@@ -45,7 +46,8 @@ class x509name
 		QString getMostPopular() const;
 		QString taggedValues() const;
 		QString hash() const;
-		bool search(const QRegExp &pattern);
+		unsigned long hashNum() const;
+		bool search(const QRegExp &pattern) const;
 };
 
 #endif
