@@ -7,11 +7,13 @@
 
 #include "x509rev.h"
 #include "db.h"
-#include "base.h"
+#include "pki_base.h"
 #include "func.h"
 #include "exception.h"
 #include <openssl/x509v3.h>
 #include <QStringList>
+#include <QVariant>
+#include <QSqlQuery>
 
 #include "openssl_compat.h"
 
@@ -56,6 +58,16 @@ QString x509rev::getReason() const
 	return crl_reasons[reason_idx].lname;
 }
 
+static int reasonBit2Idx(int bit)
+{
+	for (int i=0; crl_reasons[i].lname; i++) {
+		if (bit == crl_reasons[i].bitnum) {
+			return i;
+		}
+	}
+	return 0;
+}
+
 void x509rev::fromREVOKED(const X509_REVOKED *rev)
 {
 	ASN1_ENUMERATED *reason;
@@ -74,11 +86,7 @@ void x509rev::fromREVOKED(const X509_REVOKED *rev)
 	if (reason) {
 		r = ASN1_ENUMERATED_get(reason);
 		openssl_error();
-		for (int i=0; crl_reasons[i].lname; i++) {
-			if (r == crl_reasons[i].bitnum) {
-				reason_idx = i;
-			}
-		}
+		reason_idx = reasonBit2Idx(r);
 		ASN1_ENUMERATED_free(reason);
 	}
 	ivalDate.setUndefined();
@@ -89,7 +97,7 @@ void x509rev::fromREVOKED(const X509_REVOKED *rev)
 		ivalDate = a1time(at);
 		ASN1_GENERALIZEDTIME_free(at);
 	}
-	//dump();
+	qDebug() << *this;
 }
 
 X509_REVOKED *x509rev::toREVOKED(bool withReason) const
@@ -110,7 +118,7 @@ X509_REVOKED *x509rev::toREVOKED(bool withReason) const
 		ASN1_ENUMERATED_free(a);
 	}
 	openssl_error();
-	//dump();
+	qDebug() << *this;
 	return rev;
 }
 
@@ -139,6 +147,7 @@ void x509rev::set(const x509rev &x)
 	date = x.date;
 	ivalDate = x.ivalDate;
 	reason_idx = x.reason_idx;
+	crlNo = x.crlNo;
 }
 
 bool x509rev::identical(const x509rev &x) const
@@ -149,12 +158,33 @@ bool x509rev::identical(const x509rev &x) const
 		reason_idx == x.reason_idx;
 }
 
-void x509rev::dump() const
+x509rev::operator QString() const
 {
-	fprintf(stderr, "Rev: %s D:%s I:%s Reason: %d '%s'\n",
-		CCHAR(serial.toHex()), CCHAR(date.toSortable()),
-		CCHAR(ivalDate.toSortable()), reason_idx,
-		crl_reasons[reason_idx].lname);
+	return QString("Rev: %1 D:%2 I:%3 Reason: %4 '%5'\n")
+		.arg(serial.toHex(), date.toSortable(), ivalDate.toSortable())
+		.arg(reason_idx).arg(crl_reasons[reason_idx].lname);
+}
+
+x509rev::x509rev(QSqlRecord rec, int offset)
+{
+	qDebug() << "QSqlRecord offset:" << offset;
+	serial.setHex(rec.value(offset).toString());
+	date.fromPlain(rec.value(offset +1).toString());
+	ivalDate.fromPlain(rec.value(offset +2).toString());
+	crlNo = rec.value(offset +3).toInt();
+	reason_idx = reasonBit2Idx(rec.value(offset +4).toInt());
+	qDebug() << *this;
+}
+
+void x509rev::executeQuery(XSqlQuery &q)
+{
+	// 0 is the caId
+	q.bindValue(1, serial.toHex());
+	q.bindValue(2, date.toPlain());
+	q.bindValue(3, ivalDate.toPlain());
+	q.bindValue(4, crlNo ? QVariant(crlNo) : QVariant());
+	q.bindValue(5, crl_reasons[reason_idx].bitnum);
+	q.exec();
 }
 
 void x509revList::fromBA(QByteArray &ba)
@@ -204,3 +234,61 @@ bool x509revList::identical(const x509revList &other) const
 	}
 	return true;
 }
+
+x509revList x509revList::fromSql(QVariant caId)
+{
+	XSqlQuery q;
+	x509revList list;
+
+	SQL_PREPARE(q, "SELECT serial, date, invaldate, crlNo, reasonBit "
+			"FROM revocations WHERE caId=?");
+	q.bindValue(0, caId);
+	q.exec();
+	if (q.lastError().isValid())
+		return list;
+	while (q.next()) {
+		x509rev r(q.record());
+		list.append(r);
+	}
+	list.merged = false;
+	return list;
+}
+
+bool x509revList::sqlUpdate(QVariant caId)
+{
+	XSqlQuery q;
+	Transaction;
+
+	if (!TransBegin())
+		return false;
+
+	x509revList oldList = fromSql(caId);
+
+	SQL_PREPARE(q, "DELETE FROM revocations WHERE caId=?");
+	q.bindValue(0, caId);
+	q.exec();
+	if (q.lastError().isValid())
+		return false;
+
+	SQL_PREPARE(q, "INSERT INTO revocations "
+			"(caId, serial, date, invaldate, crlNo, reasonBit) "
+			"VALUES (?,?,?,?,?,?)");
+	q.bindValue(0, caId);
+	foreach(x509rev r, *this) {
+		if (r.getCrlNo() == 0) {
+			int idx = oldList.indexOf(r);
+			if (idx != -1) {
+				x509rev old = oldList.takeAt(idx);
+				r.setCrlNo(old.getCrlNo());
+				qDebug() << "RECOVER OLD CRL NO" << r ;
+			}
+		}
+		r.executeQuery(q);
+		if (q.lastError().isValid())
+			return false;
+	}
+
+        merged = false;
+	TransCommit();
+	return true;
+}
