@@ -21,8 +21,9 @@ CertDetail::CertDetail(QWidget *parent)
 {
 	setupUi(this);
 	setWindowTitle(XCA_TITLE);
-	descr->setReadOnly(true);
 	showConf = false;
+	keySqlId = QVariant();
+	issuerSqlId = QVariant();
 }
 
 void CertDetail::on_showExt_clicked()
@@ -47,11 +48,13 @@ void CertDetail::setX509super(pki_x509super *x)
 	pki_key *key= x->getRefKey();
 	if (key) {
 		privKey->setText(key->getIntName());
+		privKey->setClickText(key->getSqlItemId().toString());
 		if (key->isPrivKey()) {
 			privKey->setGreen();
 		} else {
 			privKey->setRed();
 		}
+		keySqlId = key->getSqlItemId();
 	} else {
 		privKey->setText(tr("Not available"));
 		privKey->setDisabled(true);
@@ -73,6 +76,11 @@ void CertDetail::setX509super(pki_x509super *x)
 
 	// Algorithm
 	sigAlgo->setText(x->getSigAlg());
+	connect(sigAlgo, SIGNAL(doubleClicked(QString)),
+		MainWindow::getResolver(), SLOT(searchOid(QString)));
+
+	// Comment
+	comment->setPlainText(x->getComment());
 }
 
 void CertDetail::setCert(pki_x509 *cert)
@@ -95,26 +103,18 @@ void CertDetail::setCert(pki_x509 *cert)
 			signature->setGreen();
 			signature->disableToolTip();
 		} else {
-			signature->setText(cert->getSigner()->getIntName());
+			pki_x509 *issuer = cert->getSigner();
+			signature->setText(issuer->getIntName());
+			signature->setClickText(issuer->getSqlItemId().toString());
 			signature->setGreen();
-		}
-
-		// check trust state
-		trustState->disableToolTip();
-		if (cert->getEffTrust() == 0) {
-			trustState->setText(tr("Not trusted"));
-			trustState->setRed();
-		}
-		else {
-			trustState->setText(tr("Trusted"));
-			trustState->setGreen();
+			issuerSqlId = issuer->getSqlItemId();
 		}
 
 		// the serial
-		serialNr->setText(cert->getSerial().toHex());
+		serialNr->setText(cert->getSerial());
 
 		// details of the issuer
-		issuer->setX509name(cert->getIssuer());
+		issuer->setX509name(cert->getIssuerName());
 
 		// The dates
 		notBefore->setText(cert->getNotBefore().toPretty());
@@ -170,7 +170,6 @@ void CertDetail::setReq(pki_x509req *req)
 			signature->setText("PKCS#10");
 		}
 		signature->disableToolTip();
-		trustState->hide();
 		fingerprints->hide();
 		validity->hide();
 		serialLabel->hide();
@@ -197,7 +196,7 @@ void CertDetail::setReq(pki_x509req *req)
 			}
 			label = new QLabel(this);
 			trans = dn_translations[nid];
-			if (translate_dn && !trans.isEmpty()) {
+			if (Settings["translate_dn"] && !trans.isEmpty()) {
 				label->setText(trans);
 				label->setToolTip(QString(OBJ_nid2sn(nid)));
 			} else {
@@ -235,3 +234,12 @@ QLabel *CertDetail::labelFromAsn1String(ASN1_STRING *s)
 	label->setToolTip(QString(ASN1_tag2str(s->type)));
 	return label;
 }
+
+void CertDetail::itemChanged(pki_base *pki)
+{
+	if (pki->getSqlItemId() == keySqlId)
+		privKey->setText(pki->getIntName());
+
+	if (pki->getSqlItemId() == issuerSqlId)
+		signature->setText(pki->getIntName());
+}
