@@ -30,7 +30,7 @@ ExportDialog::ExportDialog(MainWindow *mw, QString title, QString filt,
 	image->setPixmap(*img);
         label->setText(title);
 	if (pki) {
-		QString fn = mainwin->getPath() + QDir::separator() +
+		QString fn = Settings["workingdir"] + QDir::separator() +
 			pki->getUnderlinedName() + "." + types[0].extension;
 		filename->setText(fn);
 	}
@@ -53,16 +53,16 @@ ExportDialog::ExportDialog(MainWindow *mw, QString title, QString filt,
 	help[exportType::PEM_selected] =
 		tr("Concatenated list of all selected items in one PEM text file");
 	help[exportType::PEM_chain] = tr("Concatenated text format of the complete certificate chain in one PEM file");
-	help[exportType::PEM_trusted] =
-		tr("Concatenated text format of all trusted certificates in one PEM file");
+	help[exportType::PEM_unrevoked] =
+		tr("Concatenated text format of all unrevoked certificates in one PEM file");
 	help[exportType::PEM_all] =
 		tr("Concatenated text format of all certificates in one PEM file");
 	help[exportType::DER] = tr("Binary DER encoded file");
 	help[exportType::PKCS7] = tr("PKCS#7 encoded single certificate");
 	help[exportType::PKCS7_chain] =
 		tr("PKCS#7 encoded complete certificate chain");
-	help[exportType::PKCS7_trusted] =
-		tr("All trusted certificates encoded in one PKCS#7 file");
+	help[exportType::PKCS7_unrevoked] =
+		tr("All unrevoked certificates encoded in one PKCS#7 file");
 	help[exportType::PKCS7_selected] =
 		tr("All selected certificates encoded in one PKCS#7 file");
 	help[exportType::PKCS7_all] =
@@ -85,7 +85,7 @@ ExportDialog::ExportDialog(MainWindow *mw, QString title, QString filt,
 	help[exportType::PKCS8_encrypt] =
 		tr("Encrypted private key in PKCS#8 text format");
 	help[exportType::SSH2_public] = tr("The public key encoded in SSH2 format");
-	help[exportType::Index] = tr("Certificate Index file");
+	help[exportType::Index] = tr("OpenSSL specific Certificate Index file as created by the 'ca' command and required by the OCSP tool");
 
 	on_exportFormat_highlighted(0);
 }
@@ -150,7 +150,8 @@ void ExportDialog::accept()
 		return;
 	}
 	if (mayWriteFile(fn)) {
-		mainwin->setPath(fn.mid(0, fn.lastIndexOf(QRegExp("[/\\\\]"))));
+		Settings["workingdir"] = fn.mid(0, fn.lastIndexOf(
+							QRegExp("[/\\\\]")));
 		QDialog::accept();
 	}
 }
