@@ -24,6 +24,7 @@
 #include <QInputDialog>
 #include <QUrl>
 #include <QMimeData>
+#include <typeinfo>
 
 ImportMulti::ImportMulti(MainWindow *parent)
 	:QDialog(parent)
@@ -33,7 +34,7 @@ ImportMulti::ImportMulti(MainWindow *parent)
 	setWindowTitle(XCA_TITLE);
 	image->setPixmap(*MainWindow::certImg);
 	listView->setEditTriggers(QAbstractItemView::NoEditTriggers);
-	mcont = new db_token(QString(), parent);
+	mcont = new db_token(parent);
 	listView->setModel(mcont);
 	listView->setIconSize(pki_evp::icon[0]->size());
 	listView->setSelectionMode(QAbstractItemView::ExtendedSelection);
@@ -71,22 +72,27 @@ void ImportMulti::addItem(pki_base *pki)
 {
 	if (!pki)
 		return;
-	QString cn = pki->getClassName();
 
-	if (cn == "pki_x509" || cn == "pki_evp"  || cn == "pki_x509req" ||
-	    cn == "pki_crl"  || cn == "pki_temp" || cn == "pki_scard")
+	if (pki->pkiSource == unknown)
+		pki->pkiSource = imported;
+
+	const std::type_info &t = typeid(*pki);
+
+	if (t == typeid(pki_x509)    || t == typeid(pki_evp) ||
+	    t == typeid(pki_x509req) || t == typeid(pki_crl) ||
+	    t == typeid(pki_temp)    || t == typeid(pki_scard))
 	{
 		mcont->inToCont(pki);
 	}
-	else if (cn == "pki_pkcs7") {
-		pki_pkcs7 *p7 = ( pki_pkcs7 *)pki;
+	else if (t == typeid(pki_pkcs7)) {
+		pki_pkcs7 *p7 = static_cast<pki_pkcs7 *>(pki);
 		for (int i=0; i<p7->numCert(); i++) {
 			addItem(p7->getCert(i));
 		}
 		delete p7;
 	}
-	else if (cn == "pki_pkcs12") {
-		pki_pkcs12 *p12 = ( pki_pkcs12 *)pki;
+	else if (t == typeid(pki_pkcs12)) {
+		pki_pkcs12 *p12 = static_cast<pki_pkcs12 *>(pki);
 		addItem(p12->getKey());
 		addItem(p12->getCert());
 		for (int i=0; i<p12->numCa(); i++) {
@@ -94,8 +100,8 @@ void ImportMulti::addItem(pki_base *pki)
 		}
 		delete p12;
 	}
-	else if (cn == "pki_multi") {
-		pki_multi *pm = (pki_multi*)pki;
+	else if (t == typeid(pki_multi)) {
+		pki_multi *pm = static_cast<pki_multi*>(pki);
 		pki_base *inner;
 		while ((inner = pm->pull()))
 			addItem(inner);
@@ -103,8 +109,16 @@ void ImportMulti::addItem(pki_base *pki)
 	}
 	else  {
 		XCA_WARN(tr("The type of the item '%1' is not recognized").
-			arg(cn));
+			arg(t.name()));
 	}
+	if (t == typeid(pki_scard))
+		mcont->rename_token_in_database(dynamic_cast<pki_scard*>(pki));
+}
+
+void ImportMulti::openDB()
+{
+	if (!mainwin->keys)
+		mainwin->load_database();
 }
 
 void ImportMulti::dragEnterEvent(QDragEnterEvent *event)
@@ -150,10 +164,16 @@ void ImportMulti::on_butRemove_clicked()
 
 void ImportMulti::on_butOk_clicked()
 {
+	openDB();
+
+	Transaction;
+	if (!TransBegin())
+		return;
 	while (mcont->rootItem->childCount()) {
 		QModelIndex idx = mcont->index(0, 0, QModelIndex());
 		import(idx);
 	}
+	TransCommit();
 	accept();
 }
 
@@ -161,13 +181,18 @@ void ImportMulti::on_butImport_clicked()
 {
 	QItemSelectionModel *selectionModel = listView->selectionModel();
 	QModelIndexList indexes = selectionModel->selectedIndexes();
-	QModelIndex index;
 
-	foreach(index, indexes) {
+	openDB();
+
+	Transaction;
+	if (!TransBegin())
+		return;
+	foreach(QModelIndex index, indexes) {
 		if (index.column() != 0)
 			continue;
 		import(index);
 	}
+	TransCommit();
 }
 
 void ImportMulti::on_deleteToken_clicked()
@@ -205,20 +230,14 @@ void ImportMulti::on_renameToken_clicked()
 	}
 }
 
-static db_base *select_db(QString cn)
+static db_base *select_db(const std::type_info &t)
 {
-	if (cn == "pki_x509")
-		return MainWindow::certs;
-	if (cn == "pki_evp")
-		return MainWindow::keys;
-	if (cn == "pki_scard")
-		return MainWindow::keys;
-	if (cn == "pki_x509req")
-		return MainWindow::reqs;
-	if (cn == "pki_crl")
-		return MainWindow::crls;
-	if (cn == "pki_temp")
-		return MainWindow::temps;
+	if (t == typeid(pki_x509))    return MainWindow::certs;
+	if (t == typeid(pki_evp))     return MainWindow::keys;
+	if (t == typeid(pki_scard))   return MainWindow::keys;
+	if (t == typeid(pki_x509req)) return MainWindow::reqs;
+	if (t == typeid(pki_crl))     return MainWindow::crls;
+	if (t == typeid(pki_temp))    return MainWindow::temps;
 	return NULL;
 }
 
@@ -228,26 +247,24 @@ pki_base *ImportMulti::import(QModelIndex &idx)
 	pki_base *pki = static_cast<pki_base*>(idx.internalPointer());
 	db_base *db;
 
-	if (!pki)
+	if (!pki || !mainwin->keys)
 		return NULL;
-	if (!mainwin->keys) {
-		mainwin->load_database();
-	}
 
-	QString cn = pki->getClassName();
+	const std::type_info &t = typeid(*pki);
+
 	mcont->remFromCont(idx);
 	if (!mainwin->keys) {
 		delete pki;
 		return NULL;
 	}
 
-	if (cn == "pki_evp")
-		((pki_evp*)pki)->setOwnPass(pki_evp::ptCommon);
+	if (t == typeid(pki_evp))
+		static_cast<pki_evp*>(pki)->setOwnPass(pki_evp::ptCommon);
 
-	db = select_db(cn);
+	db = select_db(t);
 	if (!db) {
 		XCA_WARN(tr("The type of the item '%1' is not recognized").
-			arg(cn));
+			arg(t.name()));
 		delete pki;
 		return NULL;
 	}
@@ -267,37 +284,37 @@ void ImportMulti::on_butDetails_clicked()
 
 	if (!pki)
 		return;
-	QString cn = pki->getClassName();
+	const std::type_info &t = typeid(*pki);
 	try {
-		if (cn == "pki_x509"){
+		if (t == typeid(pki_x509)){
 			CertDetail *dlg;
 			dlg = new CertDetail(mainwin);
-			dlg->setCert((pki_x509 *)pki);
+			dlg->setCert(static_cast<pki_x509 *>(pki));
 			dlg->exec();
 			delete dlg;
-		} else if (cn == "pki_evp" || cn == "pki_scard") {
+		} else if (t == typeid(pki_evp) || t == typeid(pki_scard)) {
 			KeyDetail *dlg;
 			dlg = new KeyDetail(mainwin);
-			dlg->setKey((pki_key *)pki);
+			dlg->setKey(static_cast<pki_key *>(pki));
 			dlg->exec();
 			delete dlg;
-		} else if (cn == "pki_x509req") {
+		} else if (t == typeid(pki_x509req)) {
 			CertDetail *dlg;
 			dlg = new CertDetail(mainwin);
-			dlg->setReq((pki_x509req *)pki);
+			dlg->setReq(static_cast<pki_x509req *>(pki));
 			dlg->exec();
 			delete dlg;
-		} else if (cn == "pki_crl") {
+		} else if (t == typeid(pki_crl)) {
 			CrlDetail *dlg;
 			dlg = new CrlDetail(mainwin);
-			dlg->setCrl((pki_crl *)pki);
+			dlg->setCrl(static_cast<pki_crl *>(pki));
 			dlg->exec();
 			delete dlg;
-		} else if (cn == "pki_temp") {
-			XCA_WARN(tr("Details of the item '%1' cannot be shown").
-				arg(cn));
+		} else if (t == typeid(pki_temp)) {
+			XCA_WARN(tr("Details of the item '%1' cannot be shown")
+				.arg("XCA template"));
 		} else
-			XCA_WARN(tr("The type of the item '%1' is not recognized").arg(cn));
+			XCA_WARN(tr("The type of the item '%1' is not recognized").arg(t.name()));
 	}
 	catch (errorEx &err) {
 		mainwin->Error(err);
@@ -346,9 +363,10 @@ void ImportMulti::execute(int force, QStringList failed)
 	}
 	/* if there is only 1 item and force is 0 import it silently */
 	if (entries() == 1 && force == 0) {
+		openDB();
 		QModelIndex idx = mcont->index(0, 0, QModelIndex());
 		pki_base *pki = import(idx);
-		if (pki && !pki_base::suppress_messages)
+		if (pki && !Settings["suppress_messages"])
 			XCA_INFO(pki->getMsg(pki_base::msg_import).
 				arg(pki->getIntName()));
 		accept();
