@@ -21,10 +21,10 @@ KeyDetail::KeyDetail(QWidget *parent)
 	:QDialog(parent)
 {
 	setupUi(this);
-	setWindowTitle(tr(XCA_TITLE));
+	setWindowTitle(XCA_TITLE);
 	image->setPixmap(*MainWindow::keyImg);
-	keyDesc->setReadOnly(true);
 	keyModulus->setFont(XCA_application::tableFont);
+	tabWidget->setCurrentIndex(0);
 }
 
 #ifndef OPENSSL_NO_EC
@@ -45,21 +45,23 @@ void KeyDetail::setKey(pki_key *key)
 
 	keyPrivEx->disableToolTip();
 	if (!key->isToken())
-		cardBox->hide();
+		tabWidget->removeTab(1);
 	tlHeader->setText(tr("Details of the %1 key").arg(key->getTypeString()));
+	comment->setPlainText(key->getComment());
 
 	if (key->isPubKey()) {
 		keyPrivEx->setText(tr("Not available"));
 		keyPrivEx->setRed();
 	} else if (key->isToken()) {
 		image->setPixmap(*MainWindow::scardImg);
-		pki_scard *card = (pki_scard *)key;
-		cardBox->setTitle(tr("Token") +" [" +card->getCardLabel() +"]");
-		cardManufacturer->setText(card->getManufacturer() + " " +
-					card->getModel());
+		pki_scard *card = static_cast<pki_scard *>(key);
+		cardLabel->setText(card->getCardLabel());
+		cardModel->setText(card->getModel());
+		cardManufacturer->setText(card->getManufacturer());
 		cardSerial->setText(card->getSerial());
-		keyPrivEx->setText(tr("Security token ID:%1").arg(card->getId()));
-		keyBox->setTitle(tr("Key") + " [" + card->getLabel() + "]");
+		slotLabel->setText(card->getLabel());
+		cardId->setText(card->getId());
+		keyPrivEx->setText(tr("Security token"));
 	} else {
 		keyPrivEx->setText(tr("Available"));
 		keyPrivEx->setGreen();
@@ -71,7 +73,7 @@ void KeyDetail::setKey(pki_key *key)
 			break;
 		case EVP_PKEY_DSA:
 			tlPubEx->setText(tr("Sub prime"));
-			tlModulus->setText(tr("Public key"));
+			tlModulus->setTitle(tr("Public key"));
 			tlPrivEx->setText(tr("Private key"));
 			keyPubEx->setText(key->subprime());
 			keyModulus->setText(key->pubkey());
@@ -80,7 +82,7 @@ void KeyDetail::setKey(pki_key *key)
 		case EVP_PKEY_EC:
 			int nid;
 			nid = key->ecParamNid();
-			tlModulus->setText(tr("Public key"));
+			tlModulus->setTitle(tr("Public key"));
 			tlPrivEx->setText(tr("Private key"));
 			tlPubEx->setText(tr("Curve name"));
 			keyPubEx->setText(OBJ_nid2sn(nid));
