@@ -14,7 +14,7 @@
 #ifndef OPENSSL_NO_EC
 #include <openssl/ec.h>
 #endif
-#include "ui_About.h"
+#include "XcaDialog.h"
 #include "ui_Help.h"
 #include "lib/func.h"
 #include "lib/entropy.h"
@@ -31,7 +31,6 @@ void MainWindow::cmd_help(const char* msg)
 		"\n"
 		" -v show version information and exit\n"
 		" -h shows this help screen and exit\n"
-		" -e <type>:<name>  extract entry.\n"
 		" -d expect the following argument to be the database name to use\n"
 		" -i expect the following argument to be the index file to generate\n"
 		" -I expect the following argument to be the base name the index file hierarchy to generate\n"
@@ -45,8 +44,10 @@ void MainWindow::cmd_help(const char* msg)
 
 void MainWindow::about()
 {
-	Ui::About ui;
-	QDialog *about = new QDialog(this, 0);
+	QTextEdit *textbox = new QTextEdit(NULL);
+	XcaDialog *about = new XcaDialog(this, x509, textbox,
+					QString(), QString());
+	about->aboutDialog(scardImg);
 	QString openssl, qt, cont, version, brainpool;
 #ifndef OPENSSL_NO_EC
 #ifdef NID_brainpoolP160r1
@@ -81,7 +82,6 @@ void MainWindow::about()
 	} else {
 		version = QString("%1<br>QT version: %2").arg(openssl).arg(qt);
 	}
-	ui.setupUi(about);
 	Entropy::seed_rng();
 	cont = QString(
 	"<p><h3><center><u>XCA</u></center></h3>"
@@ -89,8 +89,12 @@ void MainWindow::about()
 	"<p>Version: %4<p>%1<p>%2" /* commithash, Brainpool, OpenSSL & Qt Version */
 	"<p>http://hohnstaedt.de/xca"
 	"<p>Entropy strength: %3"
-	"<p>Installation path: <i>%5</i><br/>User settings path: <i>%6</i>"
-	"<hr><table border=0>"
+	"<p><table border=\"0\">"
+	"<tr><td>Installation path:</td><td>%5</td></tr>"
+	"<tr><td>User settings path:</td><td>%6</td></tr>"
+	"<tr><td>Working directory:</td><td>%7</td></tr>"
+	"</table>"
+	"<hr><table border=\"0\">"
 	"<tr><th align=left>Christian Hohnst&auml;dt</th><td><u>&lt;christian@hohnstaedt.de&gt;</u></td></tr>"
 	"<tr><td></td><td>Programming, Translation and Testing</td></tr>"
 	"<tr><th align=left>Kerstin Steinhauff</th><td><u>&lt;tine@kerstine.de&gt;</td></u></tr>"
@@ -98,17 +102,16 @@ void MainWindow::about()
 	"</table><hr><center><u><b>Maintained Translations</b></u></center>"
 	"<p><table>"
 	"<tr><td><b>German</b></td><td>Christian Hohnst&auml;dt &lt;christian@hohnstaedt.de&gt;</td></tr>"
-	"<tr><td><b>French</b></td><td>Patrick Monnerat &lt;Patrick.Monnerat@datasphere.ch&gt;</td></tr>"
+	"<tr><td><b>French</b></td><td>Patrick Monnerat &lt;patrick@monnerat.net&gt;</td></tr>"
 	"<tr><td><b>Croatian</b></td><td>Nevenko Bartolincic &lt;nevenko.bartolincic@gmail.com&gt;</td></tr>"
 	"<tr><td><b>Slovak</b></td><td>Slavko &lt;linux@slavino.sk&gt;</td></tr>"
 	"</table>").arg(brainpool).arg(version).arg(Entropy::strength())
 			.arg(version_str(true)).arg(getPrefix())
-			.arg(getUserSettingsDir());
+			.arg(getUserSettingsDir())
+			.arg(QString(Settings["workingdir"]));
 
-	about->setWindowTitle(XCA_TITLE);
-	ui.image->setPixmap( *keyImg );
-	ui.image1->setPixmap( *certImg );
-	ui.textbox->setHtml(cont);
+	textbox->setHtml(cont);
+	textbox->setReadOnly(true);
 	about->exec();
 	delete about;
 }
@@ -132,7 +135,7 @@ void MainWindow::help()
 
 	ui.textbox->setSource(QUrl(uri));
 	ui.textbox->setSearchPaths(QStringList(path));
-	h->setWindowTitle(tr(XCA_TITLE));
+	h->setWindowTitle(XCA_TITLE);
 	h->show();
 }
 
