@@ -38,8 +38,9 @@
 #include "lib/func.h"
 #include "lib/pkcs11.h"
 #include "lib/builtin_curves.h"
-#include "ui_About.h"
+#include "XcaDialog.h"
 #include "PwDialog.h"
+#include "OpenDb.h"
 
 QPixmap *MainWindow::keyImg = NULL, *MainWindow::csrImg = NULL,
 	*MainWindow::certImg = NULL, *MainWindow::tempImg = NULL,
@@ -56,10 +57,6 @@ db_crl	*MainWindow::crls = NULL;
 NIDlist *MainWindow::eku_nid = NULL;
 NIDlist *MainWindow::dn_nid = NULL;
 
-QString MainWindow::mandatory_dn;
-QString MainWindow::explicit_dn;
-QString MainWindow::explicit_dn_default = QString("C,ST,L,O,OU,CN,emailAddress");
-
 OidResolver *MainWindow::resolver = NULL;
 
 void MainWindow::enableTokenMenu(bool enable)
@@ -72,7 +69,7 @@ void MainWindow::enableTokenMenu(bool enable)
 void MainWindow::load_engine()
 {
 	try {
-		pkcs11::load_libs(pkcs11path, false);
+		pkcs11::load_libs(Settings["pkcs11path"], false);
 	} catch (errorEx &err) {
 		Error(err);
 	}
@@ -113,6 +110,12 @@ MainWindow::MainWindow(QWidget *parent)
 	wdList << keyButtons << reqButtons << certButtons <<
 		tempButtons <<	crlButtons;
 
+	QStringList drivers = QSqlDatabase::drivers();
+	foreach(QString driver, drivers) {
+//		QSqlDatabase d = QSqlDatabase::addDatabase(driver, driver +"_C");
+		qDebug() << "DB driver:" << driver;
+	}
+
 	historyMenu = NULL;
 	init_menu();
 	setItemEnabled(false);
@@ -123,7 +126,7 @@ MainWindow::MainWindow(QWidget *parent)
 #ifdef MDEBUG
 	CRYPTO_malloc_debug_init();
 	CRYPTO_mem_ctrl(CRYPTO_MEM_CHECK_ON);
-	fprintf(stderr, "malloc() debugging on.\n");
+	qWarning() << "malloc() debugging on.";
 #endif
 
 	ERR_load_crypto_strings();
@@ -151,6 +154,7 @@ MainWindow::MainWindow(QWidget *parent)
 	certView->setMainwin(this, searchEdit);
 	tempView->setMainwin(this, searchEdit);
 	crlView->setMainwin(this, searchEdit);
+	keys = NULL; reqs = NULL; certs = NULL; temps = NULL; crls = NULL;
 }
 
 void MainWindow::dropEvent(QDropEvent *event)
@@ -185,10 +189,12 @@ void MainWindow::openURLs()
 	ImportMulti *dlgi = new ImportMulti(this);
 
 	foreach(s, urlsToOpen) {
-	        pki_multi *pki = probeAnything(s);
-		if (pki && !pki->count())
+		int ret;
+		pki_multi *pki = probeAnything(s, &ret);
+		if (ret)
 			failed << s;
-	        dlgi->addItem(pki);
+		else
+			dlgi->addItem(pki);
 	}
 	urlsToOpen.clear();
 	dlgi->execute(1, failed);
@@ -338,10 +344,12 @@ void MainWindow::read_cmdline(int argc, char *argv[])
 		} else {
 			int ret;
 			pki_multi *pki = probeAnything(file, &ret);
-			if (!pki && ret == 2)
-				 exitApp = 1;
-			if (pki && !pki->count())
-				failed << file;
+			if (!pki) {
+				if (ret == 2)
+					exitApp = 1;
+				else if (ret == 1)
+					failed << file;
+			}
 			dlgi->addItem(pki);
 		}
 		cnt++;
@@ -359,7 +367,7 @@ void MainWindow::loadPem()
 		keys->load_default(l);
 }
 
-bool MainWindow::pastePem(QString text)
+bool MainWindow::pastePem(QString text, bool silent)
 {
 	bool success = false;
 	QByteArray pemdata = text.toLatin1();
@@ -378,7 +386,8 @@ bool MainWindow::pastePem(QString text)
 		dlgi->execute(1);
 	}
 	catch (errorEx &err) {
-		Error(err);
+		if (!silent)
+			Error(err);
 	}
 	if (dlgi)
 		delete dlgi;
@@ -389,7 +398,6 @@ bool MainWindow::pastePem(QString text)
 
 void MainWindow::pastePem()
 {
-	Ui::About ui;
 	QClipboard *cb = QApplication::clipboard();
 	QString text;
 
@@ -398,25 +406,21 @@ void MainWindow::pastePem()
 		text = cb->text(QClipboard::Clipboard);
 
 	if (!text.isEmpty())
-		if (pastePem(text))
+		if (pastePem(text, true))
 			return;
 
-	QDialog *input = new QDialog(this, 0);
 
-	ui.setupUi(input);
-	delete ui.textbox;
-	QTextEdit *textbox = new QTextEdit(input);
-	ui.vboxLayout->addWidget(textbox);
-	ui.button->setText(tr("Import PEM data"));
-	input->setWindowTitle(XCA_TITLE);
+	QTextEdit *textbox = new QTextEdit();
 	textbox->setPlainText(text);
+	XcaDialog *input = new XcaDialog(this, x509, textbox,
+			tr("Import PEM data"), QString());
+	input->noSpacer();
 	if (input->exec()) {
 		text = textbox->toPlainText();
 		if (!text.isEmpty())
 			pastePem(text);
 	}
 	delete input;
-
 }
 
 void MainWindow::initToken()
@@ -550,7 +554,6 @@ void MainWindow::manageToken()
 			cert = new pki_x509("");
 			try {
 				cert->load_token(p11, objects[j]);
-				cert->setTrust(2);
 				dlgi->addItem(cert);
 			} catch (errorEx &err) {
 				Error(err);
@@ -602,36 +605,30 @@ void MainWindow::closeEvent(QCloseEvent *e)
 
 QString makeSalt(void)
 {
-	unsigned char rand[2];
-	char saltbuf[10];
+	QString s = "T";
+	unsigned char rand[8];
 
-	Entropy::get(rand, 2);
-	snprintf(saltbuf, 10, "S%02X%02X", rand[0], rand[1]);
-	return QString(saltbuf);
+	Entropy::get(rand, sizeof rand);
+	for (unsigned i=0; i< sizeof rand; i++)
+		s += QString("%1").arg(rand[i]);
+	return s;
 }
 
 int MainWindow::checkOldGetNewPass(Passwd &pass)
 {
-	QString passHash;
-	db mydb(dbfile);
-
-	if (!mydb.find(setting, "pwhash")) {
-		char *cpass;
+	QString passHash = Settings["pwhash"];
+	if (!passHash.isEmpty()) {
 		pass_info p(tr("Current Password"),
 			tr("Please enter the current database password"), this);
 
-		if ((cpass = (char *)mydb.load(NULL))) {
-			passHash = cpass;
-			free(cpass);
-		}
 		/* Try empty password */
-		if (pki_evp::sha512passwd(pass, passHash) != passHash) {
+		if (pki_evp::sha512passwT(pass, passHash) != passHash) {
 			/* Not the empty password, check it */
 			if (PwDialog::execute(&p, &pass, false) != 1)
 				return 0;
 		}
 
-		if (pki_evp::sha512passwd(pass, passHash) != passHash) {
+		if (pki_evp::sha512passwT(pass, passHash) != passHash) {
 			XCA_WARN(tr("The entered password is wrong"));
 			return 0;
 		}
@@ -647,159 +644,105 @@ int MainWindow::checkOldGetNewPass(Passwd &pass)
 void MainWindow::changeDbPass()
 {
 	Passwd pass;
-	QString tempn = dbfile + "{recrypt}";
+	XSqlQuery q;
+	QSqlDatabase db = QSqlDatabase::database();
 
 	if (!checkOldGetNewPass(pass))
 		return;
 
-	try {
-		if (!QFile::copy(dbfile, tempn))
-			throw errorEx("Could not create temporary file: " +
-				tempn);
-
-		QString passhash = updateDbPassword(tempn, pass);
-		QFile new_file(tempn);
-		/* closing the database erases 'dbfile' */
-		QString dbfile_bkup = dbfile;
-		close_database();
-		db mydb(dbfile_bkup);
-		if (mydb.mv(new_file))
-			throw errorEx(QString("Failed to rename %1 to %2").
-						arg(tempn).arg(dbfile_bkup));
-		dbfile = dbfile_bkup;
-		pki_evp::passHash = passhash;
-		pki_evp::passwd = pass;
-		init_database();
-	} catch (errorEx &ex) {
-		QFile::remove(tempn);
-		Error(ex);
-	}
-}
-
-QString MainWindow::updateDbPassword(QString newdb, Passwd pass)
-{
-	db mydb(newdb);
-
 	QString salt = makeSalt();
-	QString passhash = pki_evp::sha512passwd(pass, salt);
-	mydb.set((const unsigned char *)CCHAR(passhash),
-		passhash.length()+1, 1, setting, "pwhash");
+	QString passhash = pki_evp::sha512passwT(pass, salt);
+	QList<pki_evp*> key_list = keys->sqlSELECTpki<pki_evp>(
+		"SELECT item FROM private_keys WHERE ownPass=0");
 
-	QList<pki_evp*> klist;
-	mydb.first();
-	while (mydb.find(asym_key, QString()) == 0) {
-		QString s;
-		pki_evp *key;
-		unsigned char *p;
-		db_header_t head;
-
-		p = mydb.load(&head);
-		if (!p)
-			throw errorEx("Failed to load item");
-		key = new pki_evp();
-		if (key->getVersion() < head.version) {
-			int v = key->getVersion();
-			free(p);
-			delete key;
-			throw errorEx(QString("Item[%1]: Version %2 "
-				"> known version: %3 -> ignored")
-				.arg(head.name).arg(head.version).arg(v)
-			);
+	try {
+		Transaction;
+		if (!TransBegin()) {
+			errorEx e(tr("Transaction start failed"));
+			Error(e);
+			return;
 		}
-		key->setIntName(QString::fromUtf8(head.name));
-
-		try {
-			key->fromData(p, &head);
-		}
-		catch (errorEx &err) {
-			err.appendString(key->getIntName());
-			free(p);
-			delete key;
-			throw err;
-		}
-		free(p);
-		if (key && key->getOwnPass() == pki_key::ptCommon &&
-			!key->isPubKey())
-		{
+		foreach(pki_evp *key, key_list) {
 			EVP_PKEY *evp = key->decryptKey();
 			key->set_evp_key(evp);
 			key->encryptKey(pass.constData());
-			klist << key;
-		} else if (key)
-			delete key;
-
-		if (mydb.next())
-			break;
+			key->sqlUpdatePrivateKey();
+		}
+		Settings["pwhash"] = passhash;
+		TransCommit();
+		pki_evp::passHash = passhash;
+		pki_evp::passwd = pass;
+	} catch (errorEx &e) {
+		Error(e);
 	}
-	for (int i=0; i< klist.count(); i++) {
-		pki_evp *key = klist[i];
-		QByteArray ba = key->toData();
-		mydb.set((const unsigned char*)ba.constData(), ba.count(),
-			key->getVersion(), key->getType(), key->getIntName());
-		delete key;
-	}
-	return passhash;
 }
 
-int MainWindow::initPass()
+int MainWindow::initPass(QString dbName)
+{
+	QString passhash = Settings["pwhash"];
+	return initPass(dbName, passhash);
+}
+
+static void pwhash_upgrade()
+{
+	/* Start automatic update from sha512 to sha512*8000
+	 * if the password is correct. The old sha512 hash does
+	 * start with 'S', while the new hash starts with T. */
+
+	/* Start automatic update from md5 to salted sha512*8000
+	 * if the password is correct. The md5 hash does not
+	 * start with 'S' or 'T, but with a hex-digit */
+	if (pki_evp::passHash.startsWith("T")) {
+		/* Fine, current hash function used. */
+		return;
+	}
+	if (pki_evp::sha512passwd(pki_evp::passwd,
+				pki_evp::passHash) == pki_evp::passHash ||
+	    pki_evp::md5passwd(pki_evp::passwd) == pki_evp::passHash)
+	{
+		QString salt = makeSalt();
+		pki_evp::passHash = pki_evp::sha512passwT(
+				pki_evp::passwd, salt);
+	}
+}
+
+int MainWindow::initPass(QString dbName, QString passhash)
 {
-	db mydb(dbfile);
-	char *pass;
 	pki_evp::passHash = QString();
-	QString salt;
+	QString salt, pass;
 	int ret;
 
 	pass_info p(tr("New Password"), tr("Please enter a password, "
 			"that will be used to encrypt your private keys "
-			"in the database file:\n%1").
-			arg(compressFilename(dbfile)), this);
+			"in the database:\n%1").
+			arg(compressFilename(dbName)), this);
 
-	if (!mydb.find(setting, "pwhash")) {
-		if ((pass = (char *)mydb.load(NULL))) {
-			pki_evp::passHash = pass;
-			free(pass);
-		}
-	}
+	pki_evp::passHash = passhash;
 	if (pki_evp::passHash.isEmpty()) {
 		ret = PwDialog::execute(&p, &pki_evp::passwd, true, true);
 		if (ret != 1)
 			return ret;
 		salt = makeSalt();
-		pki_evp::passHash = pki_evp::sha512passwd(pki_evp::passwd,salt);
-		mydb.set((const unsigned char *)CCHAR(pki_evp::passHash),
-			pki_evp::passHash.length()+1, 1, setting, "pwhash");
+		pki_evp::passHash =pki_evp::sha512passwT(pki_evp::passwd,salt);
+		Settings["pwhash"] = pki_evp::passHash;
 	} else {
+		pwhash_upgrade();
 		ret = 0;
-		while (pki_evp::sha512passwd(pki_evp::passwd, pki_evp::passHash)
+		while (pki_evp::sha512passwT(pki_evp::passwd, pki_evp::passHash)
 				!= pki_evp::passHash)
 		{
 			if (ret)
 				XCA_WARN(
 				tr("Password verify error, please try again"));
 			p.setTitle(tr("Password"));
-			p.setDescription(tr("Please enter the password for unlocking the database:\n%1").arg(compressFilename(dbfile)));
+			p.setDescription(tr("Please enter the password for unlocking the database:\n%1").arg(compressFilename(dbName)));
 			ret = PwDialog::execute(&p, &pki_evp::passwd,
 						false, true);
 			if (ret != 1) {
 				pki_evp::passwd = QByteArray();
 				return ret;
 			}
-			if (pki_evp::passHash.left(1) == "S")
-				continue;
-			/* Start automatic update from md5 to salted sha512
-			 * if the password is correct. my md5 hash does not
-			 * start with 'S', while my new hash does. */
-			if (pki_evp::md5passwd(pki_evp::passwd) ==
-						pki_evp::passHash )
-			{
-				salt = makeSalt();
-				pki_evp::passHash = pki_evp::sha512passwd(
-						pki_evp::passwd, salt);
-				mydb.set((const unsigned char *)CCHAR(
-					pki_evp::passHash),
-					pki_evp::passHash.length() +1, 1,
-					setting, "pwhash");
-			}
+			pwhash_upgrade();
 		}
 	}
 	if (pki_evp::passwd.isNull())
@@ -823,110 +766,87 @@ void MainWindow::Error(errorEx &err)
 	}
 }
 
-QString MainWindow::getPath()
-{
-	return workingdir;
-}
-
-void MainWindow::setPath(QString str)
-{
-	db mydb(dbfile);
-	workingdir = str;
-	mydb.set((const unsigned char *)CCHAR(str), str.length()+1, 1, setting, "workingdir");
-}
-
-void MainWindow::setDefaultKey(QString str)
-{
-	db mydb(dbfile);
-	mydb.set((const unsigned char *)CCHAR(str), str.length()+1, 1, setting, "defaultkey");
-}
-
 void MainWindow::connNewX509(NewX509 *nx)
 {
-	connect( nx, SIGNAL(genKey(QString)), keys, SLOT(newItem(QString)) );
-	connect( keys, SIGNAL(keyDone(QString)), nx, SLOT(newKeyDone(QString)) );
-	connect( nx, SIGNAL(showReq(QString)), reqs, SLOT(showItem(QString)));
+	connect(nx, SIGNAL(genKey(QString)),
+		keys, SLOT(newItem(QString)));
+	connect(keys, SIGNAL(keyDone(pki_key*)),
+		nx, SLOT(newKeyDone(pki_key*)));
+	connect(nx, SIGNAL(showReq(pki_base*)),
+		reqs, SLOT(showPki(pki_base*)));
+	connect(reqs, SIGNAL(pkiChanged(pki_base*)),
+		nx, SLOT(itemChanged(pki_base*)));
 }
 
 void MainWindow::importAnything(QString file)
 {
+	int ret;
 	ImportMulti *dlgi = new ImportMulti(this);
 	QStringList failed;
-	pki_multi *pki = probeAnything(file);
-	if (pki && !pki->count())
+	pki_multi *pki = probeAnything(file, &ret);
+	if (ret)
 		failed << file;
-	dlgi->addItem(pki);
+	else
+		dlgi->addItem(pki);
 	dlgi->execute(1, failed);
 	delete dlgi;
 }
 
 pki_multi *MainWindow::probeAnything(QString file, int *ret)
 {
-	pki_multi *pki = new pki_multi();
+	if (ret)
+		*ret = 0;
+	pki_multi *pki = NULL;
 
 	try {
-		if (file.endsWith(".xdb")) {
-			try {
-				int r;
-				db *mydb = new db(file);
-				mydb->verify_magic();
-				delete mydb;
-				r = changeDB(file);
-				delete pki;
-				if (ret)
-					*ret = r;
-				return NULL;
-			} catch (errorEx &err) {
-			}
+		if (file.endsWith(".xdb") ||
+		    !OpenDb::splitRemoteDbName(file).isEmpty())
+		{
+			int r = init_database(file);
+			if (ret)
+				*ret = r;
+			return pki;
 		}
+		pki = new pki_multi();
 		pki->probeAnything(file);
 	} catch (errorEx &err) {
 		Error(err);
 	}
+	if (!pki->count()) {
+		delete pki;
+		pki = NULL;
+		if (ret)
+			*ret = 1;
+	}
 	return pki;
 }
 
 void MainWindow::exportIndex()
 {
-	exportIndex(NULL, false);
+	exportIndex(QFileDialog::getSaveFileName(this, XCA_TITLE,
+				Settings["workingdir"],
+				tr("Certificate Index ( index.txt )") + ";;" +
+					tr("All files ( * )")),
+			false);
 }
 
 void MainWindow::exportIndexHierarchy()
 {
-	exportIndex(NULL, true);
+	exportIndex(QFileDialog::getExistingDirectory(
+		this, XCA_TITLE, Settings["workingdir"]), true);
 }
 
 int MainWindow::exportIndex(QString fname, bool hierarchy)
 {
-	if (fname == NULL || fname.isEmpty()) {
-
-		QString filter = tr("Certificate Index ( index.txt )") + ";;" + tr("All files ( * )");
-
-		fname = QFileDialog::getSaveFileName(this, QString(), fname, filter, NULL);
-
-		if (fname.isEmpty())
-			return 1;
-
-		nativeSeparator(fname);
-	}
-
+	qDebug() << fname << hierarchy;
+	if (fname.isEmpty())
+		return 1;
 	if (certs == NULL) {
 		open_default_db();
 		if (certs == NULL)
 			return 2;
 	}
-
-	bool needChangeView = hierarchy && !certs->treeview;
-	if (needChangeView) {
-		certView->changeView();
-	}
-
 	certs->writeIndex(fname, hierarchy);
-
-	if (needChangeView) {
-		certView->changeView();
-	}
-
 	return 0;
 }
 
@@ -975,7 +895,7 @@ void MainWindow::generateDHparam()
 
 		QString fname = QString("%1/dh%2.pem").arg(homedir).arg(num);
 		fname = QFileDialog::getSaveFileName(this, QString(),
-			fname, "All files ( * )", NULL);
+			fname, tr("All files ( * )"), NULL);
 		if (fname == "")
 			throw errorEx("");
 		fp = fopen_write(fname);
@@ -1013,8 +933,9 @@ void MainWindow::changeEvent(QEvent *event)
 			if (model)
 				model->updateHeaders();
 		}
-		if (!dbfile.isEmpty())
-			dbindex->setText(tr("Database") + ": " + dbfile);
+		if (!currentDB.isEmpty())
+			dbindex->setText(tr("Database") + ": " + currentDB);
+		searchEdit->setPlaceholderText(tr("Search"));
 	}
 	QMainWindow::changeEvent(event);
 }
