@@ -9,6 +9,7 @@
 #define __MAINWINDOW_H
 
 #include "NewX509.h"
+#include "XcaWarning.h"
 #include "OidResolver.h"
 #include "ui_MainWindow.h"
 #include "lib/db_key.h"
@@ -19,32 +20,20 @@
 #include "lib/exception.h"
 #include "lib/oid.h"
 #include "lib/Passwd.h"
+#include "lib/settings.h"
 #include "lib/main.h"
 #include <QPixmap>
 #include <QFileDialog>
 #include <QMenuBar>
 #include <QList>
-#include <QMessageBox>
+#include <QtSql>
 #include <QMenu>
 #include <QToolTip>
 #include <QLocale>
 
-#define DBFILE "xca.xdb"
-
 class db_x509;
 class pki_multi;
 
-class xcaWarning: public QMessageBox
-{
-	public:
-		xcaWarning(QWidget *w, QString txt)
-			: QMessageBox(QMessageBox::Warning,
-				 XCA_TITLE, txt, QMessageBox::NoButton, w)
-		{
-			setTextFormat(Qt::PlainText);
-		}
-};
-
 class tipMenu : public QMenu
 {
 	Q_OBJECT
@@ -71,9 +60,7 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 
 	private:
 		static OidResolver *resolver;
-		QString workingdir;
 		QString string_opt;
-		QString pkcs11path;
 		QList<QWidget*> wdList;
 		QList<QWidget*> wdMenuList;
 		QList<QWidget*> scardList;
@@ -81,12 +68,15 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		QStringList history;
 		tipMenu *historyMenu;
 		void update_history_menu();
-		void set_geometry(char *p, db_header_t *head);
+		void set_geometry(QString geo);
 		QLineEdit *searchEdit;
 		QStringList urlsToOpen;
 		int checkOldGetNewPass(Passwd &pass);
-		QString updateDbPassword(QString newdb, Passwd pass);
 		int exportIndex(QString fname, bool hierarchy);
+		void checkDB();
+		QSqlError initSqlDB();
+		QString openSqlDB(QString dbName);
+		QString currentDB;
 
 	protected:
 		void init_images();
@@ -96,9 +86,6 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		QLabel *statusLabel;
 		QString homedir;
 		int changeDB(QString fname);
-		void setOptFlags(QString flags);
-		void setOptFlags_old(QString flags);
-		QString getOptFlags();
 		void keyPressEvent(QKeyEvent *e);
 
 	public:
@@ -111,18 +98,15 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 				*nsImg, *revImg, *appIco, *scardImg,
 				*doneIco, *warnIco;
 		static NIDlist *eku_nid, *dn_nid;
-		static QString mandatory_dn;
-		static QString explicit_dn;
-		static QString explicit_dn_default;
 		int exitApp;
-		QString dbfile;
 		QLabel *dbindex;
 
 		MainWindow(QWidget *parent);
 		virtual ~MainWindow();
 		void loadSettings();
 		void saveSettings();
-		int initPass();
+		int initPass(QString dbName);
+		int initPass(QString dbName, QString passhash);
 		void read_cmdline(int argc, char *argv[]);
 		void load_engine();
 		static OidResolver *getResolver()
@@ -130,11 +114,11 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 			return resolver;
 		}
 		static void Error(errorEx &err);
+		static void dbSqlError(QSqlError err = QSqlError());
+
 		void cmd_version();
 		void cmd_help(const char* msg);
 
-		QString getPath();
-		void setPath(QString path);
 		bool mkDir(QString dir);
 		void setItemEnabled(bool enable);
 		void enableTokenMenu(bool enable);
@@ -143,13 +127,16 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		void dropEvent(QDropEvent *event);
 		void dragEnterEvent(QDragEnterEvent *event);
 		int open_default_db();
-		void setDefaultKey(QString def);
 		void load_history();
 		void update_history(QString file);
 		void initResolver();
+		bool checkForOldDbFormat();
+		bool checkForOldDbFormat(QString dbfile);
+		int verifyOldDbPass(QString dbname);
+		void importOldDatabase(QString dbname);
 
 	public slots:
-		int init_database();
+		int init_database(QString dbName);
 		void new_database();
 		void load_database();
 		void close_database();
@@ -158,10 +145,9 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		void connNewX509(NewX509 *nx);
 		void about();
 		void help();
-		void import_dbdump();
 		void undelete();
 		void loadPem();
-		bool pastePem(QString text);
+		bool pastePem(QString text, bool silent=false);
 		void pastePem();
 		void changeDbPass();
 		void openURLs(QStringList &files);
@@ -169,6 +155,7 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		void changeEvent(QEvent *event);
 		void exportIndex();
 		void exportIndexHierarchy();
+		void openRemoteSqlDB();
 
 	protected slots:
 		void closeEvent(QCloseEvent * event);
