@@ -9,9 +9,10 @@
 #include "NewKey.h"
 #include "MainWindow.h"
 #include "lib/pki_evp.h"
-#include "widgets/distname.h"
-#include "widgets/clicklabel.h"
 #include "lib/pkcs11.h"
+#include "distname.h"
+#include "clicklabel.h"
+#include "ItemCombo.h"
 #include <QLabel>
 #include <QPushButton>
 #include <QLineEdit>
@@ -123,13 +124,13 @@ Q_DECLARE_METATYPE(keyListItem);
 NewKey::NewKey(QWidget *parent, QString name)
 	:QDialog(parent)
 {
-	static const char* const sizeList[] = { "1024", "2048", "4096" };
+	static const char* const sizeList[] = { "1024", "2048", "4096", "8192" };
 	size_t i;
 	slotidList p11_slots;
 	QList<keyListItem> keytypes;
 
 	setupUi(this);
-	setWindowTitle(tr(XCA_TITLE));
+	setWindowTitle(XCA_TITLE);
 	image->setPixmap(*MainWindow::keyImg);
 
 	if (!name.isEmpty())
