@@ -16,6 +16,8 @@
 #include <QPixmap>
 #include <QPushButton>
 #include <QValidator>
+#include <QMap>
+#include <QPair>
 #include "MainWindow.h"
 #include "v3ext.h"
 #include "lib/x509name.h"
@@ -34,7 +36,7 @@ NewX509::NewX509(QWidget *parent)
 	dn_nid = *MainWindow::dn_nid;
 	aia_nid << OBJ_sn2nid("OCSP") << OBJ_sn2nid("caIssuers");
 	attr_nid << NID_pkcs9_unstructuredName << NID_pkcs9_challengePassword;
-	foreach(QString dn, MainWindow::explicit_dn.split(","))
+	foreach(QString dn, Settings["explicit_dn"].split(","))
 		expl_dn_nid << OBJ_sn2nid(CCHAR(dn));
 
 	QStringList keys;
@@ -43,7 +45,7 @@ NewX509::NewX509(QWidget *parent)
 
 	/* temporary storage for creating temporary X509V3_CTX */
 	ctx_cert = NULL;
-
+	pkiSource = generated;
 	foreach(int nid, dn_nid)
 		keys << QString(OBJ_nid2ln(nid));
 
@@ -68,48 +70,36 @@ NewX509::NewX509(QWidget *parent)
 	}
 
 	nsImg->setPixmap(*MainWindow::nsImg);
-	serialNr->setValidator(new QRegExpValidator(QRegExp("[0-9a-fA-F]*"), this));
-	QStringList strings;
 
 	// are there any useable private keys  ?
-	newKeyDone("");
+	newKeyDone(NULL);
 
 	// any PKCS#10 requests to be used ?
-	strings = MainWindow::reqs->getDesc();
-	if (strings.isEmpty()) {
+	QList<pki_x509req *> requests = MainWindow::reqs->getAllRequests();
+	if (requests.isEmpty()) {
 		fromReqCB->setDisabled(true);
 		fromReqCB->setChecked(false);
 	}
 	else {
-		reqList->insertItems(0, strings);
+		reqList->insertPkiItems(requests);
 	}
 	on_fromReqCB_clicked();
 
 	// How about signing certificates ?
-	strings = MainWindow::certs->getSignerDesc();
-	if (strings.isEmpty()) {
+	QList<pki_x509*> issuers = MainWindow::certs->getAllIssuers();
+	if (issuers.isEmpty()) {
 		foreignSignRB->setDisabled(true);
 	} else {
-		certList->insertItems(0, strings);
+		certList->insertPkiItems(issuers);
 	}
-#ifdef WG_QA_SERIAL
-	selfQASignRB = new QRadioButton(signerBox);
-	setTabOrder(serialNr, selfQASignRB);
-	setTabOrder(selfQASignRB, foreignSignRB);
-	selfQASignRB->setText(tr(
-			"Create a &self signed certificate with a MD5-hashed QA serial"));
-	QBoxLayout *l = (QBoxLayout *)signerBox->layout();
-	l->insertWidget(1, selfQASignRB);
-#endif
+
 	// set dates to now and now + 1 year
-	validNumber->setText("1");
+	validN->setText("1");
 	validRange->setCurrentIndex(2);
 	on_applyTime_clicked();
 
 	// settings for the templates ....
-	strings.clear();
-	strings = MainWindow::temps->getDescPredefs();
-	tempList->insertItems(0, strings);
+	tempList->insertPkiItems(MainWindow::temps->getAllAndPredefs());
 
 	// setup Extended keyusage
 	foreach(int nid, eku_nid)
@@ -147,7 +137,7 @@ NewX509::NewX509(QWidget *parent)
 		QString trans = dn_translations[nid];
 
 		label = new DoubleClickLabel(this);
-		if (translate_dn && !trans.isEmpty()) {
+		if (Settings["translate_dn"] && !trans.isEmpty()) {
 			label->setText(trans);
 			label->setToolTip(QString("[%1] %2")
 				.arg(OBJ_nid2sn(nid)).arg(OBJ_nid2ln(nid)));
@@ -189,7 +179,7 @@ NewX509::NewX509(QWidget *parent)
 		QString trans = dn_translations[nid];
 
 		label = new DoubleClickLabel(this);
-		if (translate_dn && !trans.isEmpty()) {
+		if (Settings["translate_dn"] && !trans.isEmpty()) {
 			label->setText(trans);
 			label->setToolTip(QString(OBJ_nid2sn(nid)));
 		} else {
@@ -233,10 +223,10 @@ NewX509::NewX509(QWidget *parent)
 
 	foreach(int nid, nidLabel.keys()) {
 		DoubleClickLabel *l = nidLabel[nid];
-		l->setText(translate_dn ?
+		l->setText(Settings["translate_dn"] ?
 			dn_translations[nid] : OBJ_nid2ln(nid));
 		if (l->toolTip().isEmpty()) {
-			l->setToolTip(translate_dn ?
+			l->setToolTip(Settings["translate_dn"] ?
 				OBJ_nid2ln(nid) : dn_translations[nid]);
 		}
 		l->setClickText(OBJ_nid2sn(nid));
@@ -252,15 +242,15 @@ NewX509::NewX509(QWidget *parent)
 
 	foreach(int nid, nidGroupBox.keys()) {
 		QGroupBox *g = nidGroupBox[nid];
-		g->setTitle(translate_dn ?
+		g->setTitle(Settings["translate_dn"] ?
 			dn_translations[nid] : OBJ_nid2ln(nid));
 		if (g->toolTip().isEmpty()) {
-			g->setToolTip(translate_dn ?
+			g->setToolTip(Settings["translate_dn"] ?
 				OBJ_nid2ln(nid) : dn_translations[nid]);
 		}
 	}
 
-	if (translate_dn) {
+	if (Settings["translate_dn"]) {
 		QList<QGroupBox*> gb;
 		gb << distNameBox << keyIdentBox;
 		foreach(QGroupBox *g, gb) {
@@ -274,8 +264,32 @@ NewX509::NewX509(QWidget *parent)
 			cb->setText(tr("Critical"));
 		}
 	}
-	if (pki_x509::disable_netscape)
+	if (Settings["disable_netscape"])
 		tabWidget->removeTab(4);
+
+	// Setup widget <-> Template mapping
+#define MAP_LE(name) templateLineEdits[#name] = name;
+	MAP_LE(subAltName);
+	MAP_LE(issAltName);
+	MAP_LE(crlDist);
+	MAP_LE(nsComment);
+	MAP_LE(nsBaseUrl);
+	MAP_LE(nsRevocationUrl);
+	MAP_LE(nsCARevocationUrl);
+	MAP_LE(nsRenewalUrl);
+	MAP_LE(nsCaPolicyUrl);
+	MAP_LE(nsSslServerName);
+	MAP_LE(validN);
+	MAP_LE(basicPath);
+
+#define MAP_CB(name) templateCheckBoxes[#name] = name;
+	MAP_CB(bcCritical);
+	MAP_CB(kuCritical);
+	MAP_CB(ekuCritical);
+	MAP_CB(subKey);
+	MAP_CB(authKey);
+	MAP_CB(validMidn);
+	MAP_CB(noWellDefinedExpDate);
 }
 
 void NewX509::setRequest()
@@ -344,18 +358,15 @@ void NewX509::setReqAttributes(pki_x509req *req)
 /* Initialize dialog for Template creation */
 void NewX509::setTemp(pki_temp *temp)
 {
-	QString text = tr("Create XCA template");
-	if (temp->getIntName() != "--") {
-		description->setText(temp->getIntName());
-		description->setDisabled(true);
-		text = tr("Edit XCA template");
-	}
-	capt->setText(text);
+	description->setText(temp->getIntName());
+	capt->setText(tr("Edit XCA template"));
 	tabWidget->removeTab(0);
 	privKeyBox->setEnabled(false);
 	validityBox->setEnabled(false);
 	setImage(MainWindow::tempImg);
 	pt = tmpl;
+	fromTemplate(temp);
+	comment->setPlainText(temp->getComment());
 }
 
 /* Initialize dialog for Certificate creation */
@@ -376,17 +387,16 @@ void NewX509::defineTemplate(pki_temp *temp)
 {
 	fromTemplate(temp);
 	templateChanged(temp);
+	pkiSource = transformed;
 }
 
 /* Select a Request for signing it */
 void NewX509::defineRequest(pki_x509req *req)
 {
-	if (!req)
-		return;
 	fromReqCB->setEnabled(true);
 	fromReqCB->setChecked(true);
-	QString reqname = req->getIntName();
-	reqList->setCurrentIndex(reqList->findText(reqname));
+	reqList->setCurrentPkiItem(req);
+	pkiSource = transformed;
 	on_fromReqCB_clicked();
 }
 
@@ -402,8 +412,7 @@ void NewX509::fromX509super(pki_x509super *cert_or_req, bool applyTemp)
 	pki_key *key = cert_or_req->getRefKey();
 	if (key) {
 		usedKeysToo->setChecked(true);
-		keyList->setCurrentIndex(private_keys.indexOf(
-			key->getIntNameWithType()));
+		keyList->setCurrentPkiItem(key);
 	}
 	hashAlgo->setCurrentMD(cert_or_req->getDigest());
 
@@ -431,20 +440,27 @@ void NewX509::fromX509super(pki_x509super *cert_or_req, bool applyTemp)
 
 }
 
+pki_temp *NewX509::caTemplate(pki_x509 *ca) const
+{
+	QVariant sqlId = ca->getTemplateSqlId();
+	if (!sqlId.isValid())
+		return NULL;
+	return MainWindow::temps->lookupPki<pki_temp>(sqlId);
+}
+
 /* Preset the signing certificate */
 void NewX509::defineSigner(pki_x509 *defcert, bool applyTemp)
 {
-	int index;
 	// suggested from: Andrey Brindeew <abr@abr.pp.ru>
 	if (defcert && defcert->canSign() ) {
-		QString name = defcert->getIntName();
-		foreignSignRB->setChecked(true);
-		certList->setEnabled(true);
-		if ((index = certList->findText(name)) >= 0) {
-			certList->setCurrentIndex(index);
-		}
-		if (applyTemp && !defcert->getTemplate().isEmpty()) {
-			on_applyTemplate_clicked();
+		if (certList->setCurrentPkiItem(defcert) != -1) {
+			foreignSignRB->setChecked(true);
+			certList->setEnabled(true);
+			if (applyTemp &&
+			    defcert->getTemplateSqlId().isValid())
+			{
+				on_applyTemplate_clicked();
+			}
 		}
 	}
 }
@@ -503,39 +519,33 @@ static QString lb2QString(QListWidget *lb)
 void NewX509::subjectFromTemplate(pki_temp *temp)
 {
 	if (temp)
-		setX509name(temp->xname);
+		setX509name(temp->getSubject());
 }
 
+
 void NewX509::extensionsFromTemplate(pki_temp *temp)
 {
 	if (!temp)
 		return;
-	subAltName->setText(temp->subAltName);
-	issAltName->setText(temp->issAltName);
-	crlDist->setText(temp->crlDist);
-	setAuthInfAcc_string(temp->authInfAcc);
-	nsComment->setText(temp->nsComment);
-	nsBaseUrl->setText(temp->nsBaseUrl);
-	nsRevocationUrl->setText(temp->nsRevocationUrl);
-	nsCARevocationUrl->setText(temp->nsCARevocationUrl);
-	nsRenewalUrl->setText(temp->nsRenewalUrl);
-	nsCaPolicyUrl->setText(temp->nsCaPolicyUrl);
-	nsSslServerName->setText(temp->nsSslServerName);
-	int2lb(nsCertType, temp->nsCertType);
-	basicCA->setCurrentIndex(temp->ca);
-	bcCritical->setChecked(temp->bcCrit);
-	kuCritical->setChecked(temp->keyUseCrit);
-	ekuCritical->setChecked(temp->eKeyUseCrit);
-	subKey->setChecked(temp->subKey);
-	authKey->setChecked(temp->authKey);
-	int2lb(keyUsage, temp->keyUse);
-	QString2lb(ekeyUsage, temp->eKeyUse);
-	validNumber->setText(QString::number(temp->validN));
-	validRange->setCurrentIndex(temp->validM);
-	midnightCB->setChecked(temp->validMidn);
-	basicPath->setText(temp->pathLen);
-	nconf_data->document()->setPlainText(temp->adv_ext);
-	noWellDefinedExpDate->setChecked(temp->noWellDefined);
+
+	QMapIterator<QString, QLineEdit*> l(templateLineEdits);
+	while (l.hasNext()) {
+		l.next();
+		l.value()->setText(temp->getSetting(l.key()));
+	}
+	QMapIterator<QString, QCheckBox*> i(templateCheckBoxes);
+	while (i.hasNext()) {
+		i.next();
+		i.value()->setChecked(temp->getSettingInt(i.key()));
+	}
+	int2lb(nsCertType, temp->getSettingInt("nsCertType"));
+	basicCA->setCurrentIndex(temp->getSettingInt("ca"));
+	int2lb(keyUsage, temp->getSettingInt("keyUse"));
+	QString2lb(ekeyUsage, temp->getSetting("eKeyUse"));
+	validRange->setCurrentIndex(temp->getSettingInt("validM"));
+	nconf_data->document()->setPlainText(temp->getSetting("adv_ext"));
+	setAuthInfAcc_string(temp->getSetting("authInfAcc"));
+
 	on_applyTime_clicked();
 }
 
@@ -548,39 +558,35 @@ void NewX509::fromTemplate(pki_temp *temp)
 void NewX509::toTemplate(pki_temp *temp)
 {
 	temp->setIntName(description->text());
-	temp->xname = getX509name();
-	temp->subAltName = subAltName->text();
-	temp->issAltName = issAltName->text();
-	temp->crlDist = crlDist->text();
-	temp->authInfAcc = getAuthInfAcc_string();
-	temp->nsComment = nsComment->text();
-	temp->nsBaseUrl = nsBaseUrl->text();
-	temp->nsRevocationUrl = nsRevocationUrl->text();
-	temp->nsCARevocationUrl = nsCARevocationUrl->text();
-	temp->nsRenewalUrl = nsRenewalUrl->text();
-	temp->nsCaPolicyUrl = nsCaPolicyUrl->text();
-	temp->nsSslServerName = nsSslServerName->text();
-	temp->nsCertType =  lb2int(nsCertType);
-	temp->ca = basicCA->currentIndex();
-	temp->bcCrit = bcCritical->isChecked();
-	temp->keyUseCrit = kuCritical->isChecked();
-	temp->eKeyUseCrit = ekuCritical->isChecked();
-	temp->subKey = subKey->isChecked();
-	temp->authKey = authKey->isChecked();
-	temp->keyUse = lb2int(keyUsage);
-	temp->eKeyUse = lb2QString(ekeyUsage);
-	temp->validN = validNumber->text().toInt();
-	temp->validM = validRange->currentIndex();
-	temp->pathLen = basicPath->text();
-	if (!temp->pathLen.isEmpty())
-		temp->pathLen = QString::number(temp->pathLen.toInt());
-	temp->validMidn = midnightCB->isChecked();
-	if (nconf_data->isReadOnly()) {
-		temp->adv_ext = v3ext_backup;
-	} else {
-		temp->adv_ext = nconf_data->toPlainText();
+	temp->setSubject(getX509name());
+
+	QMapIterator<QString, QLineEdit*> l(templateLineEdits);
+	while (l.hasNext()) {
+		l.next();
+		temp->setSetting(l.key(), l.value()->text());
 	}
-	temp->noWellDefined = noWellDefinedExpDate->isChecked();
+	QMapIterator<QString, QCheckBox*> i(templateCheckBoxes);
+	while (i.hasNext()) {
+		i.next();
+		temp->setSetting(i.key(), i.value()->isChecked());
+	}
+
+	temp->setSetting("authInfAcc", getAuthInfAcc_string());
+	temp->setSetting("nsCertType", lb2int(nsCertType));
+	temp->setSetting("ca", basicCA->currentIndex());
+	temp->setSetting("keyUse", lb2int(keyUsage));
+	temp->setSetting("eKeyUse", lb2QString(ekeyUsage));
+	temp->setSetting("validN", validN->text().toInt());
+	temp->setSetting("validM", validRange->currentIndex());
+	if (!temp->getSetting("basicPath").isEmpty())
+		temp->setSetting("basicPath", temp->getSettingInt("basicPath"));
+	if (nconf_data->isReadOnly()) {
+		temp->setSetting("adv_ext", v3ext_backup);
+	} else {
+		temp->setSetting("adv_ext", nconf_data->toPlainText());
+	}
+
+	temp->setComment(comment->toPlainText());
 }
 
 void NewX509::on_fromReqCB_clicked()
@@ -649,17 +655,20 @@ void NewX509::switchHashAlgo()
 
 void NewX509::on_showReqBut_clicked()
 {
-	QString req = reqList->currentText();
-	emit showReq(req);
+	emit showReq(reqList->currentPkiItem());
+}
+
+void NewX509::itemChanged(pki_base* req)
+{
+	reqList->insertPkiItems(MainWindow::reqs->getAllRequests());
+	reqList->setCurrentPkiItem(dynamic_cast<pki_x509req*>(req));
 }
 
 void NewX509::on_genKeyBut_clicked()
 {
 	QString name = description->text();
 	if (name.isEmpty())
-		name = dnEntryByNid(NID_commonName);
-	if (name.isEmpty())
-		name = dnEntryByNid(NID_pkcs9_emailAddress);
+		name = getX509name().getMostPopular();
 	emit genKey(name);
 }
 
@@ -673,7 +682,7 @@ void NewX509::on_certList_currentIndexChanged(int)
 	if (!cert)
 		return;
 
-	QString templ = cert->getTemplate();
+	pki_temp *templ = caTemplate(cert);
 	snb = cert->getNotBefore();
 	sna = cert->getNotAfter();
 	if (snb > notBefore->getDate())
@@ -681,10 +690,8 @@ void NewX509::on_certList_currentIndexChanged(int)
 	if (sna < notAfter->getDate())
 		notAfter->setDate(sna);
 
-	if (templ.isEmpty())
-		return;
-
-	templateChanged(templ);
+	if (templ)
+		templateChanged(templ);
 }
 
 
@@ -702,33 +709,44 @@ void NewX509::templateChanged(QString tempname)
 
 void NewX509::templateChanged(pki_temp *templ)
 {
-	QString tempname = templ->getIntName();
-	templateChanged(tempname);
+	tempList->setCurrentPkiItem(templ);
 }
 
 pki_temp *NewX509::currentTemplate()
 {
 	if (!tempList->isEnabled())
 		return NULL;
-	QString name = tempList->currentText();
-	if (name.isEmpty())
-		return NULL;
-	return (pki_temp *)MainWindow::temps->getByName(name);
+	return tempList->currentPkiItem();
+}
+
+void NewX509::selfComment(QString msg)
+{
+	comment->setPlainText(appendXcaComment(comment->toPlainText(), msg));
 }
 
 void NewX509::on_applyTemplate_clicked()
 {
-	fromTemplate(currentTemplate());
+	pki_temp *t = currentTemplate();
+	if (!t)
+		return;
+	fromTemplate(t);
+	selfComment(tr("Template '%1' applied").arg(t->comboText()));
 }
 
 void NewX509::on_applySubject_clicked()
 {
-	subjectFromTemplate(currentTemplate());
+	pki_temp *t = currentTemplate();
+	subjectFromTemplate(t);
+	selfComment(tr("Subject applied from template '%1'")
+			.arg(t->comboText()));
 }
 
 void NewX509::on_applyExtensions_clicked()
 {
-	extensionsFromTemplate(currentTemplate());
+	pki_temp *t = currentTemplate();
+	extensionsFromTemplate(t);
+	selfComment(tr("Extensions applied from template '%1'")
+			.arg(t->comboText()));
 }
 
 void NewX509::on_foreignSignRB_toggled(bool)
@@ -736,51 +754,42 @@ void NewX509::on_foreignSignRB_toggled(bool)
 	switchHashAlgo();
 }
 
-void NewX509::newKeyDone(QString name)
+void NewX509::newKeyDone(pki_key *nkey)
 {
-	QStringList keys;
-	private_keys = MainWindow::keys->get0KeyDesc(true);
-	private_keys0 = MainWindow::keys->get0KeyDesc(false);
-	keyList->clear();
-	if (usedKeysToo->isChecked())
-		keys = private_keys;
-	else
-		keys = private_keys0;
-
-	keyList->insertItems(0, keys);
-	if (name.isEmpty() && keys.count() >0)
-		name = keys[0];
-	keyList->setCurrentIndex(keys.indexOf(name));
+	allKeys =   MainWindow::keys->getAllKeys();
+	unusedKeys= MainWindow::keys->getUnusedKeys();
+	on_usedKeysToo_toggled(true);
+	if (nkey) {
+		selfComment(tr("New key '%1' created")
+				.arg(nkey->comboText()));
+		keyList->setCurrentPkiItem(nkey);
+	} else {
+		keyList->setCurrentIndex(0);
+	}
 }
 
 void NewX509::on_usedKeysToo_toggled(bool)
 {
-	QString cur = keyList->currentText();
-	QStringList keys;
+	pki_key *cur = keyList->currentPkiItem();
 	keyList->clear();
-	if (usedKeysToo->isChecked())
-		keys = private_keys;
-	else
-		keys = private_keys0;
-
-	keyList->insertItems(0, keys);
-	keyList->setCurrentIndex(keys.indexOf(cur));
+	keyList->insertPkiItems(usedKeysToo->isChecked() ?
+			allKeys : unusedKeys);
+	keyList->setCurrentPkiItem(cur);
 }
 
 pki_key *NewX509::getSelectedKey()
 {
-	QString name = pki_key::removeTypeFromIntName(keyList->currentText());
-	return (pki_key *)MainWindow::keys->getByName(name);
+	return keyList->currentPkiItem();
 }
 
 pki_x509 *NewX509::getSelectedSigner()
 {
-	return (pki_x509 *)MainWindow::certs->getByName(certList->currentText());
+	return certList->currentPkiItem();
 }
 
 pki_x509req *NewX509::getSelectedReq()
 {
-	return (pki_x509req *)MainWindow::reqs->getByName(reqList->currentText());
+	return reqList->currentPkiItem();
 }
 
 x509name NewX509::getX509name(int _throw)
@@ -809,20 +818,6 @@ x509name NewX509::getX509name(int _throw)
 	return x;
 }
 
-QString NewX509::dnEntryByNid(int nid)
-{
-	foreach(nameEdit ne, nameEdits) {
-		if (ne.nid == nid && !ne.edit->text().isEmpty())
-			return ne.edit->text();
-	}
-	for (int j=0; j<extDNlist->rowCount(); j++) {
-		QStringList l = extDNlist->getRow(j);
-	        if (OBJ_txt2nid(CCHAR(l[0])) == nid && !l[1].isEmpty())
-			return l[1];
-	}
-	return QString();
-}
-
 void NewX509::setX509name(const x509name &n)
 {
 	extDNlist->deleteAllRows();
@@ -848,7 +843,7 @@ void NewX509::setX509name(const x509name &n)
 
 void NewX509::on_applyTime_clicked()
 {
-	notAfter->setDiff(notBefore, validNumber->text().toInt(),
+	notAfter->setDiff(notBefore, validN->text().toInt(),
 				     validRange->currentIndex());
 }
 
@@ -857,7 +852,7 @@ void NewX509::setupTmpCtx()
 	pki_x509 *signcert;
 	pki_x509req *req = NULL;
 	pki_key *key = NULL;
-	a1int serial;
+	a1int serial(1);
 	QString errtxt;
 
 	// initially create temporary ctx cert
@@ -878,11 +873,9 @@ void NewX509::setupTmpCtx()
 	// Step 2 - select Signing
 	if (foreignSignRB->isChecked()) {
 		signcert = getSelectedSigner();
-		serial = signcert->getCaSerial();
 	} else {
 		signcert = ctx_cert;
 		ctx_cert->setIssuer(ctx_cert->getSubject());
-		serial.setHex(serialNr->text());
 	}
 	ctx_cert->setSerial(serial);
 	initCtx(ctx_cert, signcert, req);
@@ -991,7 +984,7 @@ void NewX509::undo_validateExtensions()
 
 int NewX509::validateExtensions(QString nconf, QString &result)
 {
-	int ret = 0;
+	int ret = 0, ext_count = 0;
 	QStringList errors;
 	extList el, req_el;
 	ign_openssl_error();
@@ -999,13 +992,14 @@ int NewX509::validateExtensions(QString nconf, QString &result)
 	(void)nconf;
 	try {
 		el = getGuiExt();
-		if (!pki_x509::disable_netscape)
+		if (!Settings["disable_netscape"])
 			el += getNetscapeExt();
 		el.delInvalid();
 	} catch (errorEx &err) {
 		errors += err.getString();
 		el.clear();
 	}
+	ext_count += el.size();
 	if (el.size() > 0) {
 		result += "<h2><center>";
 		result += tr("Other Tabs") + "</center></h2><p>\n";
@@ -1017,6 +1011,7 @@ int NewX509::validateExtensions(QString nconf, QString &result)
 		errors += err.getString();
 		el.clear();
 	}
+	ext_count += el.size();
 	if (el.size() > 0) {
 		if (!result.isEmpty())
 			result += "\n<hr>\n";
@@ -1041,6 +1036,7 @@ int NewX509::validateExtensions(QString nconf, QString &result)
 				el += req_el[i];
 		}
 	}
+	ext_count += el.size();
 	if (el.size() > 0) {
 		if (!result.isEmpty())
 			result += "\n<hr>\n";
@@ -1061,7 +1057,7 @@ int NewX509::validateExtensions(QString nconf, QString &result)
 		result = errtxt + result;
 	}
 	ign_openssl_error();
-	return ret;
+	return ret == 1 ? 1 : ext_count == 0 && pt == x509 ? 2 : 0;
 }
 
 void NewX509::on_editSubAlt_clicked()
@@ -1094,11 +1090,11 @@ void NewX509::on_tabWidget_currentChanged(int tab)
 
 QString NewX509::mandatoryDnRemain()
 {
-	QStringList remain, dnl = MainWindow::mandatory_dn.split(",");
+	QStringList remain, dnl = QString(Settings["mandatory_dn"]).split(",");
 	x509name n;
 	int i;
 
-	if (MainWindow::mandatory_dn.isEmpty())
+	if (QString(Settings["mandatory_dn"]).isEmpty())
 		return QString();
 
 	if (fromReqCB->isChecked() && !reqSubChange->isChecked())
@@ -1129,6 +1125,11 @@ void NewX509::gotoTab(int tab)
 	}
 }
 
+enum pki_source NewX509::getPkiSource() const
+{
+	return pkiSource;
+}
+
 void NewX509::accept()
 {
 	x509name xn;
@@ -1177,7 +1178,7 @@ void NewX509::accept()
 		}
 	}
 	if (description->text().isEmpty() && !fromReqCB->isChecked()) {
-		QString cn = dnEntryByNid(NID_commonName);
+		QString cn = getX509name().getMostPopular();
 		if (cn.isEmpty()) {
 			gotoTab(1);
 			xcaWarning msg(this,
@@ -1318,9 +1319,16 @@ void NewX509::accept()
 				break;
 		}
 	}
-	if (do_validateExtensions()) {
-		gotoTab(5);
-		QString text = tr("The certificate contains invalid or duplicate extensions. Check the validation on the advanced tab.");
+	int r = do_validateExtensions();
+	if (r) {
+		QString text;
+		if (r == 1) {
+			text = tr("The certificate contains invalid or duplicate extensions. Check the validation on the advanced tab.");
+			gotoTab(5);
+		} else {
+			text = tr("The certificate contains no extensions. You may apply the extensions of one of the templates to define the purpose of the certificate.");
+			gotoTab(0);
+		}
 		xcaWarning msg(this, text);
 		msg.addButton(QMessageBox::Ok)->setText(tr("Edit extensions"));
 		msg.addButton(QMessageBox::Close)->setText(tr("Abort rollout"));
