@@ -53,18 +53,21 @@ class NewX509: public QDialog, public Ui::NewX509
 		X509V3_CTX ext_ctx;
 		void editV3ext(QLineEdit *le, QString types, int n);
 		enum pki_type pt;
+		enum pki_source pkiSource;
 		void templateChanged(QString templatename);
 		QString mandatoryDnRemain();
-		QStringList private_keys, private_keys0, tabnames;
+		QStringList tabnames;
+		QList<pki_key*> unusedKeys, allKeys;
 		pki_x509 *ctx_cert;
 		QString v3ext_backup;
 		kvmodel *extDNmodel;
 		extList getExtDuplicates();
 		void checkIcon(const QString &text, int nid, QLabel*img);
-		QString dnEntryByNid(int nid);
-
+		void selfComment(QString msg);
+		QMap<QString, QLineEdit*> templateLineEdits;
+		QMap<QString, QCheckBox*> templateCheckBoxes;
+		pki_temp *caTemplate(pki_x509 *ca) const;
 	public:
-		QRadioButton *selfQASignRB;
 		QLineEdit *description;
 		NewX509(QWidget *parent);
 		virtual ~NewX509();
@@ -114,12 +117,13 @@ class NewX509: public QDialog, public Ui::NewX509
 		int validateExtensions(QString nconf, QString &result);
 		int do_validateExtensions();
 		void undo_validateExtensions();
+		enum pki_source getPkiSource() const;
 
 	public slots:
 		void on_fromReqCB_clicked();
 		void on_keyList_currentIndexChanged(const QString &);
 		void on_reqList_currentIndexChanged(const QString &);
-		void newKeyDone(QString name);
+		void newKeyDone(pki_key *nkey);
 		void on_applyTime_clicked();
 		void on_editSubAlt_clicked();
 		void on_editIssAlt_clicked();
@@ -137,6 +141,7 @@ class NewX509: public QDialog, public Ui::NewX509
 		void on_tabWidget_currentChanged(int idx);
 		void on_reqSubChange_clicked();
 		void accept();
+		void itemChanged(pki_base*);
 		void setupExtDNwidget(const QString &s, QLineEdit *w);
 		void checkSubAltName(const QString & text);
 		void checkIssAltName(const QString & text);
@@ -145,7 +150,7 @@ class NewX509: public QDialog, public Ui::NewX509
 
 	signals:
 		void genKey(QString);
-		void showReq(QString req);
+		void showReq(pki_base *req);
 };
 
 #endif
