@@ -48,6 +48,8 @@ XcaTreeView::XcaTreeView(QWidget *parent)
 	throttle.setSingleShot(true);
 	connect(&throttle, SIGNAL(timeout()), this, SLOT(columnsResize()));
 	connect(&throttle, SIGNAL(timeout()), proxy, SLOT(invalidate()));
+	setFocusPolicy(Qt::StrongFocus);
+	setExpandsOnDoubleClick(false);
 }
 
 XcaTreeView::~XcaTreeView()
@@ -217,6 +219,10 @@ void XcaTreeView::deleteItems(void)
 		count++;
 	}
 
+	Transaction;
+	if (!TransBegin())
+		return;
+
 	if (count == 1)
 		msg = pki->getMsg(pki_base::msg_delete).arg(pki->getIntName());
 	else
@@ -229,6 +235,7 @@ void XcaTreeView::deleteItems(void)
 	foreach(index, indexes) {
 		basemodel->deletePKI(index);
 	}
+	TransCommit();
 }
 
 void XcaTreeView::storeItems(void)
@@ -275,6 +282,12 @@ void XcaTreeView::doubleClick(const QModelIndex &m)
 		basemodel->showItem(getIndex(m));
 }
 
+void XcaTreeView::editComment(void)
+{
+	if (basemodel)
+		basemodel->editComment(currentIndex());
+}
+
 void XcaTreeView::pem2clipboard(void)
 {
 	if (basemodel)
@@ -308,7 +321,7 @@ void XcaTreeView::contextMenu(QContextMenuEvent *e, QMenu *parent, int col)
 	menu->addAction(tr("Reset"), basemodel, SLOT(columnResetDefaults()));
 	if (col >= 0 && col < allHeaders.size()) {
 		curr_hd = allHeaders[col];
-		menu->addAction(tr("Remove Column"), this,SLOT(columnRemove()));
+		menu->addAction(tr("Hide Column"), this,SLOT(columnRemove()));
 		if (curr_hd->id > 0)
 			menu->addAction(tr("Details"), this,
 						SLOT(headerDetails()));
@@ -325,7 +338,7 @@ void XcaTreeView::contextMenu(QContextMenuEvent *e, QMenu *parent, int col)
 				current = v3ext;
 				break;
 			case dbheader::hd_v3ext_ns:
-				if (pki_x509::disable_netscape)
+				if (Settings["disable_netscape"])
 					continue;
 				current = v3ns;
 				break;
@@ -393,9 +406,10 @@ void XcaTreeView::showContextMenu(QContextMenuEvent *e,
 	menu->addAction(tr("Import"), this, SLOT(load()));
 	menu->addAction(tr("Paste PEM data"), mainwin, SLOT(pastePem()));
 
-	if (indexes.size() == 1)
+	if (indexes.size() == 1) {
 		menu->addAction(tr("Rename"), this, SLOT(editIdx()));
-
+		menu->addAction(tr("Properties"), this, SLOT(editComment()));
+	}
 	if (indexes.size() > 0) {
 		menu->addAction(tr("Delete"), this, SLOT(deleteItems()));
 		subExport = menu->addMenu(tr("Export"));
@@ -408,3 +422,23 @@ void XcaTreeView::showContextMenu(QContextMenuEvent *e,
 
 	contextMenu(e, menu, -1);
 }
+void XcaTreeView::keyPressEvent(QKeyEvent *event)
+{
+	switch (event->key()) {
+		case Qt::Key_Backspace:
+		case Qt::Key_Delete:
+			deleteItems();
+			return;
+		case Qt::Key_Enter:
+		case Qt::Key_Return:
+			showItems();
+			return;
+		case Qt::Key_F2:
+			editIdx();
+			return;
+		case Qt::Key_Escape:
+			clearSelection();
+			return;
+	}
+	QTreeView::keyPressEvent(event);
+}
