@@ -47,6 +47,7 @@ class XcaTreeView: public QTreeView
 	}
 	void contextMenu(QContextMenuEvent *e,
 			QMenu *parent = NULL, int sect = -1);
+	void keyPressEvent(QKeyEvent *event);
 
    public slots:
 	void showHideSections();
@@ -64,5 +65,6 @@ class XcaTreeView: public QTreeView
 	void headerDetails(void);
 	void columnRemove(void);
 	void columnsChanged(void);
+	void editComment(void);
 };
 #endif
